@@ -2,8 +2,8 @@
 // GraphBLAS/Demo/Program/tri_demo.c: count triangles
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -36,14 +36,14 @@
 // nx-by-ny grid.  Method is 0 to 3; refer to wathen.c for details.
 
 // macro used by OK(...) to free workspace if an error occurs
-#define FREE_ALL                \
-    GxB_Scalar_free (&Thunk) ;         \
-    GrB_Matrix_free (&C) ;             \
-    GrB_Matrix_free (&A) ;             \
-    GrB_Matrix_free (&L) ;             \
+#define FREE_ALL                    \
+    GxB_Scalar_free (&Thunk) ;      \
+    GrB_Matrix_free (&C) ;          \
+    GrB_Matrix_free (&A) ;          \
+    GrB_Matrix_free (&L) ;          \
     GrB_Matrix_free (&U) ;
 
-#include "demos.h"
+#include "graphblas_demos.h"
 
 int main (int argc, char **argv)
 {
@@ -53,7 +53,7 @@ int main (int argc, char **argv)
     double tic [2], r1, r2 ;
     OK (GrB_init (GrB_NONBLOCKING)) ;
     int nthreads ;
-    OK (GxB_get (GxB_NTHREADS, &nthreads)) ;
+    OK (GxB_Global_Option_get (GxB_GLOBAL_NTHREADS, &nthreads)) ;
     fprintf (stderr, "tri_demo: nthreads %d\n", nthreads) ;
     printf ("--------------------------------------------------------------\n");
 
@@ -62,17 +62,19 @@ int main (int argc, char **argv)
     //--------------------------------------------------------------------------
 
     // get_matrix reads in a boolean matrix.  It could easily be changed to
-    // read in uint32 matrix instead, but this would affect the other GraphBLAS
-    // demos.  So the time to typecast A = (uint32) C is added to the read
+    // read in int32 matrix instead, but this would affect the other GraphBLAS
+    // demos.  So the time to typecast A = (int32) C is added to the read
     // time, not the prep time for triangle counting.
     simple_tic (tic) ;
-    OK (get_matrix (&C, argc, argv, true, true)) ;
+    OK (get_matrix (&C, argc, argv, true, true, true)) ;
     GrB_Index n, nedges ;
     OK (GrB_Matrix_nrows (&n, C)) ;
 
-    // A = spones (C), and typecast to uint32
-    OK (GrB_Matrix_new (&A, GrB_UINT32, n, n)) ;
-    OK (GrB_Matrix_apply (A, NULL, NULL, GxB_ONE_UINT32, C, NULL)) ;
+    GrB_Type ctype = GrB_INT32 ;
+
+    // A = spones (C), and typecast to int32
+    OK (GrB_Matrix_new (&A, ctype, n, n)) ;
+    OK (GrB_Matrix_apply (A, NULL, NULL, GxB_ONE_INT32, C, NULL)) ;
     double t_read = simple_toc (tic) ;
     printf ("\ntotal time to read A matrix: %14.6f sec\n", t_read) ;
     GrB_Matrix_free (&C) ;
@@ -82,7 +84,7 @@ int main (int argc, char **argv)
     // U = triu (A,1)
     simple_tic (tic) ;
     OK (GxB_Scalar_setElement_INT64 (Thunk, (int64_t) 1)) ;
-    OK (GrB_Matrix_new (&U, GrB_UINT32, n, n)) ;
+    OK (GrB_Matrix_new (&U, ctype, n, n)) ;
     OK (GxB_Matrix_select (U, NULL, NULL, GxB_TRIU, A, Thunk, NULL)) ;
     OK (GrB_Matrix_nvals (&nedges, U)) ;
     printf ("\nn %.16g # edges %.16g\n", (double) n, (double) nedges) ;
@@ -91,7 +93,7 @@ int main (int argc, char **argv)
 
     // L = tril (A,-1)
     simple_tic (tic) ;
-    OK (GrB_Matrix_new (&L, GrB_UINT32, n, n)) ;
+    OK (GrB_Matrix_new (&L, ctype, n, n)) ;
     OK (GxB_Scalar_setElement_INT64 (Thunk, (int64_t) (-1))) ;
     OK (GxB_Matrix_select (L, NULL, NULL, GxB_TRIL, A, Thunk, NULL)) ;
     double t_L = simple_toc (tic) ;
@@ -109,12 +111,15 @@ int main (int argc, char **argv)
 
     printf ("\n------------------------------------- dot product method:\n") ;
 
-    int64_t ntri2 [nthreads_max+1], nt = -1 ;
+    #define NTHREADS_MAX 2048
+    nthreads_max = MIN (nthreads_max, NTHREADS_MAX) ;
+
+    int64_t ntri2 [NTHREADS_MAX+1], nt = -1 ;
     double t1 ;
 
     for (int nthreads = 1 ; nthreads <= nthreads_max ; nthreads *= 2)
     {
-        GxB_Global_Option_set (GxB_NTHREADS, nthreads) ;
+        GxB_Global_Option_set (GxB_GLOBAL_NTHREADS, nthreads) ;
 
         double t_dot [2] ;
         OK (tricount (&(ntri2 [nthreads]), 5, NULL, NULL, L, U, t_dot)) ;
@@ -127,7 +132,7 @@ int main (int argc, char **argv)
         }
         if (ntri2 [nthreads] != nt)
         {
-            printf ("error!\n") ;
+            printf ("error 5!\n") ;
             fprintf (stderr, "error!\n") ;
             exit (1) ;
         }
@@ -167,7 +172,7 @@ int main (int argc, char **argv)
 
     for (int nthreads = 1 ; nthreads <= nthreads_max ; nthreads *= 2)
     {
-        GxB_Global_Option_set (GxB_NTHREADS, nthreads) ;
+        GxB_Global_Option_set (GxB_GLOBAL_NTHREADS, nthreads) ;
 
         double t_dot [2] ;
         OK (tricount (&(ntri2 [nthreads]), 6, NULL, NULL, L, U, t_dot)) ;
@@ -176,7 +181,7 @@ int main (int argc, char **argv)
 //      fprintf (stderr, "# triangles %.16g\n", (double) ntri2 [nthreads]) ;
         if (ntri2 [nthreads] != nt)
         {
-            printf ("error!\n") ;
+            printf ("error 6!\n") ;
             fprintf (stderr, "error!\n") ;
             exit (1) ;
         }
@@ -215,24 +220,18 @@ int main (int argc, char **argv)
     //--------------------------------------------------------------------------
 
     printf ("\n----------------------------------- saxpy method:\n") ;
-    // warmup
-//    int64_t ntri ;
-//    double tt [2] ;
-//    GxB_Global_Option_set (GxB_NTHREADS, 1) ;
-//    OK (tricount (&ntri, 3, NULL, NULL, L, NULL, tt)) ;
-//    // printf ("warmup %g %g\n", tt [0], tt [1]) ;
 
-    int64_t ntri1 [nthreads_max+1] ;
+    int64_t ntri1 [NTHREADS_MAX+1] ;
 
     for (int nthreads = 1 ; nthreads <= nthreads_max ; nthreads *= 2)
     {
-        GxB_Global_Option_set (GxB_NTHREADS, nthreads) ;
+        GxB_Global_Option_set (GxB_GLOBAL_NTHREADS, nthreads) ;
 
         double t_mark [2] = { 0, 0 } ;
         OK (tricount (&ntri1 [nthreads], 3, NULL, NULL, L, NULL, t_mark)) ;
         if (ntri1 [nthreads] != nt)
         {
-            printf ("error!\n") ;
+            printf ("error 3!\n") ;
             fprintf (stderr, "error!\n") ;
             exit (1) ;
         }
