@@ -2,8 +2,8 @@
 // irowscale: scale the rows of an adjacency matrix by out-degree
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -92,7 +92,7 @@ GrB_Info irowscale          // GrB_SUCCESS or error condition
     //--------------------------------------------------------------------------
 
     OK (GrB_Vector_new (&dout, GrB_UINT64, n)) ;
-    OK (GrB_Matrix_reduce_BinaryOp (dout, NULL, NULL, GrB_PLUS_UINT64,
+    OK (GrB_Matrix_reduce_Monoid (dout, NULL, NULL, GrB_PLUS_MONOID_UINT64,
         A, NULL)) ;
 
     //--------------------------------------------------------------------------
@@ -110,7 +110,7 @@ GrB_Info irowscale          // GrB_SUCCESS or error condition
     OK (GrB_Vector_extractTuples_UINT64 (I, X, &nvals, dout)) ;
 
     // I and X exclude empty columns of A.  This condition is always true.
-    CHECK (nvals <= n, GrB_PANIC) ;
+    CHECK (nvals <= n, GrB_INVALID_VALUE) ;
 
     // D = diag (ZSCALE./dout) ;
     OK (GrB_Matrix_new (&D, GrB_UINT64, n, n)) ;
