@@ -14,8 +14,8 @@ classdef GrB
 %
 % constructs a GraphBLAS matrix G, which is the result of C<M>=A in
 % GraphBLAS notation (like C(M)=A(M) in MATLAB).  The matrices used in any
-% GrB.method may be MATLAB matrices (sparse or dense) or GraphBLAS sparse
-% matrices, in any combination.
+% GrB.method may be MATLAB matrices (sparse or full) or GraphBLAS matrices
+% (hyper, sparse, bitmap, or full, by row or column), in any combination.
 %
 % --------------------
 % The GrB constructor:
@@ -32,13 +32,15 @@ classdef GrB
 %   The m and n parameters above are MATLAB scalars.  The type and format
 %   parameters are strings.  The default format is 'by col', to match the
 %   format used in MATLAB (see also GrB.format), but many graph
-%   algorithms are faster if the format is 'by row'.
+%   algorithms are faster if the format is 'by row'.  The format can also
+%   specify the data structure to use (hypersparse, sparse, bitmap, and/or
+%   full).
 %
 %   The usage C = GrB (m, n, type) is analgous to A = sparse (m, n),
 %   which creates an empty MATLAB sparse matrix A.  The type parameter is
 %   a string, which defaults to 'double' if not present.
 %
-%   For the usage C = GrB (A, type), A is either a MATLAB sparse or dense
+%   For the usage C = GrB (A, type), A is either a MATLAB sparse or full
 %   matrix, or a GraphBLAS sparse matrix object.  C is created as a
 %   GraphBLAS sparse matrix object that contains a copy of A, typecasted
 %   to the given type if the type string does not match the type of A.
@@ -93,6 +95,10 @@ classdef GrB
 %   When a GraphBLAS matrix is converted into a MATLAB sparse or full
 %   matrix, it is always returned to MATLAB 'by col'.
 %
+%   The format can also specify the data structure to use.  By default
+%   GraphBLAS selects automatically between hypersparse, sparse, bitmap,
+%   and full formats.  See 'help GrB.format' for details.
+%
 %--------------------
 % Integer operations:
 %--------------------
@@ -178,7 +184,9 @@ classdef GrB
 %   C = bitxor (A, B, ...)          bitwise xor
 %
 %   C = cast (G, ...)       cast GrB matrix to MATLAB matrix
+%   C = cat (dim, ...)      contatenate matrices
 %   C = ceil (G)            round towards infinity
+%   C = cell2mat (A)        concatenate a cell array of matrices
 %   p = colamd (G)          column approximate minimum degree ordering
 %   C = complex (G)         cast GrB matrix to MATLAB sparse complex
 %   C = conj (G)            complex conjugate
@@ -257,12 +265,14 @@ classdef GrB
 %   [F, E] = log2 (G)       base-2 logarithm
 %   C = logical (G)         cast GrB matrix to MATLAB sparse logical
 %
+%   C = mat2cell (A,m,n)    break a matrix into a cell array of matrices
 %   C = max (A,B,option)    reduce via max, to vector or scalar
 %   C = min (A,B,option)    reduce via min, to vector or scalar
 %
 %   e = nnz (G)             number of entries in a GrB matrix G
 %   X = nonzeros (G)        extract all entries from a GrB matrix
 %   s = norm (G, kind)      norm of a GrB matrix
+%   C = num2cell (A,dim)    convert a matrix into a cell array
 %   e = numel (G)           m*n for an m-by-n GrB matrix G
 %   e = nzmax (G)           number of entries in a GrB matrix G
 %
@@ -315,12 +325,12 @@ classdef GrB
 %-------------------------------------------------------------------------
 %
 %   The Static Methods for the GrB class can be used on input matrices of
-%   any kind: GraphBLAS sparse matrices, MATLAB sparse matrices, or
-%   MATLAB dense matrices, in any combination.  The output matrix C is
-%   a GraphBLAS matrix, by default, but can be optionally returned as a
-%   MATLAB sparse or dense matrix.  The static methods divide into three
-%   categories: those that perform basic functions, graph algorithms,
-%   and the 12 foundational GraphBLAS operations.
+%   any kind: GraphBLAS sparse matrices, MATLAB sparse matrices, or MATLAB
+%   full matrices, in any combination.  The output matrix C is a GraphBLAS
+%   matrix, by default, but can be optionally returned as a MATLAB sparse
+%   or full matrix.  The static methods divide into three categories:
+%   those that perform basic functions, graph algorithms, and the 12
+%   foundational GraphBLAS operations.
 %
 %---------------------------
 % GraphBLAS basic functions:
@@ -362,6 +372,12 @@ classdef GrB
 %   C = GrB.random (...)         random GraphBLAS matrix (like 'sprand')
 %   C = GrB.speye (m,n,type)     identity matrix of any type (like 'speye')
 %   t = GrB.type (A)             get the type of a MATLAB or GrB matrix A
+%   v = GrB.version              string with SuiteSparse:GraphBLAS version
+%   v = GrB.ver                  struct with SuiteSparse:GraphBLAS version
+%
+%   load/save:
+%   C = GrB.load (filename)      load a single matrix from a file
+%   GrB.save (A, filename)       save a single matrix to a file
 %
 %-------------------------------------
 % Static Methods for graph algorithms:
@@ -547,8 +563,8 @@ classdef GrB
 %
 % See also sparse.
 %
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 properties (SetAccess = private, GetAccess = private)
     % The struct contains the entire opaque content of a GraphBLAS
@@ -590,8 +606,8 @@ methods
                 % GraphBLAS mexFunction into a GrB matrix object.
                 C.opaque = arg1 ;
             elseif (isobject (arg1))
-                % arg1 is already a GrB matrix; nothing to do
-                C = arg1 ;
+                % arg1 is already a GrB matrix; make a deep copy
+                C.opaque = gbnew (arg1.opaque) ;
             else
                 % arg1 is a MATLAB matrix; convert to a GrB matrix
                 C.opaque = gbnew (arg1) ;
@@ -698,7 +714,7 @@ methods
 
     % methods in MATLAB/elmat not implemented here:
     %
-    %       accumarray blkdiag bsxfun cat circshift compan gallery
+    %       accumarray blkdiag bsxfun circshift compan gallery
     %       hadamard hankel hilb inf invhilb ipermute isequaln nan ndgrid
     %       pascal permute repelem rot90 shiftdim toeplitz vander
     %       wilkinson
@@ -803,6 +819,7 @@ methods
     C = bitxor (A, B, assumedtype) ;
 
 %   C = cast (G, ...)       built-in works as-is
+    C = cat (dim, varargin) ;
     C = ceil (G) ;
     [p, varargout] = colamd (G, varargin) ;
     C = complex (A, B) ;
@@ -881,12 +898,14 @@ methods
     [F, E] = log2 (G) ;
     C = logical (G) ;
 
+    C = mat2cell (A, m, n) ;
     C = max (A, B, option) ;
     C = min (A, B, option) ;
 
     e = nnz (G) ;
     X = nonzeros (G) ;
     s = norm (G, kind) ;
+    C = num2cell (A, dim) ;
     s = numel (G) ;
     e = nzmax (G) ;
 
@@ -915,6 +934,7 @@ methods
     C = sprandsym (arg1, arg2) ;
     c = sprintf (varargin) ;
     C = sqrt (G) ;
+    S = struct (G) ;
     C = sum (G, option) ;
     [p, varargout] = symamd (G, varargin) ;
     p = symrcm (G) ;
@@ -963,6 +983,7 @@ methods (Static)
     binopinfo (op, type) ;
     C = build (I, J, X, m, n, dup, type, desc) ;
     b = burble (b) ;
+    C = cell2mat (A) ;
     c = chunk (c) ;
     clear ;
     [C, I, J] = compact (A, id) ;
@@ -977,7 +998,7 @@ methods (Static)
     [I, J, X] = extracttuples (A, desc) ;
     C = eye (m, n, type) ;
     finalize ;
-    f = format (arg) ;
+    [f, s] = format (arg) ;
     C = incidence (A, varargin) ;
     init ;
     s = isbyrow (A) ;
@@ -987,6 +1008,7 @@ methods (Static)
     C = kronecker (Cin, M, accum, op, A, B, desc) ;
     C = ktruss (A, k, check) ;                  % uses GrB matrices
     L = laplacian (A, type, check) ;
+    C = load (filename) ;
     iset = mis (A, check) ;                     % uses GrB matrices
     monoidinfo (monoid, type) ;
     C = mxm (Cin, M, accum, semiring, A, B, desc) ;
@@ -998,6 +1020,7 @@ methods (Static)
     C = prune (A, identity) ;
     C = random (varargin) ;
     C = reduce (cin, accum, monoid, A, desc) ;
+    filename_used = save (C, filename) ;
     C = select (Cin, M, accum, selectop, A, b, desc) ;
     selectopinfo (op) ;
     semiringinfo (s, type) ;
@@ -1008,6 +1031,8 @@ methods (Static)
     s = tricount (A, check, d) ;                % uses GrB matrices
     s = type (A) ;
     unopinfo (op, type) ;
+    v = version ;
+    v = ver ;
     C = vreduce (Cin, M, accum, monoid, A, desc) ;
 
 end
