@@ -24,8 +24,8 @@ function DiGraph = digraph (G, option)
 %
 % See also graph, digraph, GrB/graph.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 G = G.opaque ;
 
@@ -62,13 +62,13 @@ switch (type)
 
     case { 'logical' }
 
-        % The MATLAB digraph(...) function allows for sparse logical
+        % The MATLAB digraph(...) function allows for logical
         % adjacency matrices (no edge weights are created).
-        DiGraph = digraph (gbsparse (G, 'logical')) ;
+        DiGraph = digraph (gbmatlab (G, 'logical')) ;
 
     otherwise
 
         % typecast to double
-        DiGraph = digraph (gbsparse (G, 'double')) ;
+        DiGraph = digraph (gbmatlab (G, 'double')) ;
 end
 
