@@ -8,8 +8,8 @@ function C = eq (A, B)
 %
 % See also GrB/lt, GrB/le, GrB/gt, GrB/ge, GrB/ne.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 % The pattern of C depends on the type of inputs:
 % A scalar, B scalar:  C is scalar.
@@ -39,8 +39,8 @@ if (a_is_scalar)
         % A is a scalar, B is a matrix
         if (gb_scalar (A) == 0)
             % since a == 0, entries not present in B result in a true
-            % value, so the result is dense.  Expand A to a dense matrix.
-            A = gb_scalar_to_full (bm, bn, ctype, A) ;
+            % value, so the result is full.  Expand A to a full matrix.
+            A = gb_scalar_to_full (bm, bn, ctype, gb_fmt (B), A) ;
             C = GrB (gbemult (A, '==', gbfull (B, ctype))) ;
         else
             % since a ~= 0, entries not present in B result in a false
@@ -54,8 +54,8 @@ else
         % A is a matrix, B is a scalar
         if (gb_scalar (B) == 0)
             % since b == 0, entries not present in A result in a true
-            % value, so the result is dense.  Expand B to a dense matrix.
-            B = gb_scalar_to_full (am, an, ctype, B) ;
+            % value, so the result is full.  Expand B to a full matrix.
+            B = gb_scalar_to_full (am, an, ctype, gb_fmt (A), B) ;
             C = GrB (gbemult (gbfull (A, ctype), '==', B)) ;
         else
             % since b ~= 0, entries not present in A result in a false
