@@ -11,10 +11,10 @@ function C = int8 (G)
 % GrB/int16, GrB/int32, GrB/int64, GrB/uint8, GrB/uint16, GrB/uint32,
 % GrB/uint64.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 G = G.opaque ;
 desc.kind = 'full' ;
-C = gbfull (G, 'int8', int8 (0), desc) ;
+C = gbfull (G, 'int8', int8 (0), desc) ;        % export as a MATLAB full matrix
 
