@@ -1,8 +1,8 @@
 function result = gb_printf_helper (printf_function, varargin)
 %GB_PRINTF_HELPER wrapper for fprintf and sprintf
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 % convert all GraphBLAS matrices to full MATLAB matrices
 len = length (varargin) ;
@@ -11,7 +11,7 @@ for k = 2:len
     if (isobject (arg))
         arg = arg.opaque ;
         desc.kind = 'full' ;
-        varargin {k} = gbfull (arg, gbtype (arg), 0, desc) ;
+        varargin {k} = gbfull (arg, gbtype (arg), 0, desc) ;    % as MATLAB full
     end
 end
 
