@@ -2,8 +2,8 @@
 // gbdisp: display a GraphBLAS matrix struct
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
@@ -13,6 +13,8 @@
 
 #include "gb_matlab.h"
 
+#define USAGE "usage: gbdisp (C, cnz, level)"
+
 void mexFunction
 (
     int nargout,
@@ -26,7 +28,7 @@ void mexFunction
     // check inputs
     //--------------------------------------------------------------------------
 
-    gb_usage (nargin == 3 && nargout == 0, "usage: gbdisp (C, cnz, level)") ;
+    gb_usage (nargin == 3 && nargout == 0, USAGE) ;
 
     //--------------------------------------------------------------------------
     // get cnz and level
