@@ -2,8 +2,8 @@
 // gbextracttuples: extract all entries from a GraphBLAS matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
@@ -22,6 +22,8 @@
 
 #include "gb_matlab.h"
 
+#define USAGE "usage: [I,J,X] = GrB.extracttuples (A, desc)"
+
 void mexFunction
 (
     int nargout,
@@ -35,8 +37,7 @@ void mexFunction
     // check inputs
     //--------------------------------------------------------------------------
 
-    gb_usage (nargin >= 1 && nargin <= 2 && nargout <= 3,
-        "usage: [I,J,X] = GrB.extracttuples (A, desc)") ;
+    gb_usage (nargin >= 1 && nargin <= 2 && nargout <= 3, USAGE) ;
 
     //--------------------------------------------------------------------------
     // get the optional descriptor
@@ -45,10 +46,12 @@ void mexFunction
     base_enum_t base = BASE_DEFAULT ;
     kind_enum_t kind = KIND_FULL ;              // ignored
     GxB_Format_Value fmt = GxB_NO_FORMAT ;      // ignored
+    int sparsity = 0 ;                          // ignored
     GrB_Descriptor desc = NULL ;
     if (nargin > 1)
     { 
-        desc = gb_mxarray_to_descriptor (pargin [nargin-1], &kind, &fmt, &base);
+        desc = gb_mxarray_to_descriptor (pargin [nargin-1], &kind, &fmt,
+            &sparsity, &base) ;
     }
     OK (GrB_Descriptor_free (&desc)) ;
 
