@@ -2,8 +2,8 @@
 // gbsubassign: assign entries into a GraphBLAS matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2019, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
@@ -16,12 +16,14 @@
 
 // Usage:
 
-//      Cout = gbsubassign (Cin, M, accum, A, I, J, desc)
+//      C = gbsubassign (Cin, M, accum, A, I, J, desc)
 
-// Cin, A, and desc are required.  See GrB.m for more details.
+// Cin and A required.  See GrB.m for more details.
 
 #include "gb_matlab.h"
 
+#define USAGE "usage: C = GrB.subassign (Cin, M, accum, A, I, J, desc)"
+
 void mexFunction
 (
     int nargout,
@@ -30,7 +32,6 @@ void mexFunction
     const mxArray *pargin [ ]
 )
 {
-    gb_assign (nargout, pargout, nargin, pargin, true,
-        "usage: Cout = GrB.subassign (Cin, M, accum, A, I, J, desc)") ;
+    gb_assign (nargout, pargout, nargin, pargin, true, USAGE) ;
 }
 
