@@ -2,8 +2,8 @@
 // gb_get_deep: create a deep GrB_Matrix copy of a MATLAB X
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2019, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
@@ -17,9 +17,9 @@ GrB_Matrix gb_get_deep      // return a deep GrB_Matrix copy of a MATLAB X
 
     GrB_Matrix S = gb_get_shallow (X) ;
     GxB_Format_Value fmt ;
-    OK (GxB_get (S, GxB_FORMAT, &fmt)) ;
-    GrB_Matrix A = gb_typecast (NULL, fmt, S) ;
-    OK (GrB_free (&S)) ;
+    OK (GxB_Matrix_Option_get (S, GxB_FORMAT, &fmt)) ;
+    GrB_Matrix A = gb_typecast (S, NULL, fmt, 0) ;
+    OK (GrB_Matrix_free (&S)) ;
     return (A) ;
 }
 
