@@ -2,8 +2,8 @@
 // gb_string_and_type_to_binop: get a GraphBLAS operator from a string and type
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
@@ -19,6 +19,10 @@
 //  bitwise operators:
 //      bitand, bitor, bitxor, bitxnor, bitget, bitset, bitclr, bitshift
 
+// positional operators:
+//      firsti0, firsti1, firstj0, firstj1, secondi0, secondi1, secondj0,
+//      secondj1.  The default type is int64
+
 // The following synonyms are allowed for specifying these operators:
 //
 //      1st   first
@@ -43,12 +47,11 @@
 GrB_BinaryOp gb_string_and_type_to_binop    // return op from string and type
 (
     const char *op_name,        // name of the operator, as a string
-    const GrB_Type type         // type of the x,y inputs to the operator
+    const GrB_Type type,        // type of the x,y inputs to the operator
+    const bool type_not_given   // true if no type present in the string
 )
 {
 
-    CHECK_ERROR (type == NULL, "unsupported type") ;
-
     if (MATCH (op_name, "1st") || MATCH (op_name, "first"))
     { 
 
@@ -690,7 +693,82 @@ GrB_BinaryOp gb_string_and_type_to_binop    // return op from string and type
         if (type == GrB_UINT64) return (GxB_BSHIFT_UINT64) ;
 
     }
+    else if (MATCH (op_name, "firsti0" ) || MATCH (op_name, "1sti0"))
+    {
 
+        if (type == GrB_INT32) return (GxB_FIRSTI_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_FIRSTI_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "firsti1" ) || MATCH (op_name, "1sti1") ||
+             MATCH (op_name, "firsti"  ) || MATCH (op_name, "1sti"))
+    {
+
+        if (type == GrB_INT32) return (GxB_FIRSTI1_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_FIRSTI1_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "firstj0" ) || MATCH (op_name, "1stj0"))
+    {
+
+        if (type == GrB_INT32) return (GxB_FIRSTJ_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_FIRSTJ_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "firstj1" ) || MATCH (op_name, "1stj1") ||
+             MATCH (op_name, "firstj"  ) || MATCH (op_name, "1stj"))
+    {
+
+        if (type == GrB_INT32) return (GxB_FIRSTJ1_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_FIRSTJ1_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "secondi0") || MATCH (op_name, "2ndi0"))
+    {
+
+        if (type == GrB_INT32) return (GxB_SECONDI_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_SECONDI_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "secondi1") || MATCH (op_name, "2ndi1") ||
+             MATCH (op_name, "secondi" ) || MATCH (op_name, "2ndi"))
+    {
+
+        if (type == GrB_INT32) return (GxB_SECONDI1_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_SECONDI1_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "secondj0" ) || MATCH (op_name, "2ndj0"))
+    {
+
+        if (type == GrB_INT32) return (GxB_SECONDJ_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_SECONDJ_INT64 ) ;
+
+    }
+    else if (MATCH (op_name, "secondj1") || MATCH (op_name, "2ndj1") ||
+             MATCH (op_name, "secondj" ) || MATCH (op_name, "2ndj"))
+    {
+
+        if (type == GrB_INT32) return (GxB_SECONDJ1_INT32 ) ;
+        if (type == GrB_INT64
+        ||  type_not_given   ) return (GxB_SECONDJ1_INT64 ) ;
+
+    }
+
+    //--------------------------------------------------------------------------
+    // unknown type or operator
+    //--------------------------------------------------------------------------
+
+    // the type can be NULL for positional operators, but no others
+
+    CHECK_ERROR (type == NULL, "unknown type") ;
     ERROR2 ("unknown binary operator", op_name) ;
     return (NULL) ;
 }
