@@ -2,15 +2,15 @@
 // gb_string_to_unop: get a GraphBLAS unary operator from a string
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 //------------------------------------------------------------------------------
 
 #include "gb_matlab.h"
 
 // The string has the form op_name.op_type.  For example 'abs.double' is the
-// GxB_ABS_FP64 operator.  The type is optional.  If not present, it defaults
+// GrB_ABS_FP64 operator.  The type is optional.  If not present, it defaults
 // to the default_type parameter.
 
 GrB_UnaryOp gb_string_to_unop           // return unary operator from a string
@@ -39,6 +39,7 @@ GrB_UnaryOp gb_string_to_unop           // return unary operator from a string
     // get the operator type
     //--------------------------------------------------------------------------
 
+    bool type_not_given = (op_typename == NULL) ;
     GrB_Type type ;
     if (op_typename == NULL)
     { 
@@ -53,6 +54,6 @@ GrB_UnaryOp gb_string_to_unop           // return unary operator from a string
     // convert the string to a GraphBLAS unary operator, built-in or Complex
     //--------------------------------------------------------------------------
 
-    return (gb_string_and_type_to_unop (op_name, type)) ;
+    return (gb_string_and_type_to_unop (op_name, type, type_not_given)) ;
 }
 
