@@ -13,9 +13,9 @@ function C = true (varargin)
 %
 % See also GrB/zeros, GrB/ones, GrB/false.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 [m, n, ~] = gb_parse_args ('true', varargin {:}) ;
-C = GrB (gb_scalar_to_full (m, n, 'logical', true)) ;
+C = GrB (gb_scalar_to_full (m, n, 'logical', gbformat, true)) ;
 
