@@ -10,10 +10,10 @@ function C = uint16 (G)
 % See also GrB, GrB/double, GrB/complex, GrB/single, GrB/logical, GrB/int8,
 % GrB/int16, GrB/int32, GrB/int64, GrB/uint8, GrB/uint32, GrB/uint64.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 G = G.opaque ;
 desc.kind = 'full' ;
-C = gbfull (G, 'uint16', uint16 (0), desc) ;
+C = gbfull (G, 'uint16', uint16 (0), desc) ;    % export as a MATLAB full matrix
 
