@@ -2,40 +2,47 @@ function C = xor (A, B)
 %XOR logical exclusive OR.
 % C = xor (A,B) is the element-by-element logical OR of A and B.  One or
 % both may be scalars.  Otherwise, A and B must have the same size.
-% GraphBLAS and MATLAB matrices may be combined.
 %
-% See also GrB/and, GrB/or.
+% See also GrB/and, GrB/or, GrB/not.
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
-if (isscalar (A))
-    if (isscalar (B))
+if (isobject (A))
+    A = A.opaque ;
+end
+
+if (isobject (B))
+    B = B.opaque ;
+end
+
+if (gb_isscalar (A))
+    if (gb_isscalar (B))
         % A and B are scalars
-        C = GrB.emult (A, 'xor.logical', B) ;
+        C = GrB (gbemult (A, 'xor.logical', B)) ;
     else
         % A is a scalar, B is a matrix
-        if (gb_get_scalar (A) == 0)
+        if (gb_scalar (A) == 0)
             % A is false, so C is B typecasted to logical
-            C = GrB (B, 'logical') ;
+            C = GrB (gbnew (B, 'logical')) ;
         else
             % A is true, so C is a full matrix the same size as B
-            C = not (B) ;
+            C = GrB (gbapply ('~', gbfull (B, 'logical'))) ;
         end
     end
 else
-    if (isscalar (B))
+    if (gb_isscalar (B))
         % A is a matrix, B is a scalar
-        if (gb_get_scalar (B) == 0)
+        if (gb_scalar (B) == 0)
             % B is false, so C is A typecasted to logical
-            C = GrB (A, 'logical') ;
+            C = GrB (gbnew (A, 'logical')) ;
         else
             % B is true, so C is a full matrix the same size as A
-            C = not (A) ;
+            C = GrB (gbapply ('~', gbfull (A, 'logical'))) ;
         end
     else
         % both A and B are matrices.  C is the set union of A and B
-        C = GrB.eadd (A, 'xor.logical', B) ;
+        C = GrB (gbeadd (A, 'xor.logical', B)) ;
     end
 end
 
