@@ -1,8 +1,8 @@
 function gbtest15
 %GBTEST15 list all unary operators
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 types = gbtest_types ;
 ops = { 'identity', '~', '-', '1', 'minv', 'abs',  'sqrt', 'log', ...
@@ -11,13 +11,13 @@ ops = { 'identity', '~', '-', '1', 'minv', 'abs',  'sqrt', 'log', ...
     'signum', 'ceil', 'floor', 'round', 'trunc', 'pow2', ...
     'expm1', 'log10', 'log1p', 'log2', 'lgamma', 'tgamma', 'erf', ...
     'erfc', 'conj', 'creal', 'cimag', 'carg', 'isinf', 'isnan', ...
-    'isinfinite', 'frexpx', 'frexpe' } ;
+    'isinfinite', 'frexpx', 'frexpe', 'i0', 'i1', 'j0', 'j1' } ;
 
 nops = 0 ;
 for k1 = 1:length (ops)
     for k2 = 1:length (types)
         op = [ops{k1} '.' types{k2}] ;
-        fprintf ('\nop: [%s]\n', op) ;
+        fprintf ('\nop: (%s)\n', op) ;
         try
             GrB.unopinfo (op) ;
             GrB.unopinfo (ops {k1}, types {k2}) ;
@@ -31,7 +31,7 @@ fprintf ('\nhelp GrB.unopinfo:\n') ;
 GrB.unopinfo ;
 
 fprintf ('number of unary ops: %d\n', nops) ;
-assert (nops == 204) ;
+assert (nops == 212) ;
 
 fprintf ('gbtest15: all tests passed\n') ;
 
