@@ -1,8 +1,8 @@
 function gbtest71
 %GBTEST71 test GrB.selectopinfo
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights
-% Reserved. http://suitesparse.com.  See GraphBLAS/Doc/License.txt.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: GPL-3.0-or-later
 
 ops = {
     'tril'
@@ -28,12 +28,14 @@ ops = {
     '<'
     '<=' } ;
 
-for k = 1:length(ops)
+nops = length (ops) ;
+for k = 1:nops
     GrB.selectopinfo (ops {k}) ;
 end
 
 fprintf ('\n\n') ;
 GrB.selectopinfo
 
+fprintf ('# of select ops: %d\n', nops) ;
 fprintf ('gbtest71: all tests passed\n') ;
 
