@@ -2,8 +2,8 @@
 // GB_BinaryOp_compatible: check binary operator for type compatibility
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -32,30 +32,33 @@ GrB_Info GB_BinaryOp_compatible     // check for domain mismatch
     // ctype and btype may be NULL, but atype is never NULL
     ASSERT (atype != NULL) ;
     ASSERT (bcode <= GB_UDT_code) ;
+    GB_Opcode opcode = op->opcode ;
+    bool op_is_pair_or_positional = (opcode == GB_PAIR_opcode) 
+        || GB_OPCODE_IS_POSITIONAL (opcode) ;
 
     //--------------------------------------------------------------------------
     // first input A is cast into the type of op->xtype
     //--------------------------------------------------------------------------
 
-    if (op->opcode == GB_SECOND_opcode || op->opcode == GB_PAIR_opcode)
+    if (opcode == GB_SECOND_opcode || op_is_pair_or_positional)
     { 
         // first input is unused, so A is always compatible
         ;
     }
     else if (!GB_Type_compatible (atype, op->xtype))
     { 
-        return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
+        GB_ERROR (GrB_DOMAIN_MISMATCH,
             "Incompatible type for z=%s(x,y):\n"
             "first input of type [%s]\n"
             "cannot be typecast to x input of type [%s]",
-            op->name, atype->name, op->xtype->name))) ;
+            op->name, atype->name, op->xtype->name) ;
     }
 
     //--------------------------------------------------------------------------
     // second input B is cast into the type of op->ytype
     //--------------------------------------------------------------------------
 
-    if (op->opcode == GB_FIRST_opcode || op->opcode == GB_PAIR_opcode)
+    if (opcode == GB_FIRST_opcode || op_is_pair_or_positional)
     { 
         // second input is unused, so B is always compatible
         ;
@@ -65,11 +68,11 @@ GrB_Info GB_BinaryOp_compatible     // check for domain mismatch
         // B has a type
         if (!GB_Type_compatible (btype, op->ytype))
         { 
-            return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
+            GB_ERROR (GrB_DOMAIN_MISMATCH,
                 "Incompatible type for z=%s(x,y):\n"
                 "second input of type [%s]\n"
                 "cannot be typecast to y input of type [%s]",
-                op->name, btype->name, op->ytype->name))) ;
+                op->name, btype->name, op->ytype->name) ;
         }
     }
     else
@@ -77,11 +80,11 @@ GrB_Info GB_BinaryOp_compatible     // check for domain mismatch
         // B has a just a type code, not a type
         if (!GB_code_compatible (bcode, op->ytype->code))
         { 
-            return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
+            GB_ERROR (GrB_DOMAIN_MISMATCH,
                 "Incompatible type for z=%s(x,y):\n"
                 "second input of type [%s]\n"
                 "cannot be typecast to y input of type [%s]",
-                op->name, GB_code_string (bcode), op->ytype->name))) ;
+                op->name, GB_code_string (bcode), op->ytype->name) ;
         }
     }
 
@@ -89,13 +92,13 @@ GrB_Info GB_BinaryOp_compatible     // check for domain mismatch
     // result of binary operator of op->ztype is cast to C
     //--------------------------------------------------------------------------
 
-    if (ctype != NULL && !GB_Type_compatible (ctype, op->ztype))
+    if (!GB_Type_compatible (ctype, op->ztype))
     { 
-        return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
+        GB_ERROR (GrB_DOMAIN_MISMATCH,
             "Incompatible type for z=%s(x,y):\n"
             "operator output z of type [%s]\n"
             "cannot be typecast to result of type [%s]",
-            op->name, op->ztype->name, ctype->name))) ;
+            op->name, op->ztype->name, ctype->name) ;
     }
 
     return (GrB_SUCCESS) ;
