@@ -2,8 +2,8 @@
 // GB_Index_multiply:  multiply two integers and guard against overflow
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -14,7 +14,7 @@
 
 bool GB_Index_multiply      // true if ok, false if overflow
 (
-    GrB_Index *GB_RESTRICT c,  // c = a*b, or zero if overflow occurs
+    GrB_Index *restrict c,  // c = a*b, or zero if overflow occurs
     const int64_t a,
     const int64_t b
 )
@@ -34,9 +34,7 @@ bool GB_Index_multiply      // true if ok, false if overflow
         return (false) ;
     }
 
-    double da = ceil (log2 ((double) a)) ;
-    double db = ceil (log2 ((double) b)) ;
-    if (da + db > 60)
+    if (GB_CEIL_LOG2 (a) + GB_CEIL_LOG2 (b) > 60)
     { 
         // a * b may overflow
         return (false) ;
