@@ -2,8 +2,8 @@
 // GB_Matrix_check: print a GraphBLAS matrix and check if it is valid
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,10 +15,9 @@ GrB_Info GB_Matrix_check    // check a GraphBLAS matrix
     const GrB_Matrix A,     // GraphBLAS matrix to print and check
     const char *name,       // name of the matrix
     int pr,                 // print level
-    FILE *f,                // file for output
-    GB_Context Context
+    FILE *f                 // file for output
 )
 { 
-    return (GB_matvec_check (A, name, pr, f, "matrix", Context)) ;
+    return (GB_matvec_check (A, name, pr, f, "matrix")) ;
 }
 
