@@ -2,26 +2,30 @@
 // GB_Monoid_new: create a GrB_Monoid
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
-// Create a user-defined monoid with an operator, identity value, and
+// Create a monoid with an operator, (optionally) an identity value, and
 // (optionally) a terminal value.  If using a built-in operator, a duplicate
-// boolean operators is first replaced with its unique equivalent.  If the
+// boolean operator is first replaced with its unique equivalent.  If the
 // operator is built-in and corresponds to a known monoid, then the identity
-// value and terminal value provided on input are ignored, and the known values
-// are used instead.  This is to allow the use of the hard-coded functions for
-// built-in monoids.
+// value and terminal value provided on input are silently ignored, and the
+// known values are used instead.  This is to allow the use of the hard-coded
+// functions for built-in monoids.
+
+// User-defined monoids may have a NULL terminal value, which denotes that the
+// monoid does not have a terminal value.
 
 #include "GB.h"
+#include "GB_binop.h"
 
 GrB_Info GB_Monoid_new          // create a monoid
 (
     GrB_Monoid *monoid,         // handle of monoid to create
     GrB_BinaryOp op,            // binary operator of the monoid
-    const void *identity,       // identity value
+    const void *identity,       // identity value, if any
     const void *terminal,       // terminal value, if any (may be NULL)
     GB_Type_code idcode,        // identity and terminal type code
     GB_Context Context
@@ -34,8 +38,8 @@ GrB_Info GB_Monoid_new          // create a monoid
 
     GB_RETURN_IF_NULL (monoid) ;
     (*monoid) = NULL ;
+    GB_RETURN_IF_NULL (identity) ;
     GB_RETURN_IF_NULL_OR_FAULTY (op) ;
-    ASSERT (identity != NULL) ;
 
     ASSERT_BINARYOP_OK (op, "op for monoid", GB0) ;
     ASSERT (idcode <= GB_UDT_code) ;
@@ -47,72 +51,20 @@ GrB_Info GB_Monoid_new          // create a monoid
     // If the user requests the creation of a monoid based on a duplicate
     // built-in binary operator, the unique boolean operator is used instead.
     // See also GB_boolean_rename, which does this for opcodes, not operators.
-    // This is done before the operator is checked, so that any error messages
-    // reflect the renaming.
 
-    if (op == GrB_DIV_BOOL)
-    { 
-        // FIRST and DIV are the same for boolean:
-        op = GrB_FIRST_BOOL ;
-    }
-    else if (op == GxB_RDIV_BOOL)
-    { 
-        // SECOND and RDIV are the same for boolean:
-        op = GrB_SECOND_BOOL ;
-    }
-    else if (op == GrB_MIN_BOOL || op == GrB_TIMES_BOOL)
-    { 
-        // MIN, TIMES, and LAND are the same for boolean:
-        op = GrB_LAND ;
-    }
-    else if (op == GrB_MAX_BOOL || op == GrB_PLUS_BOOL)
-    { 
-        // MAX, PLUS, and OR are the same for boolean:
-        op = GrB_LOR ;
-    }
-    else if (op == GxB_ISNE_BOOL || op == GrB_NE_BOOL || op == GrB_MINUS_BOOL
-        || op == GxB_RMINUS_BOOL)
-    { 
-        // ISNE, NE, MINUS, RMINUS, and XOR are the same for boolean:
-        op = GrB_LXOR ;
-    }
-    else if (op == GxB_ISEQ_BOOL || op == GrB_LXNOR)
-    { 
-        // LXNOR, ISEQ, EQ are the same for boolean:
-        op = GrB_EQ_BOOL ;
-    }
-    else if (op == GxB_ISGT_BOOL)
-    { 
-        // ISGT, GT are the same for boolean:
-        op = GrB_GT_BOOL ;
-    }
-    else if (op == GxB_ISLT_BOOL)
-    { 
-        // ISLT, LT are the same for boolean:
-        op = GrB_LT_BOOL ;
-    }
-    else if (op == GxB_ISGE_BOOL)
-    { 
-        // ISGE, GE are the same for boolean:
-        op = GrB_GE_BOOL ;
-    }
-    else if (op == GxB_ISLE_BOOL)
-    { 
-        // ISLE, LE are the same for boolean:
-        op = GrB_LE_BOOL ;
-    }
+    op = GB_boolean_rename_op (op) ;
+    ASSERT_BINARYOP_OK (op, "revised op", GB0) ;
 
     //--------------------------------------------------------------------------
     // continue checking inputs
     //--------------------------------------------------------------------------
 
-    // check operator types; all must be identical
-    if (op->xtype != op->ztype || op->ytype != op->ztype)
+    // check operator types; all must be identical, and the operator cannot
+    // be positional
+    if (op->xtype != op->ztype || op->ytype != op->ztype ||
+        GB_OP_IS_POSITIONAL (op))
     { 
-        return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-            "All domains of monoid operator must be identical;\n"
-            "operator is: [%s] = %s ([%s],[%s])",
-            op->ztype->name, op->name, op->xtype->name, op->ytype->name))) ;
+        return (GrB_DOMAIN_MISMATCH) ;
     }
 
     // The idcode must match the monoid->op->ztype->code for built-in types,
@@ -121,15 +73,12 @@ GrB_Info GB_Monoid_new          // create a monoid
     // compared with the input op->ztype parameter.  Only the type code,
     // GB_UDT_code, can be checked to see if it matches.  In
     // that case, all that is known is that identity is a void * pointer that
-    // points to something, hopefully a scalar of the proper user-defined type.
+    // points to something; it must a scalar of the proper user-defined type.
 
     GB_Type_code zcode = op->ztype->code ;
     if (idcode != zcode)
     { 
-        return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-            "Identity type [%s]\n"
-            "must be identical to monoid operator z=%s(x,y) of type [%s]",
-            GB_code_string (idcode), op->name, op->ztype->name))) ;
+        return (GrB_DOMAIN_MISMATCH) ;
     }
 
     //--------------------------------------------------------------------------
@@ -137,51 +86,52 @@ GrB_Info GB_Monoid_new          // create a monoid
     //--------------------------------------------------------------------------
 
     // allocate the monoid
-    (*monoid) = GB_CALLOC (1, struct GB_Monoid_opaque) ;
+    size_t header_size ;
+    (*monoid) = GB_MALLOC (1, struct GB_Monoid_opaque, &header_size) ;
     if (*monoid == NULL)
     { 
         // out of memory
-        return (GB_OUT_OF_MEMORY) ;
+        return (GrB_OUT_OF_MEMORY) ;
     }
 
     // initialize the monoid
     GrB_Monoid mon = *monoid ;
     mon->magic = GB_MAGIC ;
+    mon->header_size = header_size ;
     mon->op = op ;
-    mon->builtin = false ;
     size_t zsize = op->ztype->size ;
-    mon->op_ztype_size = zsize ;
-    mon->identity = NULL ;                  // defined below
+    mon->identity = NULL ;                  // defined below (if present)
     mon->terminal = NULL ;                  // defined below (if present)
+    mon->identity_size = 0 ;
+    mon->terminal_size = 0 ;
 
     //--------------------------------------------------------------------------
-    // allocation of idenity and terminal values
+    // allocation of identity and terminal values
     //--------------------------------------------------------------------------
 
-    // allocate both the identity and terminal value
-    #define GB_ALLOC_IDENTITY_AND_TERMINAL                                  \
+    // allocate the identity value
+    #define GB_ALLOC_IDENTITY                                               \
     {                                                                       \
-        mon->identity = GB_CALLOC (zsize, GB_void) ;                        \
-        mon->terminal = GB_CALLOC (zsize, GB_void) ;                        \
-        if (mon->identity == NULL || mon->terminal == NULL)                 \
-        {                                                                   \
-            /* out of memory */                                             \
-            GB_FREE (mon->identity) ;                                       \
-            GB_FREE (mon->terminal) ;                                       \
-            GB_FREE (*monoid) ;                                             \
-            return (GB_OUT_OF_MEMORY) ;                                     \
-        }                                                                   \
-    }
-
-    // allocate just the identity, not the terminal
-    #define GB_ALLOC_JUST_IDENTITY                                          \
-    {                                                                       \
-        mon->identity = GB_CALLOC (zsize, GB_void) ;                        \
+        mon->identity = GB_MALLOC (zsize, GB_void, &(mon->identity_size)) ; \
         if (mon->identity == NULL)                                          \
         {                                                                   \
             /* out of memory */                                             \
-            GB_FREE (*monoid) ;                                             \
-            return (GB_OUT_OF_MEMORY) ;                                     \
+            GB_FREE (&(mon->terminal), mon->terminal_size) ;                \
+            GB_FREE (monoid, header_size) ;                                 \
+            return (GrB_OUT_OF_MEMORY) ;                                    \
+        }                                                                   \
+    }
+
+    // allocate the terminal value
+    #define GB_ALLOC_TERMINAL                                               \
+    {                                                                       \
+        mon->terminal = GB_MALLOC (zsize, GB_void, &(mon->terminal_size)) ; \
+        if (mon->terminal == NULL)                                          \
+        {                                                                   \
+            /* out of memory */                                             \
+            GB_FREE (&(mon->identity), mon->identity_size) ;                \
+            GB_FREE (monoid, header_size) ;                                 \
+            return (GrB_OUT_OF_MEMORY) ;                                    \
         }                                                                   \
     }
 
@@ -192,16 +142,20 @@ GrB_Info GB_Monoid_new          // create a monoid
     bool done = false ;
 
     // If the user requests the creation of a monoid based on a built-in
-    // operator that corresponds to known monoid, the identity and terminal
-    // values provided by the user to GrB_Monoid_new are ignored, since these
-    // are all handled by the same hard-coded functions as the built-in monoids
-    // based on the same operator.
+    // operator that corresponds to known built-in monoid, the identity and
+    // terminal values provided by the user to GrB_Monoid_new are ignored,
+    // since these are all handled by the same hard-coded functions as the
+    // built-in monoids based on the same operator.
+
+    // All of the monoids created in the switch statement below have non-NULL
+    // identity values; some have terminal values and some do not.
 
     // create a monoid with both an identity and a terminal value,
     // based on a built-in operator that is a known monoid
     #define GB_IT(ztype,identity_value,terminal_value)                      \
     {                                                                       \
-        GB_ALLOC_IDENTITY_AND_TERMINAL ;                                    \
+        GB_ALLOC_TERMINAL ;                                                 \
+        GB_ALLOC_IDENTITY ;                                                 \
         ztype *identity = (ztype *) mon->identity ;                         \
         ztype *terminal = (ztype *) mon->terminal ;                         \
         (*identity) = identity_value ;                                      \
@@ -214,7 +168,7 @@ GrB_Info GB_Monoid_new          // create a monoid
     // based on a built-in operator that is a known monoid
     #define GB_IN(ztype,identity_value)                                     \
     {                                                                       \
-        GB_ALLOC_JUST_IDENTITY ;                                            \
+        GB_ALLOC_IDENTITY ;                                                 \
         ztype *identity = (ztype *) mon->identity ;                         \
         (*identity) = identity_value ;                                      \
         done = true ;                                                       \
@@ -223,9 +177,10 @@ GrB_Info GB_Monoid_new          // create a monoid
 
     switch (op->opcode)
     {
-        case GB_MIN_opcode :
+        case GB_MIN_opcode : 
 
             // MIN monoid:  identity is +inf, terminal is -inf
+            // note there is no MIN monoid for complex types
             switch (zcode)
             {
                 case GB_INT8_code   : GB_IT (int8_t  , INT8_MAX  , INT8_MIN  )
@@ -243,9 +198,10 @@ GrB_Info GB_Monoid_new          // create a monoid
             }
             break ;
 
-        case GB_MAX_opcode :
+        case GB_MAX_opcode : 
 
             // MAX monoid:  identity is -inf, terminal is +inf
+            // note there is no MAX monoid for complex types
             switch (zcode)
             {
                 case GB_INT8_code   : GB_IT (int8_t  , INT8_MIN  , INT8_MAX  )
@@ -263,7 +219,7 @@ GrB_Info GB_Monoid_new          // create a monoid
             }
             break ;
 
-        case GB_PLUS_opcode :
+        case GB_PLUS_opcode : 
 
             // PLUS monoid:  identity is zero, no terminal value
             switch (zcode)
@@ -284,7 +240,7 @@ GrB_Info GB_Monoid_new          // create a monoid
             }
             break ;
 
-        case GB_TIMES_opcode :
+        case GB_TIMES_opcode : 
 
             // TIMES monoid:  identity is 1, no terminal value
             switch (zcode)
@@ -305,7 +261,7 @@ GrB_Info GB_Monoid_new          // create a monoid
             }
             break ;
 
-        case GB_ANY_opcode :
+        case GB_ANY_opcode : 
 
             // ANY monoid:  identity is anything, terminal value is anything
             switch (zcode)
@@ -321,9 +277,9 @@ GrB_Info GB_Monoid_new          // create a monoid
                 case GB_UINT64_code : GB_IT (uint64_t, 0, 0 )
                 case GB_FP32_code   : GB_IT (float   , 0, 0 )
                 case GB_FP64_code   : GB_IT (double  , 0, 0 )
-                case GB_FC32_code   :
+                case GB_FC32_code   : 
                     GB_IT (GxB_FC32_t, GxB_CMPLXF(0,0), GxB_CMPLXF(0,0))
-                case GB_FC64_code   :
+                case GB_FC64_code   : 
                     GB_IT (GxB_FC64_t, GxB_CMPLX(0,0), GxB_CMPLX(0,0))
                 default: ;
             }
@@ -364,17 +320,13 @@ GrB_Info GB_Monoid_new          // create a monoid
 
     if (!done)
     {
-        if (terminal == NULL)
+        // create the monoid identity value
+        GB_ALLOC_IDENTITY ;
+        memcpy (mon->identity, identity, zsize) ;
+        if (terminal != NULL)
         { 
-            // create a monoid with just an identity but no terminal value
-            GB_ALLOC_JUST_IDENTITY ;
-            memcpy (mon->identity, identity, zsize) ;
-        }
-        else
-        { 
-            // create a monoid with both an identity and a terminal value
-            GB_ALLOC_IDENTITY_AND_TERMINAL ;
-            memcpy (mon->identity, identity, zsize) ;
+            // create the monoid terminal value
+            GB_ALLOC_TERMINAL ;
             memcpy (mon->terminal, terminal, zsize) ;
         }
     }
