@@ -2,8 +2,8 @@
 // GB_Pending_n: return the # of pending tuples in a matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -16,12 +16,10 @@ int64_t GB_Pending_n        // return # of pending tuples in A
 {
 
     int64_t n = 0 ;
-    if (A != NULL)
-    {
-        if (A->Pending != NULL)
-        { 
-            n = A->Pending->n ;
-        }
+    if (A != NULL && A->Pending != NULL)
+    { 
+        // only sparse and hypersparse matries can have pending tuples
+        n = A->Pending->n ;
     }
     return (n) ;
 }
