@@ -2,8 +2,8 @@
 // GB_Type_new: create a new user-defined type
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -24,17 +24,20 @@ GrB_Info GB_Type_new
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GrB_Type_new (&type, sizeof (ctype))") ;
+    GB_WHERE1 ("GrB_Type_new (&type, sizeof (ctype))") ;
     GB_RETURN_IF_NULL (type) ;
     (*type) = NULL ;
 
     #if ( ! GB_HAS_VLA )
 
+        // Microsoft Visual Studio does not support variable-length arrays
+        // allocating automatically on the stack.  These arrays are used for
+        // scalar values for a given type.  If VLA is not supported,
+        // user-defined types can be no larger than GB_VLA_MAXSIZE.
+
         if (sizeof_ctype > GB_VLA_MAXSIZE)
         {
-            return (GB_ERROR (GrB_INVALID_VALUE, (GB_LOG, "user-defined types"
-                " are limited to %d bytes (ANSI C99 or later is required)",
-                GB_VLA_MAXSIZE))) ;
+            return (GrB_INVALID_VALUE) ;
         }
 
     #endif
@@ -44,26 +47,26 @@ GrB_Info GB_Type_new
     //--------------------------------------------------------------------------
 
     // allocate the type
-    (*type) = GB_CALLOC (1, struct GB_Type_opaque) ;
+    size_t header_size ;
+    (*type) = GB_MALLOC (1, struct GB_Type_opaque, &header_size) ;
     if (*type == NULL)
     { 
         // out of memory
-        return (GB_OUT_OF_MEMORY) ;
+        return (GrB_OUT_OF_MEMORY) ;
     }
 
     // initialize the type
     GrB_Type t = *type ;
     t->magic = GB_MAGIC ;
+    t->header_size = header_size ;
     t->size = GB_IMAX (sizeof_ctype, 1) ;
     t->code = GB_UDT_code ;     // user-defined type
+    t->name [0] = '\0' ;
 
     //--------------------------------------------------------------------------
     // get the name
     //--------------------------------------------------------------------------
 
-    // if no name found, a generic name is used instead
-    strncpy (t->name, "user-type", GB_LEN-1) ;
-
     char input2 [GB_LEN+1] ;
     char *p = NULL ;
 
@@ -92,6 +95,10 @@ GrB_Info GB_Type_new
         strncpy (t->name, p, GB_LEN-1) ;
     }
 
+    //--------------------------------------------------------------------------
+    // return result
+    //--------------------------------------------------------------------------
+
     return (GrB_SUCCESS) ;
 }
 
