@@ -2,8 +2,8 @@
 // GB_UnaryOp_check: check and print a unary operator
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,8 +15,7 @@ GrB_Info GB_UnaryOp_check   // check a GraphBLAS unary operator
     const GrB_UnaryOp op,   // GraphBLAS operator to print and check
     const char *name,       // name of the operator
     int pr,                 // print level
-    FILE *f,                // file for output
-    GB_Context Context
+    FILE *f                 // file for output
 )
 {
 
@@ -24,11 +23,10 @@ GrB_Info GB_UnaryOp_check   // check a GraphBLAS unary operator
     // check inputs
     //--------------------------------------------------------------------------
 
-    GBPR0 ("\n    GraphBLAS UnaryOp: %s ", GB_NAME) ;
+    GBPR0 ("\n    GraphBLAS UnaryOp: %s ", ((name != NULL) ? name : "")) ;
 
     if (op == NULL)
     { 
-        // GrB_error status not modified since this may be an optional argument
         GBPR0 ("NULL\n") ;
         return (GrB_NULL_POINTER) ;
     }
@@ -39,7 +37,8 @@ GrB_Info GB_UnaryOp_check   // check a GraphBLAS unary operator
 
     GB_CHECK_MAGIC (op, "UnaryOp") ;
 
-    if (op->opcode >= GB_USER_opcode)
+    GB_Opcode opcode = op->opcode ;
+    if (opcode >= GB_USER_opcode)
     { 
         GBPR0 ("(user-defined) ") ;
     }
@@ -50,41 +49,41 @@ GrB_Info GB_UnaryOp_check   // check a GraphBLAS unary operator
 
     GBPR0 ("z=%s(x)\n", op->name) ;
 
-    if (op->function == NULL)
+    bool op_is_positional = GB_OPCODE_IS_POSITIONAL (opcode) ;
+    bool op_is_one = (opcode == GB_ONE_opcode) ;
+
+    if (!op_is_positional && op->function == NULL)
     { 
         GBPR0 ("    function pointer is NULL\n") ;
-        return (GB_ERROR (GrB_INVALID_OBJECT, (GB_LOG,
-            "UnaryOp has a NULL function pointer: %s [%s]",
-            GB_NAME, op->name))) ;
+        return (GrB_INVALID_OBJECT) ;
     }
 
-    if (op->opcode != GB_USER_opcode)
+    if (opcode != GB_USER_opcode)
     {
-        if (op->opcode < GB_ONE_opcode || op->opcode >= GB_FIRST_opcode)
+        if (opcode < GB_ONE_opcode || opcode >= GB_FIRST_opcode)
         { 
             GBPR0 ("    invalid opcode\n") ;
-            return (GB_ERROR (GrB_INVALID_OBJECT, (GB_LOG,
-                "UnaryOp has an invalid opcode: %s [%s]",
-                GB_NAME, op->name))) ;
+            return (GrB_INVALID_OBJECT) ;
         }
     }
 
     GrB_Info info ;
 
-    info = GB_Type_check (op->ztype, "ztype", pr, f, Context) ;
+    info = GB_Type_check (op->ztype, "ztype", pr, f) ;
     if (info != GrB_SUCCESS)
     { 
         GBPR0 ("    UnaryOP has an invalid ztype\n") ;
-        return (GB_ERROR (GrB_INVALID_OBJECT, (GB_LOG,
-            "UnaryOp has an invalid ztype: %s [%s]", GB_NAME, op->name))) ;
+        return (GrB_INVALID_OBJECT) ;
     }
 
-    info = GB_Type_check (op->xtype, "xtype", pr, f, Context) ;
-    if (info != GrB_SUCCESS)
-    { 
-        GBPR0 ("    UnaryOP has an invalid xtype\n") ;
-        return (GB_ERROR (GrB_INVALID_OBJECT, (GB_LOG,
-            "UnaryOp has an invalid xtype: %s [%s]", GB_NAME, op->name))) ;
+    if (!op_is_positional && !op_is_one)
+    {
+        info = GB_Type_check (op->xtype, "xtype", pr, f) ;
+        if (info != GrB_SUCCESS)
+        { 
+            GBPR0 ("    UnaryOP has an invalid xtype\n") ;
+            return (GrB_INVALID_OBJECT) ;
+        }
     }
 
     return (GrB_SUCCESS) ;
