@@ -2,23 +2,19 @@
 // GB_add_phase0: find vectors of C to compute for C=A+B or C<M>=A+B
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
 // The eWise add of two matrices, C=A+B, C<M>=A+B, or C<!M>=A+B starts with
 // this phase, which determines which vectors of C need to be computed.
-// This phase is also used for GB_masker.
+// This phase is also used for GB_masker, and for GB_SUBASSIGN_TWO_SLICE.
 
 // On input, A and B are the two matrices being added, and M is the optional
 // mask matrix (not complemented).  The complemented mask is handed in GB_mask,
 // not here.
 
-// The A matrix can be sparse, hypersparse, slice, or hyperslice.  The B matrix
-// can only be sparse or hypersparse.  See GB_Matrix_wait, which can pass in A
-// as any of the four formats.  In this case, no mask is present.
-
 // On output, an integer (Cnvec) a boolean (Ch_to_Mh) and up to 3 arrays are
 // returned, either NULL or of size Cnvec.  Let n = A->vdim be the vector
 // dimension of A, B, M and C.
@@ -29,40 +25,42 @@
 //      the mask, or because the vector j appeared in A or B, but is empty.
 //      It is pruned at the end of GB_add_phase2.  If Ch is NULL then it is an
 //      implicit list of size n, and Ch [k] == k for all k = 0:n-1.  In this
-//      case, C will be a standard matrix, not hypersparse.  Thus, the kth
-//      vector is j = (Ch == NULL) ? k : Ch [k].
+//      case, C will be a sparse matrix, not hypersparse.  Thus, the kth
+//      vector is j = GBH (Ch, k).
 
 //      Ch is freed by GB_add if phase1 fails.  phase2 either frees it or
-//      transplants it into C.
+//      transplants it into C, if C is hypersparse.
 
 //      Ch_is_Mh:  true if the mask M is present, hypersparse, and not
 //      complemented, false otherwise.  In this case Ch is a deep copy of Mh.
-//      Only GB_add uses this option; it is not used by GB_masker (Ch_is_Mh
-//      is always false for GB_masker).  This is determined by passing in
-//      p_Ch_is_Mh as a NULL or non-NULL pointer.
+//      Only GB_add uses this option; it is not used by GB_masker or
+//      GB_SUBASSIGN_TWO_SLICE (Ch_is_Mh is always false in this case).  This
+//      is determined by passing in p_Ch_is_Mh as a NULL or non-NULL pointer.
 
-//      C_to_A:  if A is hypersparse, then C_to_A [k] = kA if the kth vector, j
-//      = (Ch == NULL) ? k : Ch [k] appears in A, as j = Ah [kA].  If j does
-//      not appear in A, then C_to_A [k] = -1.  If A is not hypersparse, then
-//      C_to_A is returned as NULL.
+//      C_to_A:  if A is hypersparse, then C_to_A [k] = kA if the kth vector,
+//      j = GBH (Ch, k) appears in A, as j = Ah [kA].  If j does not appear in
+//      A, then C_to_A [k] = -1.  If A is not hypersparse, then C_to_A is
+//      returned as NULL.
 
-//      C_to_B:  if B is hypersparse, then C_to_B [k] = kB if the kth vector, j
-//      = (Ch == NULL) ? k : Ch [k] appears in B, as j = Bh [kB].  If j does
-//      not appear in B, then C_to_B [k] = -1.  If B is not hypersparse, then
-//      C_to_B is returned as NULL.
+//      C_to_B:  if B is hypersparse, then C_to_B [k] = kB if the kth vector,
+//      j = GBH (Ch, k) appears in B, as j = Bh [kB].  If j does not appear in
+//      B, then C_to_B [k] = -1.  If B is not hypersparse, then C_to_B is
+//      returned as NULL.
 
 //      C_to_M:  if M is hypersparse, and Ch_is_Mh is false, then C_to_M [k] =
-//      kM if the kth vector, j = (Ch == NULL) ? k : Ch [k] appears in M, as j
-//      = Mh [kM].  If j does not appear in M, then C_to_M [k] = -1.  If M is
-//      not hypersparse, then C_to_M is returned as NULL.
+//      kM if the kth vector, j = GBH (Ch, k) appears in M, as j = Mh [kM].  If
+//      j does not appear in M, then C_to_M [k] = -1.  If M is not hypersparse,
+//      then C_to_M is returned as NULL.
+
+// M, A, B: any sparsity structure (hypersparse, sparse, bitmap, or full)
+// C: not present here, but its sparsity structure is finalized, via the
+// input/output parameter C_sparsity.
 
 #include "GB_add.h"
 
-#define GB_FREE_WORK        \
-{                           \
-    GB_FREE (kA_start) ;    \
-    GB_FREE (kB_start) ;    \
-    GB_FREE (kC_start) ;    \
+#define GB_FREE_WORK                \
+{                                   \
+    GB_WERK_POP (Work, int64_t) ;   \
 }
 
 //------------------------------------------------------------------------------
@@ -72,31 +70,31 @@
 static inline bool GB_allocate_result
 (
     int64_t Cnvec,
-    int64_t *GB_RESTRICT *Ch_handle,
-    int64_t *GB_RESTRICT *C_to_M_handle,
-    int64_t *GB_RESTRICT *C_to_A_handle,
-    int64_t *GB_RESTRICT *C_to_B_handle
+    int64_t *restrict *Ch_handle,        size_t *Ch_size_handle,
+    int64_t *restrict *C_to_M_handle,    size_t *C_to_M_size_handle,
+    int64_t *restrict *C_to_A_handle,    size_t *C_to_A_size_handle,
+    int64_t *restrict *C_to_B_handle,    size_t *C_to_B_size_handle
 )
 {
     bool ok = true ;
     if (Ch_handle != NULL)
     { 
-        (*Ch_handle) = GB_MALLOC (Cnvec, int64_t) ;
+        (*Ch_handle) = GB_MALLOC (Cnvec, int64_t, Ch_size_handle) ;
         ok = (*Ch_handle != NULL) ;
     }
     if (C_to_M_handle != NULL)
     { 
-        (*C_to_M_handle) = GB_MALLOC (Cnvec, int64_t) ;
+        (*C_to_M_handle) = GB_MALLOC_WERK (Cnvec, int64_t, C_to_M_size_handle) ;
         ok = ok && (*C_to_M_handle != NULL) ;
     }
     if (C_to_A_handle != NULL)
     { 
-        *C_to_A_handle = GB_MALLOC (Cnvec, int64_t) ;
+        *C_to_A_handle = GB_MALLOC_WERK (Cnvec, int64_t, C_to_A_size_handle) ;
         ok = ok && (*C_to_A_handle != NULL) ;
     }
     if (C_to_B_handle != NULL)
     { 
-        *C_to_B_handle = GB_MALLOC (Cnvec, int64_t) ;
+        *C_to_B_handle = GB_MALLOC_WERK (Cnvec, int64_t, C_to_B_size_handle) ;
         ok = ok && (*C_to_B_handle != NULL) ;
     }
 
@@ -105,19 +103,19 @@ static inline bool GB_allocate_result
         // out of memory
         if (Ch_handle != NULL)
         { 
-            GB_FREE (*Ch_handle) ;
+            GB_FREE (Ch_handle, *Ch_size_handle) ;
         }
         if (C_to_M_handle != NULL)
         { 
-            GB_FREE (*C_to_M_handle) ;
+            GB_FREE_WERK (C_to_M_handle, *C_to_M_size_handle) ;
         }
         if (C_to_A_handle != NULL)
         { 
-            GB_FREE (*C_to_A_handle) ;
+            GB_FREE_WERK (C_to_A_handle, *C_to_A_size_handle) ;
         }
         if (C_to_B_handle != NULL)
         { 
-            GB_FREE (*C_to_B_handle) ;
+            GB_FREE_WERK (C_to_B_handle, *C_to_B_size_handle) ;
         }
     }
     return (ok) ;
@@ -130,14 +128,19 @@ static inline bool GB_allocate_result
 GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
 (
     int64_t *p_Cnvec,           // # of vectors to compute in C
-    int64_t *GB_RESTRICT *Ch_handle,        // Ch: size Cnvec, or NULL
-    int64_t *GB_RESTRICT *C_to_M_handle,    // C_to_M: size Cnvec, or NULL
-    int64_t *GB_RESTRICT *C_to_A_handle,    // C_to_A: size Cnvec, or NULL
-    int64_t *GB_RESTRICT *C_to_B_handle,    // C_to_B: of size Cnvec, or NULL
+    int64_t *restrict *Ch_handle,        // Ch: size Cnvec, or NULL
+    size_t *Ch_size_handle,                 // size of Ch in bytes
+    int64_t *restrict *C_to_M_handle,    // C_to_M: size Cnvec, or NULL
+    size_t *C_to_M_size_handle,             // size of C_to_M in bytes
+    int64_t *restrict *C_to_A_handle,    // C_to_A: size Cnvec, or NULL
+    size_t *C_to_A_size_handle,             // size of C_to_A in bytes
+    int64_t *restrict *C_to_B_handle,    // C_to_B: of size Cnvec, or NULL
+    size_t *C_to_B_size_handle,             // size of C_to_A in bytes
     bool *p_Ch_is_Mh,           // if true, then Ch == Mh
+    int *C_sparsity,            // sparsity structure of C
     const GrB_Matrix M,         // optional mask, may be NULL; not complemented
-    const GrB_Matrix A,         // standard, hypersparse, slice, or hyperslice
-    const GrB_Matrix B,         // standard or hypersparse; never a slice
+    const GrB_Matrix A,         // first input matrix
+    const GrB_Matrix B,         // second input matrix
     GB_Context Context
 )
 {
@@ -146,36 +149,63 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
     // check inputs
     //--------------------------------------------------------------------------
 
+    // M, A, and B can be jumbled for this phase, but not phase1 or phase2
+
     ASSERT (p_Cnvec != NULL) ;
     ASSERT (Ch_handle != NULL) ;
     ASSERT (C_to_A_handle != NULL) ;
     ASSERT (C_to_B_handle != NULL) ;
-    ASSERT_MATRIX_OK (A, "A for add phase0", GB0) ;
-    ASSERT_MATRIX_OK (B, "B for add phase0", GB0) ;
+
     ASSERT_MATRIX_OK_OR_NULL (M, "M for add phase0", GB0) ;
+    ASSERT (!GB_ZOMBIES (M)) ;
+    ASSERT (GB_JUMBLED_OK (M)) ;        // pattern not accessed
+    ASSERT (!GB_PENDING (M)) ;
+
+    ASSERT_MATRIX_OK (A, "A for add phase0", GB0) ;
+    ASSERT (!GB_ZOMBIES (A)) ;
+    ASSERT (GB_JUMBLED_OK (B)) ;        // pattern not accessed
+    ASSERT (!GB_PENDING (A)) ;
+
+    ASSERT_MATRIX_OK (B, "B for add phase0", GB0) ;
+    ASSERT (!GB_ZOMBIES (B)) ;
+    ASSERT (GB_JUMBLED_OK (A)) ;        // pattern not accessed
+    ASSERT (!GB_PENDING (B)) ;
+
     ASSERT (A->vdim == B->vdim) ;
+    ASSERT (A->vlen == B->vlen) ;
     ASSERT (GB_IMPLIES (M != NULL, A->vdim == M->vdim)) ;
+    ASSERT (GB_IMPLIES (M != NULL, A->vlen == M->vlen)) ;
 
     //--------------------------------------------------------------------------
     // initializations
     //--------------------------------------------------------------------------
 
-    int64_t *GB_RESTRICT Ch = NULL ;
-    int64_t *GB_RESTRICT C_to_M = NULL ;
-    int64_t *GB_RESTRICT C_to_A = NULL ;
-    int64_t *GB_RESTRICT C_to_B = NULL ;
-
+    (*p_Cnvec) = 0 ;
     (*Ch_handle) = NULL ;
-    (*C_to_A_handle) = NULL ;
-    (*C_to_B_handle) = NULL ;
     if (C_to_M_handle != NULL)
     { 
         (*C_to_M_handle) = NULL ;
     }
+    (*C_to_A_handle) = NULL ;
+    (*C_to_B_handle) = NULL ;
+    if (p_Ch_is_Mh != NULL)
+    { 
+        (*p_Ch_is_Mh) = false ;
+    }
 
-    int64_t *GB_RESTRICT kA_start = NULL ;
-    int64_t *GB_RESTRICT kB_start = NULL ;
-    int64_t *GB_RESTRICT kC_start = NULL ;
+    if ((*C_sparsity) == GxB_BITMAP || (*C_sparsity) == GxB_FULL)
+    { 
+        // nothing to do in phase0 for C bitmap or full
+        (*p_Cnvec) = A->vdim ;  // not needed; to be consistent with GB_emult
+        return (GrB_SUCCESS) ;
+    }
+
+    int64_t *restrict Ch     = NULL ; size_t Ch_size = 0 ;
+    int64_t *restrict C_to_M = NULL ; size_t C_to_M_size = 0 ;
+    int64_t *restrict C_to_A = NULL ; size_t C_to_A_size = 0 ;
+    int64_t *restrict C_to_B = NULL ; size_t C_to_B_size = 0 ;
+
+    GB_WERK_DECLARE (Work, int64_t) ;
     int ntasks = 0 ;
 
     //--------------------------------------------------------------------------
@@ -193,30 +223,26 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
 
     int64_t n = A->vdim ;
     int64_t Anvec = A->nvec ;
-    bool A_is_hyper = A->is_hyper ;
-    bool A_is_slice = A->is_slice ;
-    const int64_t *GB_RESTRICT Ap = A->p ;
-    const int64_t *GB_RESTRICT Ah = (A_is_hyper) ? A->h : NULL ;
-    const int64_t A_hfirst = A->hfirst ;
-    #define GB_Ah(k) (A_is_hyper ? Ah [k] : (A_hfirst + (k)))
+    int64_t vlen = A->vlen ;
+    const int64_t *restrict Ap = A->p ;
+    const int64_t *restrict Ah = A->h ;
+    bool A_is_hyper = (Ah != NULL) ;
+    #define GB_Ah(k) (A_is_hyper ? Ah [k] : (k))
 
     int64_t Bnvec = B->nvec ;
-    const int64_t *GB_RESTRICT Bp = B->p ;
-    const int64_t *GB_RESTRICT Bh = B->h ;
-    bool B_is_hyper = B->is_hyper ;
-    ASSERT (!B->is_slice) ;
+    const int64_t *restrict Bp = B->p ;
+    const int64_t *restrict Bh = B->h ;
+    bool B_is_hyper = (Bh != NULL) ;
 
     int64_t Mnvec = 0 ;
-    const int64_t *GB_RESTRICT Mp = NULL ;
-    const int64_t *GB_RESTRICT Mh = NULL ;
-    bool M_is_hyper = false ;
+    const int64_t *restrict Mp = NULL ;
+    const int64_t *restrict Mh = NULL ;
+    bool M_is_hyper = GB_IS_HYPERSPARSE (M) ;
     if (M != NULL)
     { 
         Mnvec = M->nvec ;
         Mp = M->p ;
         Mh = M->h ;
-        M_is_hyper = M->is_hyper ;
-        ASSERT (!M->is_slice) ;
     }
 
     // For GB_add, if M is present, hypersparse, and not complemented, then C
@@ -232,24 +258,23 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
     {
 
         //----------------------------------------------------------------------
-        // C is hypersparse, with the same vectors as the hypersparse M
+        // C and M are hypersparse, with the same vectors as the hypersparse M
         //----------------------------------------------------------------------
 
-        // This step is done for GB_add only, not GB_masker.  GB_Matrix_wait is
-        // the only place where A may be a slice, and it does not use a mask.
-        // So this phase can ignore the case where A is a slice.
-
+        (*C_sparsity) = GxB_HYPERSPARSE ;
+        ASSERT (M_is_hyper) ;
         Cnvec = Mnvec ;
         nthreads = GB_nthreads (Cnvec, chunk, nthreads_max) ;
 
-        ASSERT (!A_is_slice) ;
-
-        if (!GB_allocate_result (Cnvec, &Ch, NULL,
-            (A_is_hyper) ? (&C_to_A) : NULL, (B_is_hyper) ? (&C_to_B) : NULL))
+        if (!GB_allocate_result (Cnvec,
+            &Ch,    &Ch_size,
+            NULL,   NULL,
+            (A_is_hyper) ? (&C_to_A) : NULL, &C_to_A_size,
+            (B_is_hyper) ? (&C_to_B) : NULL, &C_to_B_size))
         { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
 
         // copy Mh into Ch.  Ch is Mh so C_to_M is not needed.
@@ -267,32 +292,36 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
                 { 
                     // C_to_A [k] = kA if Ah [kA] == j and A(:,j) is non-empty
                     int64_t kA = 0, pA, pA_end ;
-                    GB_lookup (true, Ah, Ap, &kA, Anvec-1, j, &pA, &pA_end) ;
+                    GB_lookup (true, Ah, Ap, vlen, &kA, Anvec-1, j,
+                        &pA, &pA_end) ;
                     C_to_A [k] = (pA < pA_end) ? kA : -1 ;
                 }
                 if (B_is_hyper)
                 { 
                     // C_to_B [k] = kB if Bh [kB] == j and B(:,j) is non-empty
                     int64_t kB = 0, pB, pB_end ;
-                    GB_lookup (true, Bh, Bp, &kB, Bnvec-1, j, &pB, &pB_end) ;
+                    GB_lookup (true, Bh, Bp, vlen, &kB, Bnvec-1, j,
+                        &pB, &pB_end) ;
                     C_to_B [k] = (pB < pB_end) ? kB : -1 ;
                 }
             }
         }
 
     }
-    else if ((A_is_hyper || A_is_slice) && B_is_hyper)
+    else if (A_is_hyper && B_is_hyper)
     {
 
         //----------------------------------------------------------------------
-        // A is hypersparse or a hyperslice, and B is hypersparse
+        // A and B are hypersparse: C will be hypersparse
         //----------------------------------------------------------------------
 
         // Ch is the set union of Ah and Bh.  This is handled with a parallel
         // merge, since Ah and Bh are both sorted lists.
 
+        (*C_sparsity) = GxB_HYPERSPARSE ;
+
         //----------------------------------------------------------------------
-        // phase 0: create the tasks
+        // create the tasks to construct Ch
         //----------------------------------------------------------------------
 
         double work = GB_IMIN (Anvec + Bnvec, n) ;
@@ -302,15 +331,16 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         ntasks = GB_IMIN (ntasks, work) ;
 
         // allocate workspace
-        kA_start = GB_MALLOC (ntasks+1, int64_t) ;
-        kB_start = GB_MALLOC (ntasks+1, int64_t) ;
-        kC_start = GB_MALLOC (ntasks+1, int64_t) ;
-        if (kA_start == NULL || kB_start == NULL || kC_start == NULL)
-        {
+        GB_WERK_PUSH (Work, 3*(ntasks+1), int64_t) ;
+        if (Work == NULL)
+        { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
+        int64_t *restrict kA_start = Work ;
+        int64_t *restrict kB_start = Work + (ntasks+1) ;
+        int64_t *restrict kC_start = Work + (ntasks+1)*2 ;
 
         kA_start [0] = (Anvec == 0) ? -1 : 0 ;
         kB_start [0] = (Bnvec == 0) ? -1 : 0 ;
@@ -324,14 +354,14 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
             GB_slice_vector (NULL, NULL,
                 &(kA_start [taskid]), &(kB_start [taskid]),
                 0, 0, NULL,                 // Mi not present
-                0, Anvec, Ah, A_hfirst,     // Ah, explicit or implicit list
+                0, Anvec, Ah,               // Ah, explicit list
                 0, Bnvec, Bh,               // Bh, explicit list
                 n,                          // Ah and Bh have dimension n
                 target_work) ;
         }
 
         //----------------------------------------------------------------------
-        // phase 1: count the entries in the result of each task
+        // count the entries in Ch for each task
         //----------------------------------------------------------------------
 
         int taskid ;
@@ -369,29 +399,32 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         }
 
         //----------------------------------------------------------------------
-        // phase 1b: cumulative sum of entries for each task
+        // cumulative sum of entries in Ch for each task
         //----------------------------------------------------------------------
 
-        GB_cumsum (kC_start, ntasks, NULL, 1) ;
+        GB_cumsum (kC_start, ntasks, NULL, 1, NULL) ;
         Cnvec = kC_start [ntasks] ;
 
         //----------------------------------------------------------------------
-        // allocate the result
+        // allocate the result: Ch and the mappings C_to_[MAB]
         //----------------------------------------------------------------------
 
         // C will be hypersparse, so Ch is allocated.  The mask M is ignored
         // for computing Ch.  Ch is the set union of Ah and Bh.
 
-        if (!GB_allocate_result (Cnvec, &Ch,
-            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_A, &C_to_B))
+        if (!GB_allocate_result (Cnvec,
+            &Ch,    &Ch_size,
+            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_M_size,
+            &C_to_A, &C_to_A_size,
+            &C_to_B, &C_to_B_size))
         { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
 
         //----------------------------------------------------------------------
-        // phase 2: compute the result 
+        // compute the result: Ch and the mappings C_to_[AB]
         //----------------------------------------------------------------------
 
         #pragma omp parallel for num_threads(nthreads) schedule (dynamic,1)
@@ -521,24 +554,27 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         #endif
 
     }
-    else if ((A_is_hyper || A_is_slice) && !B_is_hyper)
+    else if (A_is_hyper && !B_is_hyper)
     {
 
         //----------------------------------------------------------------------
-        // A is hypersparse, B is standard
+        // A is hypersparse, B is not hypersparse
         //----------------------------------------------------------------------
 
-        // C will be standard.  Construct the C_to_A mapping.
+        // C will be sparse.  Construct the C_to_A mapping.
 
         Cnvec = n ;
         nthreads = GB_nthreads (Cnvec, chunk, nthreads_max) ;
 
-        if (!GB_allocate_result (Cnvec, NULL,
-            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_A, NULL))
+        if (!GB_allocate_result (Cnvec,
+            NULL, NULL,
+            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_M_size,
+            &C_to_A, &C_to_A_size,
+            NULL, NULL))
         { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
 
         int64_t j ;
@@ -558,24 +594,27 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         }
 
     }
-    else if (!(A_is_hyper || A_is_slice) && B_is_hyper)
+    else if (!A_is_hyper && B_is_hyper)
     {
 
         //----------------------------------------------------------------------
-        // A is standard, B is hypersparse
+        // A is not hypersparse, B is hypersparse
         //----------------------------------------------------------------------
 
-        // C will be standard.  Construct the C_to_B mapping.
+        // C will be sparse.  Construct the C_to_B mapping.
 
         Cnvec = n ;
         nthreads = GB_nthreads (Cnvec, chunk, nthreads_max) ;
 
-        if (!GB_allocate_result (Cnvec, NULL,
-            (M_is_hyper) ? (&C_to_M) : NULL, NULL, &C_to_B))
+        if (!GB_allocate_result (Cnvec,
+            NULL, NULL,
+            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_M_size,
+            NULL, NULL,
+            &C_to_B, &C_to_B_size))
         { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
 
         int64_t j ;
@@ -599,30 +638,32 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
     {
 
         //----------------------------------------------------------------------
-        // A and B are both standard
+        // A and B are both non-hypersparse
         //----------------------------------------------------------------------
 
-        // C will be standard
-
+        // C will be sparse
         Cnvec = n ;
         nthreads = GB_nthreads (Cnvec, chunk, nthreads_max) ;
 
-        if (!GB_allocate_result (Cnvec, NULL,
-            (M_is_hyper) ? (&C_to_M) : NULL, NULL, NULL))
+        if (!GB_allocate_result (Cnvec,
+            NULL, NULL,
+            (M_is_hyper) ? (&C_to_M) : NULL, &C_to_M_size,
+            NULL, NULL,
+            NULL, NULL))
         { 
             // out of memory
             GB_FREE_WORK ;
-            return (GB_OUT_OF_MEMORY) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
-
     }
 
     //--------------------------------------------------------------------------
-    // construct C_to_M if needed
+    // construct C_to_M if needed, if M is hypersparse
     //--------------------------------------------------------------------------
 
     if (C_to_M != NULL)
     {
+        ASSERT (M_is_hyper) ;
         if (Ch != NULL)
         {
             // C is hypersparse
@@ -633,13 +674,13 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
                 int64_t j = Ch [k] ;
                 // C_to_M [k] = kM if Mh [kM] == j and M(:,j) is non-empty
                 int64_t kM = 0, pM, pM_end ;
-                GB_lookup (true, Mh, Mp, &kM, Mnvec-1, j, &pM, &pM_end) ;
+                GB_lookup (true, Mh, Mp, vlen, &kM, Mnvec-1, j, &pM, &pM_end) ;
                 C_to_M [k] = (pM < pM_end) ? kM : -1 ;
             }
         }
         else
         {
-            // C is standard
+            // C is sparse
             int64_t j ;
             #pragma omp parallel for num_threads(nthreads) schedule(static)
             for (j = 0 ; j < n ; j++)
@@ -661,25 +702,27 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
     // return result
     //--------------------------------------------------------------------------
 
-    (*p_Cnvec      ) = Cnvec ;
+    (*p_Cnvec) = Cnvec ;
+    (*Ch_handle) = Ch ;             (*Ch_size_handle) = Ch_size ;
+    if (C_to_M_handle != NULL)
+    { 
+        (*C_to_M_handle) = C_to_M ; (*C_to_M_size_handle) = C_to_M_size ;
+    }
+    (*C_to_A_handle) = C_to_A ;     (*C_to_A_size_handle) = C_to_A_size ;
+    (*C_to_B_handle) = C_to_B ;     (*C_to_B_size_handle) = C_to_B_size ;
     if (p_Ch_is_Mh != NULL)
     { 
         // return Ch_is_Mh to GB_add.  For GB_masker, Ch is never Mh.
         (*p_Ch_is_Mh) = Ch_is_Mh ;
     }
-    (*Ch_handle    ) = Ch ;
-    (*C_to_A_handle) = C_to_A ;
-    (*C_to_B_handle) = C_to_B ;
-    if (C_to_M_handle != NULL)
-    { 
-        (*C_to_M_handle) = C_to_M ;
-    }
 
     //--------------------------------------------------------------------------
     // The code below describes what the output contains:
     //--------------------------------------------------------------------------
 
     #ifdef GB_DEBUG
+    // the mappings are only constructed when C is sparse or hypersparse
+    ASSERT ((*C_sparsity) == GxB_SPARSE || (*C_sparsity) == GxB_HYPERSPARSE) ;
     ASSERT (A != NULL) ;        // A and B are always present
     ASSERT (B != NULL) ;
     int64_t jlast = -1 ;
@@ -690,12 +733,14 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         int64_t j ;
         if (Ch == NULL)
         {
-            // C will be constructed as standard sparse
+            // C will be constructed as sparse
+            ASSERT ((*C_sparsity) == GxB_SPARSE) ;
             j = k ;
         }
         else
         {
             // C will be constructed as hypersparse
+            ASSERT ((*C_sparsity) == GxB_HYPERSPARSE) ;
             j = Ch [k] ;
         }
 
@@ -707,8 +752,8 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         // see if A (:,j) exists
         if (C_to_A != NULL)
         {
-            // A is hypersparse, or a slice
-            ASSERT (A->is_hyper || A->is_slice) ;
+            // A is hypersparse
+            ASSERT (A_is_hyper) ;
             int64_t kA = C_to_A [k] ;
             ASSERT (kA >= -1 && kA < A->nvec) ;
             if (kA >= 0)
@@ -719,16 +764,16 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         }
         else
         {
-            // A is in standard sparse form
+            // A is not hypersparse
             // C_to_A exists only if A is hypersparse
-            ASSERT (!(A->is_hyper || A->is_slice)) ;
+            ASSERT (!A_is_hyper) ;
         }
 
         // see if B (:,j) exists
         if (C_to_B != NULL)
         {
             // B is hypersparse
-            ASSERT (B->is_hyper) ;
+            ASSERT (B_is_hyper) ;
             int64_t kB = C_to_B [k] ;
             ASSERT (kB >= -1 && kB < B->nvec) ;
             if (kB >= 0)
@@ -739,9 +784,9 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         }
         else
         {
-            // B is in standard sparse form
+            // B is not hypersparse
             // C_to_B exists only if B is hypersparse
-            ASSERT (!B->is_hyper) ;
+            ASSERT (!B_is_hyper) ;
         }
 
         // see if M (:,j) exists
@@ -749,7 +794,7 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         {
             // Ch is the same as Mh
             ASSERT (M != NULL) ;
-            ASSERT (M->is_hyper) ;
+            ASSERT (M_is_hyper) ;
             ASSERT (Ch != NULL && M->h != NULL && Ch [k] == M->h [k]) ;
             ASSERT (C_to_M == NULL) ;
         }
@@ -757,7 +802,7 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         {
             // M is present and hypersparse
             ASSERT (M != NULL) ;
-            ASSERT (M->is_hyper) ;
+            ASSERT (M_is_hyper) ;
             int64_t kM = C_to_M [k] ;
             ASSERT (kM >= -1 && kM < M->nvec) ;
             if (kM >= 0)
@@ -768,8 +813,8 @@ GrB_Info GB_add_phase0          // find vectors in C for C=A+B or C<M>=A+B
         }
         else
         {
-            // M is not present, or in standard form
-            ASSERT (M == NULL || !(M->is_hyper)) ;
+            // M is not present, or present and sparse, bitmap or full
+            ASSERT (M == NULL || !M_is_hyper) ;
         }
     }
     #endif
