@@ -2,8 +2,8 @@
 // GB_apply: apply a unary operator; optionally transpose a matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -41,7 +41,10 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
 
     // C may be aliased with M and/or A
 
-    GB_RETURN_IF_FAULTY (accum) ;
+    struct GB_Matrix_opaque T_header ;
+    GrB_Matrix T = GB_clear_static_header (&T_header) ;
+
+    GB_RETURN_IF_FAULTY_OR_POSITIONAL (accum) ;
     ASSERT_MATRIX_OK (C, "C input for GB_apply", GB0) ;
     ASSERT_MATRIX_OK_OR_NULL (M, "M for GB_apply", GB0) ;
     ASSERT_BINARYOP_OK_OR_NULL (accum, "accum for GB_apply", GB0) ;
@@ -58,14 +61,17 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
         ASSERT_UNARYOP_OK (op1, "op1 for GB_apply", GB0) ;
         T_type = op1->ztype ;
         opcode = op1->opcode ;
-        // A must also be compatible with op1->xtype
-        if (!GB_Type_compatible (A->type, op1->xtype))
-        { 
-            return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-                "Incompatible type for z=%s(x):\n"
-                "input A of type [%s]\n"
-                "cannot be typecast to x input of type [%s]",
-                op1->name, A->type->name, op1->xtype->name))) ;
+        if (!GB_OPCODE_IS_POSITIONAL (opcode))
+        {
+            // A must also be compatible with op1->xtype
+            if (!GB_Type_compatible (A->type, op1->xtype))
+            { 
+                GB_ERROR (GrB_DOMAIN_MISMATCH,
+                    "Incompatible type for z=%s(x):\n"
+                    "input A of type [%s]\n"
+                    "cannot be typecast to x input of type [%s]",
+                    op1->name, A->type->name, op1->xtype->name) ;
+            }
         }
     }
     else if (op2 != NULL)
@@ -76,97 +82,95 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
         ASSERT_SCALAR_OK (scalar, "scalar for GB_apply", GB0) ;
         T_type = op2->ztype ;
         opcode = op2->opcode ;
-        bool op_is_first  = opcode == GB_FIRST_opcode ;
-        bool op_is_second = opcode == GB_SECOND_opcode ;
-        bool op_is_pair   = opcode == GB_PAIR_opcode ;
-        if (binop_bind1st)
+        if (!GB_OPCODE_IS_POSITIONAL (opcode))
         {
-            // C = op (scalar,A)
-            // A must be compatible with op2->ytype
-            if (!(op_is_first || op_is_pair ||
-                  GB_Type_compatible (A->type, op2->ytype)))
-            { 
-                return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-                    "Incompatible type for z=%s(x,y):\n"
-                    "input A of type [%s]\n"
-                    "cannot be typecast to y input of type [%s]",
-                    op2->name, A->type->name, op2->ytype->name))) ;
+            bool op_is_first  = opcode == GB_FIRST_opcode ;
+            bool op_is_second = opcode == GB_SECOND_opcode ;
+            bool op_is_pair   = opcode == GB_PAIR_opcode ;
+            if (binop_bind1st)
+            {
+                // C = op (scalar,A)
+                // A must be compatible with op2->ytype
+                if (!(op_is_first || op_is_pair ||
+                      GB_Type_compatible (A->type, op2->ytype)))
+                { 
+                    GB_ERROR (GrB_DOMAIN_MISMATCH,
+                        "Incompatible type for z=%s(x,y):\n"
+                        "input A of type [%s]\n"
+                        "cannot be typecast to y input of type [%s]",
+                        op2->name, A->type->name, op2->ytype->name) ;
+                }
+                // scalar must be compatible with op2->xtype
+                if (!(op_is_second || op_is_pair ||
+                      GB_Type_compatible (scalar->type, op2->xtype)))
+                { 
+                    GB_ERROR (GrB_DOMAIN_MISMATCH,
+                        "Incompatible type for z=%s(x,y):\n"
+                        "input scalar of type [%s]\n"
+                        "cannot be typecast to x input of type [%s]",
+                        op2->name, scalar->type->name, op2->xtype->name) ;
+                }
             }
-            // scalar must be compatible with op2->xtype
-            if (!(op_is_second || op_is_pair ||
-                  GB_Type_compatible (scalar->type, op2->xtype)))
-            { 
-                return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-                    "Incompatible type for z=%s(x,y):\n"
-                    "input scalar of type [%s]\n"
-                    "cannot be typecast to x input of type [%s]",
-                    op2->name, scalar->type->name, op2->xtype->name))) ;
-            }
-        }
-        else
-        {
-            // C = op (A,scalar)
-            // A must be compatible with op2->xtype
-            if (!(op_is_first || op_is_pair ||
-                  GB_Type_compatible (A->type, op2->xtype)))
-            { 
-                return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-                    "Incompatible type for z=%s(x,y):\n"
-                    "input scalar of type [%s]\n"
-                    "cannot be typecast to x input of type [%s]",
-                    op2->name, A->type->name, op2->xtype->name))) ;
-            }
-            // scalar must be compatible with op2->ytype
-            if (!(op_is_second || op_is_pair
-                  || GB_Type_compatible (scalar->type, op2->ytype)))
-            { 
-                return (GB_ERROR (GrB_DOMAIN_MISMATCH, (GB_LOG,
-                    "Incompatible type for z=%s(x,y):\n"
-                    "input A of type [%s]\n"
-                    "cannot be typecast to y input of type [%s]",
-                    op2->name, scalar->type->name, op2->ytype->name))) ;
+            else
+            {
+                // C = op (A,scalar)
+                // A must be compatible with op2->xtype
+                if (!(op_is_first || op_is_pair ||
+                      GB_Type_compatible (A->type, op2->xtype)))
+                { 
+                    GB_ERROR (GrB_DOMAIN_MISMATCH,
+                        "Incompatible type for z=%s(x,y):\n"
+                        "input scalar of type [%s]\n"
+                        "cannot be typecast to x input of type [%s]",
+                        op2->name, A->type->name, op2->xtype->name) ;
+                }
+                // scalar must be compatible with op2->ytype
+                if (!(op_is_second || op_is_pair
+                      || GB_Type_compatible (scalar->type, op2->ytype)))
+                { 
+                    GB_ERROR (GrB_DOMAIN_MISMATCH,
+                        "Incompatible type for z=%s(x,y):\n"
+                        "input A of type [%s]\n"
+                        "cannot be typecast to y input of type [%s]",
+                        op2->name, scalar->type->name, op2->ytype->name) ;
+                }
             }
         }
     }
     else
-    {
-        return (GB_ERROR (GrB_NULL_POINTER, (GB_LOG,
-            "Required argument is null: [op]"))) ;
+    { 
+        GB_ERROR (GrB_NULL_POINTER,
+            "Required argument is null: [%s]", "op") ;
     }
 
     // check domains and dimensions for C<M> = accum (C,T)
-    GrB_Info info = GB_compatible (C->type, C, M, accum, T_type, Context) ;
-    if (info != GrB_SUCCESS)
-    { 
-        return (info) ;
-    }
+    GrB_Info info ;
+    GB_OK (GB_compatible (C->type, C, M, Mask_struct, accum, T_type, Context)) ;
 
     // check the dimensions
     int64_t tnrows = (A_transpose) ? GB_NCOLS (A) : GB_NROWS (A) ;
     int64_t tncols = (A_transpose) ? GB_NROWS (A) : GB_NCOLS (A) ;
     if (GB_NROWS (C) != tnrows || GB_NCOLS (C) != tncols)
     { 
-        return (GB_ERROR (GrB_DIMENSION_MISMATCH, (GB_LOG,
+        GB_ERROR (GrB_DIMENSION_MISMATCH,
             "Dimensions not compatible:\n"
             "output is " GBd "-by-" GBd "\n"
             "input is " GBd "-by-" GBd "%s",
             GB_NROWS (C), GB_NCOLS (C),
-            tnrows, tncols, A_transpose ? " (transposed)" : ""))) ;
+            tnrows, tncols, A_transpose ? " (transposed)" : "") ;
     }
 
     // quick return if an empty mask is complemented
-    GB_RETURN_IF_QUICK_MASK (C, C_replace, M, Mask_comp) ;
+    GB_RETURN_IF_QUICK_MASK (C, C_replace, M, Mask_comp, Mask_struct) ;
 
     // delete any lingering zombies and assemble any pending tuples
-    GB_MATRIX_WAIT (M) ;
-    GB_MATRIX_WAIT (A) ;
-    GB_SCALAR_WAIT (scalar) ;
+    GB_MATRIX_WAIT_IF_PENDING_OR_ZOMBIES (A) ;      // A can be jumbled
+    GB_MATRIX_WAIT (scalar) ;
 
     if (op2 != NULL && GB_NNZ (scalar) != 1)
-    {
+    { 
         // the scalar entry must be present
-        return (GB_ERROR (GrB_INVALID_VALUE, (GB_LOG,
-            "Scalar is missing; it must contain a single entry"))) ;
+        GB_ERROR (GrB_INVALID_VALUE, "%s", "Scalar must contain an entry") ;
     }
 
     //--------------------------------------------------------------------------
@@ -175,7 +179,6 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
 
     if (op2 != NULL)
     { 
-        // GxB_print (op2, 3) ;
         // first(A,x), second(y,A), and any(...) become identity(A)
         if ((opcode == GB_ANY_opcode) ||
             (opcode == GB_FIRST_opcode  && !binop_bind1st) ||
@@ -198,7 +201,6 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
                 case GB_FC32_code    : op1 = GxB_IDENTITY_FC32   ; break ;
                 case GB_FC64_code    : op1 = GxB_IDENTITY_FC64   ; break ;
             }
-            // printf ("renamed: ") ; GxB_print (op1, 3) ;
             op2 = NULL ;
         }
         else if (opcode == GB_PAIR_opcode)
@@ -221,92 +223,75 @@ GrB_Info GB_apply                   // C<M> = accum (C, op(A)) or op(A')
                 case GB_FC32_code    : op1 = GxB_ONE_FC32   ; break ;
                 case GB_FC64_code    : op1 = GxB_ONE_FC64   ; break ;
             }
-            // printf ("renamed: ") ; GxB_print (op1, 3) ;
             op2 = NULL ;
         }
-
-#if 0
-        else
-        {
-            switch (opcode)
-            {
-                // commutative operators, no need for bind1st workers:
-                case PLUS_opcode      :
-                case TIMES_opcode     :
-                case PAIR_opcode      :
-                case ANY_opcode       :
-                case ISEQ_opcode      :
-                case ISNE_opcode      :
-                case EQ_opcode        :
-                case NE_opcode        :
-                case MIN_opcode       :
-                case MAX_opcode       :
-                case LOR_opcode       :
-                case LAND_opcode      :
-                case LXOR_opcode      :
-                case LXNOR_opcode     :
-                case HYPOT_opcode     :
-                case BOR_opcode       :
-                case BAND_opcode      :
-                case BXOR_opcode      :
-                case BXNOR_opcode     : binop_bind1st = false ;
-                default : ;
-            }
-        }
-#endif
-
     }
 
     //--------------------------------------------------------------------------
     // T = op(A) or op(A')
     //--------------------------------------------------------------------------
 
-    bool C_is_csc = C->is_csc ;
-    if (C_is_csc != A->is_csc)
+    bool T_is_csc = C->is_csc ;
+    if (T_is_csc != A->is_csc)
     { 
         // Flip the sense of A_transpose
         A_transpose = !A_transpose ;
     }
 
-    GrB_Matrix T = NULL ;
+    if (!T_is_csc)
+    {
+        // positional ops must be flipped, with i and j swapped
+        if (op1 != NULL)
+        { 
+            op1 = GB_positional_unop_ijflip (op1) ;
+            opcode = op1->opcode ;
+        }
+        else if (op2 != NULL)
+        { 
+            op2 = GB_positional_binop_ijflip (op2) ;
+            opcode = op2->opcode ;
+        }
+    }
 
     if (A_transpose)
     { 
         // T = op (A'), typecasting to op*->ztype
-        // transpose: typecast, apply an op, not in place
-        GBBURBLE ("(transpose-op) ") ;
-        info = GB_transpose (&T, T_type, C_is_csc, A,
-            op1, op2, scalar, binop_bind1st,
-            Context) ;
+        // transpose: typecast, apply an op, not in-place.
+        GBURBLE ("(transpose-op) ") ;
+        info = GB_transpose (&T, T_type, T_is_csc, A,   // T static
+            op1, op2, scalar, binop_bind1st, Context) ;
+        ASSERT (GB_JUMBLED_OK (T)) ;
+        // A positional op is applied to C after the transpose is computed,
+        // using the T_is_csc format.  The ijflip is handled
+        // above.
     }
     else if (M == NULL && accum == NULL && (C == A) && C->type == T_type)
-    { 
-        GBBURBLE ("(inplace-op) ") ;
-        // C = op (C), operating on the values in place, with no typecasting
-        // of the output of the operator with the matrix C.  Always succeeds.
+    {
+        GBURBLE ("(inplace-op) ") ;
+        // C = op (C), operating on the values in-place, with no typecasting
+        // of the output of the operator with the matrix C.
         // No work to do if the op is identity.
         // FUTURE::: also handle C += op(C), with accum.
         if (opcode != GB_IDENTITY_opcode)
         { 
+            // the output Cx is aliased with C->x in GB_apply_op.
             GB_void *Cx = (GB_void *) C->x ;
-            GB_apply_op (Cx,
-                op1, op2, scalar, binop_bind1st,
-                Cx, C->type, GB_NNZ (C), Context) ;
+            info = GB_apply_op (Cx, op1, op2,   // op1 != identity
+                scalar, binop_bind1st, C, Context) ;
         }
-        return (GrB_SUCCESS) ;
+        return (info) ;
     }
     else
     { 
         // T = op (A), pattern is a shallow copy of A, type is op*->ztype.
-        GBBURBLE ("(shallow-op) ") ;
-        info = GB_shallow_op (&T, C_is_csc,
-            op1, op2, scalar, binop_bind1st,
-            A, Context) ;
+        GBURBLE ("(shallow-op) ") ;
+        info = GB_shallow_op (T, T_is_csc,
+            op1, op2, scalar, binop_bind1st, A, Context) ;
     }
 
     if (info != GrB_SUCCESS)
     { 
-        GB_MATRIX_FREE (&T) ;
+        GB_phbix_free (T) ;
         return (info) ;
     }
 
