@@ -2,15 +2,17 @@
 // GB_assign_zombie1: delete all entries in C(:,j) for GB_assign
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
 // C(:,j)<!> = anything: GrB_Row_assign or GrB_Col_assign with an empty
 // complemented mask requires all entries in the C(:,j) vector to be deleted.
+// C must be sparse or hypersparse.
 
 #include "GB_assign.h"
+#include "GB_assign_zombie.h"
 
 void GB_assign_zombie1
 (
@@ -20,13 +22,24 @@ void GB_assign_zombie1
 )
 {
 
+    //--------------------------------------------------------------------------
+    // check inputs
+    //--------------------------------------------------------------------------
+
+    ASSERT (!GB_IS_FULL (C)) ;
+    ASSERT (!GB_IS_BITMAP (C)) ;
+    ASSERT (GB_ZOMBIES_OK (C)) ;
+    ASSERT (GB_JUMBLED_OK (C)) ;
+    ASSERT (!GB_PENDING (C)) ;
+
     //--------------------------------------------------------------------------
     // get C(:,j)
     //--------------------------------------------------------------------------
 
-    int64_t *GB_RESTRICT Ci = C->i ;
+    int64_t *restrict Ci = C->i ;
     int64_t pC_start, pC_end, pleft = 0, pright = C->nvec-1 ;
-    GB_lookup (C->is_hyper, C->h, C->p, &pleft, pright, j, &pC_start, &pC_end) ;
+    GB_lookup (C->h != NULL, C->h, C->p, C->vlen, &pleft, pright, j,
+        &pC_start, &pC_end) ;
     int64_t cjnz = pC_end - pC_start ;
     int64_t nzombies = C->nzombies ;
 
