@@ -2,8 +2,8 @@
 // GB_dense_ewise3_accum: C += A+B where all 3 matries are dense
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -17,7 +17,7 @@ void GB_dense_ewise3_accum          // C += A+B, all matrices dense
     GrB_Matrix C,                   // input/output matrix
     const GrB_Matrix A,
     const GrB_Matrix B,
-    const GrB_BinaryOp op,
+    const GrB_BinaryOp op,          // only GB_BINOP_SUBSET operators supported
     GB_Context Context
 )
 {
@@ -27,13 +27,29 @@ void GB_dense_ewise3_accum          // C += A+B, all matrices dense
     //--------------------------------------------------------------------------
 
     ASSERT_MATRIX_OK (C, "C for dense C+=A+B", GB0) ;
-    ASSERT (!GB_PENDING (C)) ; ASSERT (!GB_ZOMBIES (C)) ;
-    ASSERT (!GB_PENDING (A)) ; ASSERT (!GB_ZOMBIES (A)) ;
-    ASSERT (!GB_PENDING (B)) ; ASSERT (!GB_ZOMBIES (B)) ;
+    ASSERT (!GB_ZOMBIES (C)) ;
+    ASSERT (!GB_JUMBLED (C)) ;
+    ASSERT (!GB_PENDING (C)) ;
     ASSERT (GB_is_dense (C)) ;
+
+    ASSERT_MATRIX_OK (A, "A for dense C+=A+B", GB0) ;
+    ASSERT (!GB_ZOMBIES (A)) ;
+    ASSERT (!GB_JUMBLED (A)) ;
+    ASSERT (!GB_PENDING (A)) ;
     ASSERT (GB_is_dense (A)) ;
+
+    ASSERT_MATRIX_OK (B, "B for dense C+=A+B", GB0) ;
+    ASSERT (!GB_ZOMBIES (B)) ;
+    ASSERT (!GB_JUMBLED (B)) ;
+    ASSERT (!GB_PENDING (B)) ;
     ASSERT (GB_is_dense (B)) ;
+
+    ASSERT (!GB_IS_BITMAP (C)) ;
+    ASSERT (!GB_IS_BITMAP (A)) ;
+    ASSERT (!GB_IS_BITMAP (B)) ;
+
     ASSERT_BINARYOP_OK (op, "op for dense C+=A+B", GB0) ;
+    ASSERT (!GB_OP_IS_POSITIONAL (op)) ;
     ASSERT (op->ztype == C->type) ;
     ASSERT (op->ztype == A->type) ;
     ASSERT (op->ztype == B->type) ;
@@ -42,6 +58,8 @@ void GB_dense_ewise3_accum          // C += A+B, all matrices dense
     ASSERT (op->opcode >= GB_MIN_opcode) ;
     ASSERT (op->opcode <= GB_RDIV_opcode) ;
 
+    GB_ENSURE_FULL (C) ;        // convert C to full
+
     // FUTURE::: handle IS*, LOR, LAND, LXOR operators
 
     //--------------------------------------------------------------------------
@@ -58,7 +76,7 @@ void GB_dense_ewise3_accum          // C += A+B, all matrices dense
     //--------------------------------------------------------------------------
 
     #define GB_Cdense_ewise3_accum(op,xname) \
-        GB_Cdense_ewise3_accum_ ## op ## xname
+        GB (_Cdense_ewise3_accum_ ## op ## xname)
 
     #define GB_BINOP_WORKER(op,xname)                                       \
     {                                                                       \
