@@ -2,8 +2,8 @@
 // GB_dense_subassign_06d: C(:,:)<A> = A; C is dense, and M and A are aliased
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -11,23 +11,35 @@
 
 // M:           present
 // Mask_comp:   false
+// Mask_struct: true or false (both cases handled)
 // C_replace:   false
 // accum:       NULL
 // A:           matrix, and aliased to M
 // S:           none
 
+// C must be a packed matrix.  No entries are deleted and thus no zombies are
+// introduced into C.  C can be hypersparse, sparse, bitmap, or full, and its
+// sparsity structure does not change.  If C is hypersparse, sparse, or full,
+// then the pattern does not change (all entries are present, and this does not
+// change), and these cases can all be treated the same (as if full).  If C is
+// bitmap, new entries can be inserted into the bitmap C->b.
+
+// TODO the caller checks GB_as_if_full (C), which is more restrictive than
+// what this function tolerates (GB_is_packed (C)).
+
+// C and A can have any sparsity structure.
+
 #include "GB_subassign_methods.h"
 #include "GB_dense.h"
 #ifndef GBCOMPACT
 #include "GB_type__include.h"
 #endif
 
-#undef  GB_FREE_WORK
-#define GB_FREE_WORK \
-    GB_ek_slice_free (&pstart_slice, &kfirst_slice, &klast_slice) ;
-
 #undef  GB_FREE_ALL
-#define GB_FREE_ALL GB_FREE_WORK
+#define GB_FREE_ALL                         \
+{                                           \
+    GB_WERK_POP (A_ek_slicing, int64_t) ;   \
+}
 
 GrB_Info GB_dense_subassign_06d
 (
@@ -44,12 +56,24 @@ GrB_Info GB_dense_subassign_06d
     //--------------------------------------------------------------------------
 
     GrB_Info info ;
-    ASSERT (GB_is_dense (C)) ;
-    ASSERT (!GB_PENDING (C)) ; ASSERT (!GB_ZOMBIES (C)) ;
-    ASSERT (!GB_PENDING (A)) ; ASSERT (!GB_ZOMBIES (A)) ;
+    GB_WERK_DECLARE (A_ek_slicing, int64_t) ;
+
     ASSERT_MATRIX_OK (C, "C for subassign method_06d", GB0) ;
+    ASSERT (!GB_ZOMBIES (C)) ;
+    ASSERT (!GB_JUMBLED (C)) ;
+    ASSERT (!GB_PENDING (C)) ;
+    ASSERT (GB_is_packed (C)) ;
+    ASSERT (!GB_aliased (C, A)) ;   // NO ALIAS of C==A
+
     ASSERT_MATRIX_OK (A, "A for subassign method_06d", GB0) ;
+    ASSERT (!GB_ZOMBIES (A)) ;
+    ASSERT (GB_JUMBLED_OK (A)) ;
+    ASSERT (!GB_PENDING (A)) ;
+
     const GB_Type_code ccode = C->type->code ;
+    const bool C_is_bitmap = GB_IS_BITMAP (C) ;
+    const bool A_is_bitmap = GB_IS_BITMAP (A) ;
+    const bool A_is_dense = GB_as_if_full (A) ;
 
     //--------------------------------------------------------------------------
     // Method 06d: C(:,:)<A> = A ; no S; C is dense, M and A are aliased
@@ -63,24 +87,22 @@ GrB_Info GB_dense_subassign_06d
     //--------------------------------------------------------------------------
 
     GB_GET_NTHREADS_MAX (nthreads_max, chunk, Context) ;
-    int nthreads = GB_nthreads (GB_NNZ (A) + A->nvec, chunk, nthreads_max) ;
-    int ntasks = (nthreads == 1) ? 1 : (8 * nthreads) ;
-    ntasks = GB_IMIN (ntasks, GB_NNZ (A)) ;
-    ntasks = GB_IMAX (ntasks, 1) ;
 
     //--------------------------------------------------------------------------
     // slice the entries for each task
     //--------------------------------------------------------------------------
 
-    // Task tid does entries pstart_slice [tid] to pstart_slice [tid+1]-1 and
-    // vectors kfirst_slice [tid] to klast_slice [tid].  The first and last
-    // vectors may be shared with prior slices and subsequent slices.
-
-    int64_t *pstart_slice = NULL, *kfirst_slice = NULL, *klast_slice = NULL ;
-    if (!GB_ek_slice (&pstart_slice, &kfirst_slice, &klast_slice, A, ntasks))
+    int A_ntasks, A_nthreads ;
+    if (A_is_bitmap || A_is_dense)
     { 
-        // out of memory
-        return (GB_OUT_OF_MEMORY) ;
+        // no need to construct tasks
+        int64_t anz = GB_NNZ_HELD (A) ;
+        A_nthreads = GB_nthreads ((anz + A->nvec), 32*chunk, nthreads_max) ;
+        A_ntasks = (A_nthreads == 1) ? 1 : (8 * A_nthreads) ;
+    }
+    else
+    {
+        GB_SLICE_MATRIX (A, 8, 32*chunk) ;
     }
 
     //--------------------------------------------------------------------------
@@ -95,12 +117,12 @@ GrB_Info GB_dense_subassign_06d
         // define the worker for the switch factory
         //----------------------------------------------------------------------
 
-        #define GB_Cdense_06d(cname) GB_Cdense_06d_ ## cname
+        #define GB_Cdense_06d(cname) GB (_Cdense_06d_ ## cname)
 
         #define GB_WORKER(cname)                                              \
         {                                                                     \
             info = GB_Cdense_06d(cname) (C, A, Mask_struct,                   \
-                kfirst_slice, klast_slice, pstart_slice, ntasks, nthreads) ;  \
+                A_ek_slicing, A_ntasks, A_nthreads) ;                         \
             done = (info != GrB_NO_VALUE) ;                                   \
         }                                                                     \
         break ;
@@ -144,7 +166,7 @@ GrB_Info GB_dense_subassign_06d
         // get operators, functions, workspace, contents of A and C
         //----------------------------------------------------------------------
 
-        GB_BURBLE_MATRIX (A, "generic ") ;
+        GB_BURBLE_MATRIX (A, "(generic C(:,:)<Z>=Z assign) ") ;
 
         const size_t csize = C->type->size ;
         const size_t asize = A->type->size ;
@@ -163,6 +185,8 @@ GrB_Info GB_dense_subassign_06d
 
         // no vectorization
         #define GB_PRAGMA_SIMD_VECTORIZE ;
+        #undef  GB_PRAGMA_SIMD_REDUCTION
+        #define GB_PRAGMA_SIMD_REDUCTION(op,s) ;
 
         #include "GB_dense_subassign_06d_template.c"
     }
@@ -171,7 +195,7 @@ GrB_Info GB_dense_subassign_06d
     // free workspace and return result
     //--------------------------------------------------------------------------
 
-    GB_FREE_WORK ;
+    GB_FREE_ALL ;
     ASSERT_MATRIX_OK (C, "C output for subassign method_06d", GB0) ;
     return (GrB_SUCCESS) ;
 }
