@@ -2,12 +2,12 @@
 // GB_dense_subassign_25: C(:,:)<M,s> = A; C empty, A dense, M structural
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
-// Method 25: C(:,:)<M,s> = A ; C is empty, M structure, A dense
+// Method 25: C(:,:)<M,s> = A ; C is empty, M structural, A dense
 
 // M:           present
 // Mask_comp:   false
@@ -17,18 +17,24 @@
 // A:           matrix
 // S:           none
 
+// C and M are sparse or hypersparse.
+// A can have any sparsity structure, even bitmap.  M may be jumbled.
+// If so, C is constructed as jumbled.  C is reconstructed with the same
+// structure as M and can have any sparsity structure on input.  The only
+// constraint is nnz(C) is zero on input.  A must be dense with no pending
+// work, or bitmap.
+
 #include "GB_subassign_methods.h"
 #include "GB_dense.h"
 #ifndef GBCOMPACT
 #include "GB_type__include.h"
 #endif
 
-#undef  GB_FREE_WORK
-#define GB_FREE_WORK \
-    GB_ek_slice_free (&pstart_slice, &kfirst_slice, &klast_slice) ;
-
 #undef  GB_FREE_ALL
-#define GB_FREE_ALL GB_FREE_WORK
+#define GB_FREE_ALL                         \
+{                                           \
+    GB_WERK_POP (M_ek_slicing, int64_t) ;   \
+}
 
 GrB_Info GB_dense_subassign_25
 (
@@ -40,18 +46,33 @@ GrB_Info GB_dense_subassign_25
 )
 {
 
+    //--------------------------------------------------------------------------
+    // check inputs
+    //--------------------------------------------------------------------------
+
+    ASSERT (!GB_IS_BITMAP (M)) ; ASSERT (!GB_IS_FULL (M)) ;
+    ASSERT (!GB_aliased (C, M)) ;   // NO ALIAS of C==M
+    ASSERT (!GB_aliased (C, A)) ;   // NO ALIAS of C==A
+
     //--------------------------------------------------------------------------
     // get inputs
     //--------------------------------------------------------------------------
 
     GrB_Info info ;
     ASSERT_MATRIX_OK (C, "C for subassign method_25", GB0) ;
-    ASSERT_MATRIX_OK (M, "M for subassign method_25", GB0) ;
-    ASSERT_MATRIX_OK (A, "A for subassign method_25", GB0) ;
     ASSERT (GB_NNZ (C) == 0) ;
-    ASSERT (!GB_PENDING (C)) ; ASSERT (!GB_ZOMBIES (C)) ;
-    ASSERT (!GB_PENDING (M)) ; ASSERT (!GB_ZOMBIES (M)) ;
-    ASSERT (!GB_PENDING (A)) ; ASSERT (!GB_ZOMBIES (A)) ;
+    ASSERT (!GB_ZOMBIES (C)) ;
+    ASSERT (!GB_JUMBLED (C)) ;
+    ASSERT (!GB_PENDING (C)) ;
+
+    ASSERT_MATRIX_OK (M, "M for subassign method_25", GB0) ;
+    ASSERT (!GB_ZOMBIES (M)) ;
+    ASSERT (GB_JUMBLED_OK (M)) ;
+    ASSERT (!GB_PENDING (M)) ;
+
+    ASSERT_MATRIX_OK (A, "A for subassign method_25", GB0) ;
+    ASSERT (GB_as_if_full (A) || GB_IS_BITMAP (A)) ;
+
     const GB_Type_code ccode = C->type->code ;
 
     //--------------------------------------------------------------------------
@@ -66,26 +87,14 @@ GrB_Info GB_dense_subassign_25
     //--------------------------------------------------------------------------
 
     GB_GET_NTHREADS_MAX (nthreads_max, chunk, Context) ;
-    int64_t mnz = GB_NNZ (M) ;
-    int nthreads = GB_nthreads (mnz + M->nvec, chunk, nthreads_max) ;
-    int ntasks = (nthreads == 1) ? 1 : (8 * nthreads) ;
-    ntasks = GB_IMIN (ntasks, mnz) ;
-    ntasks = GB_IMAX (ntasks, 1) ;
 
     //--------------------------------------------------------------------------
     // slice the entries for each task
     //--------------------------------------------------------------------------
 
-    // Task tid does entries pstart_slice [tid] to pstart_slice [tid+1]-1 and
-    // vectors kfirst_slice [tid] to klast_slice [tid].  The first and last
-    // vectors may be shared with prior slices and subsequent slices.
-
-    int64_t *pstart_slice = NULL, *kfirst_slice = NULL, *klast_slice = NULL ;
-    if (!GB_ek_slice (&pstart_slice, &kfirst_slice, &klast_slice, M, ntasks))
-    { 
-        // out of memory
-        return (GB_OUT_OF_MEMORY) ;
-    }
+    GB_WERK_DECLARE (M_ek_slicing, int64_t) ;
+    int M_nthreads, M_ntasks ;
+    GB_SLICE_MATRIX (M, 8, chunk) ;
 
     //--------------------------------------------------------------------------
     // allocate C and create its pattern
@@ -96,8 +105,8 @@ GrB_Info GB_dense_subassign_25
     // initialize them.
 
     bool C_is_csc = C->is_csc ;
-    GB_PHIX_FREE (C) ;
-    GB_OK (GB_dup2 (&C, M, false, C->type, Context)) ;
+    GB_phbix_free (C) ;
+    GB_OK (GB_dup2 (&C, M, false, C->type, Context)) ;  // reuse old header
     C->is_csc = C_is_csc ;
 
     //--------------------------------------------------------------------------
@@ -112,12 +121,12 @@ GrB_Info GB_dense_subassign_25
         // define the worker for the switch factory
         //----------------------------------------------------------------------
 
-        #define GB_Cdense_25(cname) GB_Cdense_25_ ## cname
+        #define GB_Cdense_25(cname) GB (_Cdense_25_ ## cname)
 
         #define GB_WORKER(cname)                                              \
         {                                                                     \
             info = GB_Cdense_25(cname) (C, M, A,                              \
-                kfirst_slice, klast_slice, pstart_slice, ntasks, nthreads) ;  \
+                M_ek_slicing, M_ntasks, M_nthreads) ;                         \
             done = (info != GrB_NO_VALUE) ;                                   \
         }                                                                     \
         break ;
@@ -161,7 +170,7 @@ GrB_Info GB_dense_subassign_25
         // get operators, functions, workspace, contents of A and C
         //----------------------------------------------------------------------
 
-        GB_BURBLE_MATRIX (A, "generic ") ;
+        GB_BURBLE_MATRIX (A, "(generic C(:,:)<M,struct>=A assign, method 25) ");
 
         const size_t csize = C->type->size ;
         const size_t asize = A->type->size ;
@@ -185,8 +194,11 @@ GrB_Info GB_dense_subassign_25
     // free workspace and return result
     //--------------------------------------------------------------------------
 
-    GB_FREE_WORK ;
+    GB_FREE_ALL ;
     ASSERT_MATRIX_OK (C, "C output for subassign method_25", GB0) ;
+    ASSERT (GB_ZOMBIES_OK (C)) ;
+    ASSERT (GB_JUMBLED_OK (C)) ;
+    ASSERT (!GB_PENDING (C)) ;
     return (GrB_SUCCESS) ;
 }
 
