@@ -2,8 +2,8 @@
 // GB_entry_check: print a single entry for a built-in type
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,8 +15,7 @@ GrB_Info GB_entry_check     // print a single value
     const GrB_Type type,    // type of value to print
     const void *x,          // value to print
     int pr,                 // print level
-    FILE *f,                // file to print to
-    GB_Context Context
+    FILE *f                 // file to print to
 )
 { 
 
@@ -31,6 +30,6 @@ GrB_Info GB_entry_check     // print a single value
     // print the value
     //--------------------------------------------------------------------------
 
-    return (GB_code_check (type->code, x, pr, f, Context)) ;
+    return (GB_code_check (type->code, x, pr, f)) ;
 }
 
