@@ -2,8 +2,8 @@
 // GB_extract: C<M> = accum(C,A(I,J))
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -52,7 +52,7 @@ GrB_Info GB_extract                 // C<M> = accum (C, A(I,J))
     GrB_Info info ;
     GB_RETURN_IF_NULL (Rows) ;
     GB_RETURN_IF_NULL (Cols) ;
-    GB_RETURN_IF_FAULTY (accum) ;
+    GB_RETURN_IF_FAULTY_OR_POSITIONAL (accum) ;
 
     ASSERT_MATRIX_OK (C, "C input for GB_Matrix_extract", GB0) ;
     ASSERT_MATRIX_OK_OR_NULL (M, "M for GB_Matrix_extract", GB0) ;
@@ -60,7 +60,8 @@ GrB_Info GB_extract                 // C<M> = accum (C, A(I,J))
     ASSERT_MATRIX_OK (A, "A input for GB_Matrix_extract", GB0) ;
 
     // check domains and dimensions for C<M> = accum (C,T)
-    GB_OK (GB_compatible (C->type, C, M, accum, A->type, Context)) ;
+    GB_OK (GB_compatible (C->type, C, M, Mask_struct, accum, A->type,
+        Context)) ;
 
     // check the dimensions of C
     int64_t cnrows = GB_NROWS (C) ;
@@ -84,19 +85,23 @@ GrB_Info GB_extract                 // C<M> = accum (C, A(I,J))
 
     if (cnrows != nRows || cncols != nCols)
     { 
-        return (GB_ERROR (GrB_DIMENSION_MISMATCH, (GB_LOG,
+        GB_ERROR (GrB_DIMENSION_MISMATCH,
             "Dimensions not compatible:\n"
             "required size of output is " GBd "-by-" GBd "\n"
             "but actual size output is  " GBd "-by-" GBd "\n",
-            nRows, nCols, cnrows, cncols))) ;
+            nRows, nCols, cnrows, cncols) ;
     }
 
     // quick return if an empty mask is complemented
-    GB_RETURN_IF_QUICK_MASK (C, C_replace, M, Mask_comp) ;
+    GB_RETURN_IF_QUICK_MASK (C, C_replace, M, Mask_comp, Mask_struct) ;
 
     // delete any lingering zombies and assemble any pending tuples
-    GB_MATRIX_WAIT (M) ;
-    GB_MATRIX_WAIT (A) ;
+    GB_MATRIX_WAIT (M) ;        // cannot be jumbled
+    GB_MATRIX_WAIT (A) ;        // cannot be jumbled
+
+    GB_BURBLE_DENSE (C, "(C %s) ") ;
+    GB_BURBLE_DENSE (M, "(M %s) ") ;
+    GB_BURBLE_DENSE (A, "(A %s) ") ;
 
     //--------------------------------------------------------------------------
     // handle the CSR/CSC format and transpose; T = A (I,J) or T = A (J,I)
@@ -151,33 +156,15 @@ GrB_Info GB_extract                 // C<M> = accum (C, A(I,J))
 
     // T has the same hypersparsity as A.
 
-    // If T and C have different CRS/CSC formats, then GB_accum_mask must
-    // transpose T, and thus T can be returned from GB_subref with
-    // jumbled indices.  If T and C have the same CSR/CSC formats, then
-    // GB_subref must return T with sorted indices in each vector
-    // because GB_accum_mask will not transpose T.
-
-    // If T is a single column or a single row, it must be sorted, because the
-    // row/column transpose methods in GB_transpose do not do the sort.
-
-    bool must_sort = (T_is_csc == C->is_csc) || (cnrows == 1) || (cncols == 1) ;
-
     //--------------------------------------------------------------------------
     // T = A (I,J)
     //--------------------------------------------------------------------------
 
-    GrB_Matrix T ;
-    GB_OK (GB_subref (&T, T_is_csc, A, I, ni, J, nj, false, must_sort,
-        Context)) ;
-
-    if (must_sort)
-    { 
-        ASSERT_MATRIX_OK (T, "T extracted", GB0) ;
-    }
-    else
-    { 
-        ASSERT_MATRIX_OK_OR_JUMBLED (T, "T extracted (jumbled OK)", GB0) ;
-    }
+    struct GB_Matrix_opaque T_header ;
+    GrB_Matrix T = GB_clear_static_header (&T_header) ;
+    GB_OK (GB_subref (T, T_is_csc, A, I, ni, J, nj, false, Context)) ;
+    ASSERT_MATRIX_OK (T, "T extracted", GB0) ;
+    ASSERT (GB_JUMBLED_OK (T)) ;
 
     //--------------------------------------------------------------------------
     // C<M> = accum (C,T): accumulate the results into C via the mask M
