@@ -2,8 +2,8 @@
 // GB_hyper_realloc: reallocate a matrix hyperlist
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -25,26 +25,28 @@ GrB_Info GB_hyper_realloc
     //--------------------------------------------------------------------------
 
     ASSERT (A != NULL) ;
-    ASSERT (!A->p_shallow) ;
-    ASSERT (!A->h_shallow) ;
-    ASSERT (A->p != NULL) ;
+    ASSERT (GB_ZOMBIES_OK (A)) ;        // pattern not accessed
+    ASSERT (GB_JUMBLED_OK (A)) ;
+    ASSERT (GB_PENDING_OK (A)) ;
 
     //--------------------------------------------------------------------------
     // reallocate the hyperlist
     //--------------------------------------------------------------------------
 
-    if (A->is_hyper)
+    if (GB_IS_HYPERSPARSE (A))
     {
-
-        ASSERT (A->h != NULL) ;
+        ASSERT (!A->p_shallow) ;
+        ASSERT (!A->h_shallow) ;
 
         // old size of A->p and A->h
         int64_t plen_old = A->plen ;
 
         // change the size of A->h and A->p
         bool ok1 = true, ok2 = true ;
-        A->p = GB_REALLOC (A->p, plen_new+1, plen_old+1, int64_t, &ok1) ;
-        A->h = GB_REALLOC (A->h, plen_new,   plen_old,   int64_t, &ok2) ;
+        GB_REALLOC (A->p, plen_new+1, plen_old+1, int64_t, &(A->p_size), &ok1,
+            Context) ;
+        GB_REALLOC (A->h, plen_new,   plen_old,   int64_t, &(A->h_size), &ok2,
+            Context) ;
         bool ok = ok1 && ok2 ;
 
         // always succeeds if the space shrinks
@@ -53,8 +55,8 @@ GrB_Info GB_hyper_realloc
         if (!ok)
         { 
             // out of memory
-            GB_PHIX_FREE (A) ;
-            return (GB_OUT_OF_MEMORY) ;
+            GB_phbix_free (A) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
 
         // size of A->p and A->h has been changed
