@@ -2,8 +2,8 @@
 // GB_ij.h: definitions for I and J index lists
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -12,12 +12,6 @@
 
 #include "GB.h"
 
-// kind of index list, Ikind and Jkind:
-#define GB_ALL 0
-#define GB_RANGE 1
-#define GB_STRIDE 2
-#define GB_LIST 4
-
 GB_PUBLIC   // accessed by the MATLAB interface only
 void GB_ijlength            // get the length and kind of an index list I
 (
@@ -51,11 +45,13 @@ GrB_Info GB_ijproperties        // check I and determine its properties
 
 GrB_Info GB_ijsort
 (
-    const GrB_Index *GB_RESTRICT I, // size ni, where ni > 1 always holds
-    int64_t *GB_RESTRICT p_ni,      // : size of I, output: # of indices in I2
-    GrB_Index *GB_RESTRICT *p_I2,   // size ni2, where I2 [0..ni2-1]
+    const GrB_Index *restrict I, // size ni, where ni > 1 always holds
+    int64_t *restrict p_ni,      // : size of I, output: # of indices in I2
+    GrB_Index *restrict *p_I2,   // size ni2, where I2 [0..ni2-1]
                         // contains the sorted indices with duplicates removed.
-    GrB_Index *GB_RESTRICT *p_I2k,  // output array of size ni2
+    size_t *I2_size_handle,
+    GrB_Index *restrict *p_I2k,  // output array of size ni2
+    size_t *I2k_size_handle,
     GB_Context Context
 ) ;
 
@@ -96,7 +92,7 @@ static inline int64_t GB_ijlist     // get the kth item in a list of indices
 static inline bool GB_ij_is_in_list // determine if i is in the list I
 (
     const GrB_Index *I,         // list of indices for GB_LIST
-    const int64_t nI,           // length of I
+    const int64_t nI,           // length of I if Ikind is GB_LIST
     int64_t i,                  // find i = I [k] in the list
     const int Ikind,            // GB_ALL, GB_RANGE, GB_STRIDE, or GB_LIST
     const int64_t Icolon [3]    // begin:inc:end for all but GB_LIST
@@ -126,7 +122,8 @@ static inline bool GB_ij_is_in_list // determine if i is in the list I
         int64_t e   = Icolon [GxB_END] ;
         if (inc == 0)
         { 
-            // I is empty if inc is zero, so i is not in I
+            // lo:stride:hi with stride of zero.
+            // I is empty if inc is zero, so i is not in I.
             return (false) ;
         }
         else if (inc > 0)
