@@ -2,8 +2,8 @@
 // GB_ijproperties: check I and determine its properties
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -16,8 +16,8 @@
 {                                                                           \
     if ((i) < 0 || (i) >= (limit))                                          \
     {                                                                       \
-        return (GB_ERROR (GrB_INDEX_OUT_OF_BOUNDS, (GB_LOG,                 \
-        "index " GBd " out of bounds, must be < " GBd , (i), (limit)))) ;   \
+        GB_ERROR (GrB_INDEX_OUT_OF_BOUNDS,                                  \
+        "index " GBd " out of bounds, must be < " GBd , (i), (limit)) ;     \
     }                                                                       \
 }
 
@@ -73,7 +73,7 @@ GrB_Info GB_ijproperties        // check I and determine its properties
     //--------------------------------------------------------------------------
 
     // scan the list of indices: check if OK, determine if they are
-    // jumbled, or contiguous, their min and max index, and actual length
+    // unsorted, or contiguous, their min and max index, and actual length
     bool I_unsorted = false ;
     bool I_has_duplicates = false ;
     bool I_contig = true ;
@@ -199,14 +199,17 @@ GrB_Info GB_ijproperties        // check I and determine its properties
         imax = -1 ;
 
         // allocate workspace for imin and imax
-        int64_t *Work = GB_MALLOC (2*ntasks, int64_t) ;
-        if (Work == NULL)
+        GB_WERK_DECLARE (Work_imin, int64_t) ;
+        GB_WERK_DECLARE (Work_imax, int64_t) ;
+        GB_WERK_PUSH (Work_imin, ntasks, int64_t) ;
+        GB_WERK_PUSH (Work_imax, ntasks, int64_t) ;
+        if (Work_imin == NULL || Work_imax == NULL)
         { 
             // out of memory
-            return (GB_OUT_OF_MEMORY) ;
+            GB_WERK_POP (Work_imax, int64_t) ;
+            GB_WERK_POP (Work_imin, int64_t) ;
+            return (GrB_OUT_OF_MEMORY) ;
         }
-        int64_t *Work_imin = Work ;
-        int64_t *Work_imax = Work + ntasks ;
 
         int tid ;
         #pragma omp parallel for num_threads(nthreads) schedule(dynamic,1) \
@@ -262,7 +265,8 @@ GrB_Info GB_ijproperties        // check I and determine its properties
         }
 
         // free workspace
-        GB_FREE (Work) ;
+        GB_WERK_POP (Work_imax, int64_t) ;
+        GB_WERK_POP (Work_imin, int64_t) ;
 
         #ifdef GB_DEBUG
         {
@@ -320,7 +324,7 @@ GrB_Info GB_ijproperties        // check I and determine its properties
         //----------------------------------------------------------------------
 
         if (I_contig)
-        { 
+        {
             // I is a contigous list of stride 1, imin:imax.
             // change Ikind to GB_ALL if 0:limit-1, or GB_RANGE otherwise
             if (imin == 0 && imax == limit-1)
