@@ -2,8 +2,8 @@
 // GB_shallow_op:  create a shallow copy and apply a unary operator to a matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -29,7 +29,7 @@
 GB_PUBLIC   // accessed by the MATLAB tests in GraphBLAS/Test only
 GrB_Info GB_shallow_op      // create shallow matrix and apply operator
 (
-    GrB_Matrix *Chandle,    // output matrix C, of type op*->ztype
+    GrB_Matrix C,           // output C, of type op*->ztype, static header
     const bool C_is_csc,    // desired CSR/CSC format of C
         const GrB_UnaryOp op1,          // unary operator to apply
         const GrB_BinaryOp op2,         // binary operator to apply
@@ -44,62 +44,62 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
     // check inputs
     //--------------------------------------------------------------------------
 
-    ASSERT (Chandle != NULL) ;
+    ASSERT (C != NULL && C->static_header) ;
     ASSERT_MATRIX_OK (A, "A for shallow_op", GB0) ;
-    GrB_Type ztype, op_intype ;
-    GB_Opcode opcode ;
+    ASSERT (!GB_ZOMBIES (A)) ;
+    ASSERT (GB_JUMBLED_OK (A)) ;
+    ASSERT (!GB_PENDING (A)) ;
+
+    GrB_Type ztype, op_intype = NULL ;
+    GB_Opcode opcode = (op1 != NULL) ? op1->opcode : op2->opcode ;
+    bool op_is_positional = GB_OPCODE_IS_POSITIONAL (opcode) ;
     if (op1 != NULL)
     {
         ASSERT_UNARYOP_OK (op1, "unop for shallow_op", GB0) ;
-        ASSERT (GB_Type_compatible (op1->xtype, A->type)) ;
-        op_intype = op1->xtype ;
+        if (!op_is_positional)
+        { 
+            ASSERT (GB_Type_compatible (op1->xtype, A->type)) ;
+            op_intype = op1->xtype ;
+        }
         ztype = op1->ztype ;
-        opcode = op1->opcode ;
     }
     else // op2 != NULL
     {
         ASSERT_BINARYOP_OK (op2, "binop for shallow_op", GB0) ;
-        op_intype = (binop_bind1st) ? op2->xtype : op2->ytype ;
+        if (!op_is_positional)
+        { 
+            op_intype = (binop_bind1st) ? op2->xtype : op2->ytype ;
+            ASSERT (GB_Type_compatible (op_intype, A->type)) ;
+        }
         ztype = op2->ztype ;
-        opcode = op2->opcode ;
     }
-    ASSERT (GB_Type_compatible (op_intype, A->type)) ;
-    ASSERT ((A->nzmax == 0) == (A->i == NULL && A->x == NULL)) ;
-    ASSERT (!GB_PENDING (A)) ; ASSERT (!GB_ZOMBIES (A)) ;
-
-    (*Chandle) = NULL ;
 
     //--------------------------------------------------------------------------
     // construct a shallow copy of A for the pattern of C
     //--------------------------------------------------------------------------
 
-    // allocate the struct for C, but do not allocate C->h, C->p, C->i, or C->x.
-    // C has the exact same hypersparsity as A.
+    // initialized the header for C, but do not allocate C->{p,h,b,i,x}
+    // C has the exact same sparsity structure as A.
     GrB_Info info ;
-    GrB_Matrix C = NULL ;           // allocate a new header for C
-    info = GB_new (&C, ztype, A->vlen, A->vdim, GB_Ap_null, C_is_csc,
-        GB_SAME_HYPER_AS (A->is_hyper), A->hyper_ratio, 0, Context) ;
-    if (info != GrB_SUCCESS)
-    { 
-        // out of memory
-        return (info) ;
-    }
+    info = GB_new (&C, true, // any sparsity, static header
+        ztype, A->vlen, A->vdim, GB_Ap_null, C_is_csc,
+        GB_sparsity (A), A->hyper_switch, 0, Context) ;
+    ASSERT (info == GrB_SUCCESS) ;
 
     //--------------------------------------------------------------------------
     // make a shallow copy of the vector pointers
     //--------------------------------------------------------------------------
 
-    ASSERT (C->magic == GB_MAGIC2) ;   // [ be careful; C not yet initialized
-    C->p_shallow = true ;           // C->p not freed when freeing C
-    C->h_shallow = true ;           // C->h not freed when freeing C
-    C->p = A->p ;                   // C->p is of size A->plen + 1
-    C->h = A->h ;                   // C->h is of size A->plen
-    C->plen = A->plen ;             // C and A have the same hyperlist sizes
+    C->p_shallow = (A->p != NULL) ;     // C->p not freed when freeing C
+    C->h_shallow = (A->h != NULL) ;     // C->h not freed when freeing C
+    C->p = A->p ;                       // C->p is of size A->plen + 1
+    C->h = A->h ;                       // C->h is of size A->plen
+    C->plen = A->plen ;                 // C and A have the same hyperlist sizes
     C->nvec = A->nvec ;
-    ASSERT (A->nvec_nonempty == -1 ||   // can be postponed
-            A->nvec_nonempty == GB_nvec_nonempty (A, Context)) ;
     C->nvec_nonempty = A->nvec_nonempty ;
-    C->magic = GB_MAGIC ;           // C is now initialized ]
+    C->jumbled = A->jumbled ;           // C is jumbled if A is jumbled
+    C->nvals = A->nvals ;               // if A bitmap 
+    C->magic = GB_MAGIC ;
 
     //--------------------------------------------------------------------------
     // check for empty matrix
@@ -109,12 +109,14 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
     { 
         // C->p and C->h are shallow but the rest is empty
         C->nzmax = 0 ;
+        C->b = NULL ;
         C->i = NULL ;
         C->x = NULL ;
+        C->b_shallow = false ;
         C->i_shallow = false ;
         C->x_shallow = false ;
+        C->jumbled = false ;
         ASSERT_MATRIX_OK (C, "C = quick copy of empty A", GB0) ;
-        (*Chandle) = C ;
         return (GrB_SUCCESS) ;
     }
 
@@ -122,8 +124,11 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
     // make a shallow copy of the pattern
     //--------------------------------------------------------------------------
 
+    C->b = A->b ;               // of size A->nzmax
+    C->b_shallow = (A->b != NULL) ;  // C->b will not be freed when freeing C
+
     C->i = A->i ;               // of size A->nzmax
-    C->i_shallow = true ;       // C->i will not be freed when freeing C
+    C->i_shallow = (A->i != NULL) ; // C->i will not be freed when freeing C
 
     //--------------------------------------------------------------------------
     // make a shallow copy of the values, if possible
@@ -133,8 +138,8 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
     // are used with no typecasting, C->x becomes a shallow copy of A->x, and
     // no work is done.
 
-    int64_t anz = GB_NNZ (A) ;
-    ASSERT (A->nzmax >= GB_IMAX (anz,1)) ;
+    int64_t anz = GB_NNZ_HELD (A) ;
+    ASSERT (A->nzmax >= GB_IMAX (anz, 1)) ;
 
     if (A->type == op_intype &&
         ((opcode == GB_IDENTITY_opcode) || (opcode == GB_ANY_opcode) ||
@@ -142,12 +147,12 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
          (opcode == GB_SECOND_opcode &&  binop_bind1st)))
     { 
         // no work is done at all.  C is a pure shallow copy
-        GBBURBLE ("(pure shallow) ") ;
+        GBURBLE ("(pure shallow) ") ;
         C->nzmax = A->nzmax ;
         C->x = A->x ;
         C->x_shallow = true ;       // C->x will not be freed when freeing C
+        ASSERT (C->x_size == 0) ;
         ASSERT_MATRIX_OK (C, "C = pure shallow (A)", GB0) ;
-        (*Chandle) = C ;
         return (GrB_SUCCESS) ;
     }
 
@@ -156,28 +161,31 @@ GrB_Info GB_shallow_op      // create shallow matrix and apply operator
     //--------------------------------------------------------------------------
 
     // allocate new space for the numerical values of C
-    C->nzmax = GB_IMAX (anz,1) ;
-    C->x = GB_MALLOC (C->nzmax * C->type->size, GB_void) ;
+    C->nzmax = GB_IMAX (anz, 1) ;
+    C->x = GB_MALLOC (C->nzmax * C->type->size, GB_void, &(C->x_size)) ;
     C->x_shallow = false ;          // free C->x when freeing C
     if (C->x == NULL)
     { 
         // out of memory
-        GB_MATRIX_FREE (&C) ;
-        return (GB_OUT_OF_MEMORY) ;
+        GB_phbix_free (C) ;
+        return (GrB_OUT_OF_MEMORY) ;
     }
 
-          GB_void *Cx = (GB_void *) C->x ;
-    const GB_void *Ax = (GB_void *) A->x ;
-    GB_apply_op (Cx, 
-        op1, op2, scalar, binop_bind1st,
-        Ax, A->type, anz, Context) ;
+    GB_void *Cx = (GB_void *) C->x ;
+    info = GB_apply_op (Cx, op1,    // op1 is never identity of same types
+        op2, scalar, binop_bind1st, A, Context) ;
+    if (info != GrB_SUCCESS)
+    { 
+        // out of memory
+        GB_phbix_free (C) ;
+        return (GrB_OUT_OF_MEMORY) ;
+    }
 
     //--------------------------------------------------------------------------
     // return the result
     //--------------------------------------------------------------------------
 
     ASSERT_MATRIX_OK (C, "C = shallow (op (A))", GB0) ;
-    (*Chandle) = C ;
     return (GrB_SUCCESS) ;
 }
 
