@@ -2,8 +2,8 @@
 // GB_subassign_01: C(I,J) = scalar ; using S
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -16,6 +16,8 @@
 // A:           scalar
 // S:           constructed
 
+// C: not bitmap
+
 #include "GB_subassign_methods.h"
 
 GrB_Info GB_subassign_01
@@ -23,34 +25,45 @@ GrB_Info GB_subassign_01
     GrB_Matrix C,
     // input:
     const GrB_Index *I,
+    const int64_t ni,
     const int64_t nI,
     const int Ikind,
     const int64_t Icolon [3],
     const GrB_Index *J,
+    const int64_t nj,
     const int64_t nJ,
     const int Jkind,
     const int64_t Jcolon [3],
     const void *scalar,
     const GrB_Type atype,
-    const GrB_Matrix S,
     GB_Context Context
 )
 {
 
+    //--------------------------------------------------------------------------
+    // check inputs
+    //--------------------------------------------------------------------------
+
+    ASSERT (!GB_IS_BITMAP (C)) ;
+
+    //--------------------------------------------------------------------------
+    // S = C(I,J)
+    //--------------------------------------------------------------------------
+
+    GB_EMPTY_TASKLIST ;
+    GB_OK (GB_subassign_symbolic (S, C, I, ni, J, nj, true, Context)) ;
+
     //--------------------------------------------------------------------------
     // get inputs
     //--------------------------------------------------------------------------
 
-    GB_GET_C ;
-    const bool C_is_hyper = C->is_hyper ;
-    const int64_t *GB_RESTRICT Ch = C->h ;
-    const int64_t *GB_RESTRICT Cp = C->p ;
+    GB_GET_C ;      // C must not be bitmap
+    const int64_t *restrict Ch = C->h ;
+    const int64_t *restrict Cp = C->p ;
+    const bool C_is_hyper = (Ch != NULL) ;
     const int64_t Cnvec = C->nvec ;
     GB_GET_SCALAR ;
     GB_GET_S ;
-    const int64_t *GB_RESTRICT Sh = S->h ;
-    const int64_t Snvec = S->nvec ;
-    const bool S_is_hyper = S->is_hyper ;
     GrB_BinaryOp accum = NULL ;
 
     //--------------------------------------------------------------------------
@@ -75,7 +88,6 @@ GrB_Info GB_subassign_01
     // phase 1: create zombies, update entries, and count pending tuples
     //--------------------------------------------------------------------------
 
-    int taskid ;
     #pragma omp parallel for num_threads(nthreads) schedule(dynamic,1) \
         reduction(+:nzombies)
     for (taskid = 0 ; taskid < ntasks ; taskid++)
@@ -85,7 +97,7 @@ GrB_Info GB_subassign_01
         // get the task descriptor
         //----------------------------------------------------------------------
 
-        GB_GET_IXJ_TASK_DESCRIPTOR_PHASE1 ;
+        GB_GET_IXJ_TASK_DESCRIPTOR_PHASE1 (iA_start, iA_end) ;
 
         //----------------------------------------------------------------------
         // compute all vectors in this task
@@ -98,13 +110,13 @@ GrB_Info GB_subassign_01
             // get jC, the corresponding vector of C
             //------------------------------------------------------------------
 
-            GB_GET_jC ;
+            int64_t jC = GB_ijlist (J, j, Jkind, Jcolon) ;
 
             //------------------------------------------------------------------
             // get S(iA_start:end,j)
             //------------------------------------------------------------------
 
-            GB_GET_VECTOR_FOR_IXJ (S) ;
+            GB_GET_VECTOR_FOR_IXJ (S, iA_start) ;
 
             //------------------------------------------------------------------
             // C(I(iA_start,iA_end-1),jC) = scalar
@@ -112,7 +124,7 @@ GrB_Info GB_subassign_01
 
             for (int64_t iA = iA_start ; iA < iA_end ; iA++)
             {
-                bool found = (pS < pS_end) && (Si [pS] == iA) ;
+                bool found = (pS < pS_end) && (GBI (Si, pS, Svlen) == iA) ;
                 if (!found)
                 { 
                     // ----[. A 1]----------------------------------------------
@@ -151,7 +163,7 @@ GrB_Info GB_subassign_01
         // get the task descriptor
         //----------------------------------------------------------------------
 
-        GB_GET_IXJ_TASK_DESCRIPTOR_PHASE2 ;
+        GB_GET_IXJ_TASK_DESCRIPTOR_PHASE2 (iA_start, iA_end) ;
 
         //----------------------------------------------------------------------
         // compute all vectors in this task
@@ -164,13 +176,13 @@ GrB_Info GB_subassign_01
             // get jC, the corresponding vector of C
             //------------------------------------------------------------------
 
-            GB_GET_jC ;
+            int64_t jC = GB_ijlist (J, j, Jkind, Jcolon) ;
 
             //------------------------------------------------------------------
             // get S(iA_start:end,j)
             //------------------------------------------------------------------
 
-            GB_GET_VECTOR_FOR_IXJ (S) ;
+            GB_GET_VECTOR_FOR_IXJ (S, iA_start) ;
 
             //------------------------------------------------------------------
             // C(I(iA_start,iA_end-1),jC) = scalar
@@ -178,7 +190,7 @@ GrB_Info GB_subassign_01
 
             for (int64_t iA = iA_start ; iA < iA_end ; iA++)
             {
-                bool found = (pS < pS_end) && (Si [pS] == iA) ;
+                bool found = (pS < pS_end) && (GBI (Si, pS, Svlen) == iA) ;
                 if (!found)
                 { 
                     // ----[. A 1]----------------------------------------------
