@@ -2,8 +2,8 @@
 // GB_subassign_06n: C(I,J)<M> = A ; no S
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -16,6 +16,17 @@
 // A:           matrix
 // S:           none (see also GB_subassign_06s)
 
+// FULL: if A and C are dense, then C remains dense.
+
+// If A is sparse and C dense, C will likely become sparse, except if M(i,j)=0
+// wherever A(i,j) is not present.  So if M==A is aliased and A is sparse, then
+// C remains dense.  Need C(I,J)<A,struct>=A kernel.  Then in that case, if C
+// is dense it remains dense, even if A is sparse.   If that change is made,
+// this kernel can start with converting C to sparse if A is sparse.
+
+// C is not bitmap: GB_bitmap_assign is used if C is bitmap.
+// M and A are not bitmap: 06s is used instead, if M or A are bitmap.
+
 #include "GB_subassign_methods.h"
 
 GrB_Info GB_subassign_06n
@@ -37,23 +48,40 @@ GrB_Info GB_subassign_06n
 )
 {
 
+    //--------------------------------------------------------------------------
+    // check inputs
+    //--------------------------------------------------------------------------
+
+    ASSERT (!GB_IS_BITMAP (C)) ; ASSERT (!GB_IS_FULL (C)) ;
+    ASSERT (!GB_IS_BITMAP (M)) ;    // Method 06n is not used for M bitmap
+    ASSERT (!GB_IS_BITMAP (A)) ;    // Method 06n is not used for A bitmap
+    ASSERT (!GB_aliased (C, M)) ;   // NO ALIAS of C==M
+    ASSERT (!GB_aliased (C, A)) ;   // NO ALIAS of C==A
+
+    ASSERT_MATRIX_OK (C, "C input for 06n", GB0) ;
+    ASSERT_MATRIX_OK (M, "M input for 06n", GB0) ;
+    ASSERT_MATRIX_OK (A, "A input for 06n", GB0) ;
+
     //--------------------------------------------------------------------------
     // get inputs
     //--------------------------------------------------------------------------
 
-    GB_GET_C ;
+    GB_EMPTY_TASKLIST ;
+    GB_MATRIX_WAIT_IF_JUMBLED (C) ;
+    GB_MATRIX_WAIT_IF_JUMBLED (M) ;
+    GB_MATRIX_WAIT_IF_JUMBLED (A) ;
+
+    GB_GET_C ;      // C must not be bitmap
     int64_t zorig = C->nzombies ;
-    const bool C_is_hyper = C->is_hyper ;
     const int64_t Cnvec = C->nvec ;
-    const int64_t cvlen = C->vlen ;
-    const int64_t *GB_RESTRICT Ch = C->h ;
-    const int64_t *GB_RESTRICT Cp = C->p ;
+    const int64_t *restrict Ch = C->h ;
+    const int64_t *restrict Cp = C->p ;
+    const bool C_is_hyper = (Ch != NULL) ;
     GB_GET_MASK ;
     GB_GET_A ;
-    const int64_t *GB_RESTRICT Ah = A->h ;
+    const int64_t *restrict Ah = A->h ;
     const int64_t Anvec = A->nvec ;
-    const bool A_is_hyper = A->is_hyper ;
-    const int64_t avlen = A->vlen ;
+    const bool A_is_hyper = (Ah != NULL) ;
     GrB_BinaryOp accum = NULL ;
 
     //--------------------------------------------------------------------------
@@ -74,13 +102,12 @@ GrB_Info GB_subassign_06n
     // Parallel: slice M into coarse/fine tasks (Method 05, 06n, 07)
     //--------------------------------------------------------------------------
 
-    GB_SUBASSIGN_ONE_SLICE (M) ;
+    GB_SUBASSIGN_ONE_SLICE (M) ;    // M cannot be jumbled 
 
     //--------------------------------------------------------------------------
     // phase 1: create zombies, update entries, and count pending tuples
     //--------------------------------------------------------------------------
 
-    int taskid ;
     #pragma omp parallel for num_threads(nthreads) schedule(dynamic,1) \
         reduction(+:nzombies)
     for (taskid = 0 ; taskid < ntasks ; taskid++)
@@ -103,8 +130,8 @@ GrB_Info GB_subassign_06n
             // get j, the kth vector of M
             //------------------------------------------------------------------
 
-            int64_t j = (Mh == NULL) ? k : Mh [k] ;
-            GB_GET_VECTOR (pM, pM_end, pA, pA_end, Mp, k) ;
+            int64_t j = GBH (Mh, k) ;
+            GB_GET_VECTOR (pM, pM_end, pA, pA_end, Mp, k, Mvlen) ;
             int64_t mjnz = pM_end - pM ;
             if (mjnz == 0) continue ;
 
@@ -115,7 +142,7 @@ GrB_Info GB_subassign_06n
             int64_t pA, pA_end ;
             GB_VECTOR_LOOKUP (pA, pA_end, A, j) ;
             int64_t ajnz = pA_end - pA ;
-            bool ajdense = (ajnz == avlen) ;
+            bool ajdense = (ajnz == Avlen) ;
             int64_t pA_start = pA ;
 
             //------------------------------------------------------------------
@@ -125,7 +152,7 @@ GrB_Info GB_subassign_06n
             GB_GET_jC ;
             int64_t cjnz = pC_end - pC_start ;
             if (cjnz == 0 && ajnz == 0) continue ;
-            bool cjdense = (cjnz == cvlen) ;
+            bool cjdense = (cjnz == Cvlen) ;
 
             //------------------------------------------------------------------
             // C(I,jC)<M(:,j)> = A(:,j) ; no S
@@ -147,13 +174,13 @@ GrB_Info GB_subassign_06n
 
                     if (GB_mcast (Mx, pM, msize))
                     { 
-                        int64_t iA = Mi [pM] ;
+                        int64_t iA = GBI (Mi, pM, Mvlen) ;
                         GB_iC_DENSE_LOOKUP ;
 
                         // find iA in A(:,j)
                         // A(:,j) is dense; no need for binary search
                         pA = pA_start + iA ;
-                        ASSERT (Ai [pA] == iA) ;
+                        ASSERT (GBI (Ai, pA, Avlen) == iA) ;
                         // ----[C A 1] or [X A 1]-----------------------
                         // [C A 1]: action: ( =A ): copy A to C, no acc
                         // [X A 1]: action: ( undelete ): zombie lives
@@ -177,8 +204,8 @@ GrB_Info GB_subassign_06n
                     //----------------------------------------------------------
 
                     if (GB_mcast (Mx, pM, msize))
-                    { 
-                        int64_t iA = Mi [pM] ;
+                    {
+                        int64_t iA = GBI (Mi, pM, Mvlen) ;
                         GB_iC_DENSE_LOOKUP ;
 
                         // find iA in A(:,j)
@@ -221,14 +248,14 @@ GrB_Info GB_subassign_06n
 
                     if (GB_mcast (Mx, pM, msize))
                     {
-                        int64_t iA = Mi [pM] ;
+                        int64_t iA = GBI (Mi, pM, Mvlen) ;
 
                         // find C(iC,jC) in C(:,jC)
                         GB_iC_BINARY_SEARCH ;
 
                         // lookup iA in A(:,j)
                         pA = pA_start + iA ;
-                        ASSERT (Ai [pA] == iA) ;
+                        ASSERT (GBI (Ai, pA, Avlen) == iA) ;
 
                         if (cij_found)
                         { 
@@ -264,7 +291,7 @@ GrB_Info GB_subassign_06n
 
                     if (GB_mcast (Mx, pM, msize))
                     {
-                        int64_t iA = Mi [pM] ;
+                        int64_t iA = GBI (Mi, pM, Mvlen) ;
 
                         // find C(iC,jC) in C(:,jC)
                         GB_iC_BINARY_SEARCH ;
@@ -333,8 +360,8 @@ GrB_Info GB_subassign_06n
             // get j, the kth vector of M
             //------------------------------------------------------------------
 
-            int64_t j = (Mh == NULL) ? k : Mh [k] ;
-            GB_GET_VECTOR (pM, pM_end, pA, pA_end, Mp, k) ;
+            int64_t j = GBH (Mh, k) ;
+            GB_GET_VECTOR (pM, pM_end, pA, pA_end, Mp, k, Mvlen) ;
             int64_t mjnz = pM_end - pM ;
             if (mjnz == 0) continue ;
 
@@ -346,7 +373,7 @@ GrB_Info GB_subassign_06n
             GB_VECTOR_LOOKUP (pA, pA_end, A, j) ;
             int64_t ajnz = pA_end - pA ;
             if (ajnz == 0) continue ;
-            bool ajdense = (ajnz == avlen) ;
+            bool ajdense = (ajnz == Avlen) ;
             int64_t pA_start = pA ;
 
             //------------------------------------------------------------------
@@ -354,7 +381,7 @@ GrB_Info GB_subassign_06n
             //------------------------------------------------------------------
 
             GB_GET_jC ;
-            bool cjdense = ((pC_end - pC_start) == cvlen) ;
+            bool cjdense = ((pC_end - pC_start) == Cvlen) ;
 
             //------------------------------------------------------------------
             // C(I,jC)<M(:,j)> = A(:,j)
@@ -376,17 +403,17 @@ GrB_Info GB_subassign_06n
 
                     if (GB_mcast (Mx, pM, msize))
                     {
-                        int64_t iA = Mi [pM] ;
+                        int64_t iA = GBI (Mi, pM, Mvlen) ;
 
                         // find iA in A(:,j)
                         if (ajdense)
-                        {
+                        { 
                             // A(:,j) is dense; no need for binary search
                             pA = pA_start + iA ;
-                            ASSERT (Ai [pA] == iA) ;
+                            ASSERT (GBI (Ai, pA, Avlen) == iA) ;
                         }
                         else
-                        {
+                        { 
                             // A(:,j) is sparse; use binary search
                             int64_t apright = pA_end - 1 ;
                             bool aij_found ;
