@@ -2,8 +2,8 @@
 // GB_subref_phase2: C=A(I,J)
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,12 +15,13 @@
 
 GrB_Info GB_subref_phase2   // C=A(I,J)
 (
-    GrB_Matrix *Chandle,    // output matrix (unallocated on input)
+    GrB_Matrix C,               // output matrix, static header
     // from phase1:
-    const int64_t *GB_RESTRICT Cp,         // vector pointers for C
+    int64_t **Cp_handle,        // vector pointers for C
+    size_t Cp_size,
     const int64_t Cnvec_nonempty,       // # of non-empty vectors in C
     // from phase0b:
-    const GB_task_struct *GB_RESTRICT TaskList,    // array of structs
+    const GB_task_struct *restrict TaskList,    // array of structs
     const int ntasks,                           // # of tasks
     const int nthreads,                         // # of threads to use
     const bool post_sort,               // true if post-sort needed
@@ -28,9 +29,10 @@ GrB_Info GB_subref_phase2   // C=A(I,J)
     const int64_t *Inext,               // for I inverse buckets, size nI
     const int64_t nduplicates,          // # of duplicates, if I inverted
     // from phase0:
-    const int64_t *GB_RESTRICT Ch,
-    const int64_t *GB_RESTRICT Ap_start,
-    const int64_t *GB_RESTRICT Ap_end,
+    int64_t **Ch_handle,
+    size_t Ch_size,
+    const int64_t *restrict Ap_start,
+    const int64_t *restrict Ap_end,
     const int64_t Cnvec,
     const bool need_qsort,
     const int Ikind,
@@ -50,46 +52,54 @@ GrB_Info GB_subref_phase2   // C=A(I,J)
     // check inputs
     //--------------------------------------------------------------------------
 
+    ASSERT (C != NULL && C->static_header) ;
+    ASSERT (Cp_handle != NULL) ;
+    ASSERT (Ch_handle != NULL) ;
+    const int64_t *restrict Ch = (*Ch_handle) ;
+    const int64_t *restrict Cp = (*Cp_handle) ;
     ASSERT (Cp != NULL) ;
     ASSERT_MATRIX_OK (A, "A for subref phase2", GB0) ;
+    ASSERT (!GB_IS_BITMAP (A)) ;    // GB_bitmap_subref is used instead
 
     //--------------------------------------------------------------------------
     // allocate the output matrix C
     //--------------------------------------------------------------------------
 
     int64_t cnz = Cp [Cnvec] ;
-    (*Chandle) = NULL ;
 
     bool C_is_hyper = (Ch != NULL) ;
 
     GrB_Type ctype = (symbolic) ? GrB_INT64 : A->type ;
 
     // allocate the result C (but do not allocate C->p or C->h)
-    GrB_Matrix C = NULL ;           // allocate a new header for C
-    GrB_Info info = GB_create (&C, ctype, nI, nJ, GB_Ap_null, C_is_csc,
-        GB_SAME_HYPER_AS (C_is_hyper), A->hyper_ratio, Cnvec, cnz, true,
-        Context) ;
+    int sparsity = C_is_hyper ? GxB_HYPERSPARSE : GxB_SPARSE ;
+    GrB_Info info = GB_new_bix (&C, true, // sparse or hyper, static header
+        ctype, nI, nJ, GB_Ap_null, C_is_csc,
+        sparsity, true, A->hyper_switch, Cnvec, cnz, true, Context) ;
     if (info != GrB_SUCCESS)
     { 
         // out of memory
-        GB_FREE (Cp) ;
-        GB_FREE (Ch) ;
+        GB_FREE (Cp_handle, Cp_size) ;
+        GB_FREE (Ch_handle, Ch_size) ;
         return (info) ;
     }
 
     // add Cp as the vector pointers for C, from GB_subref_phase1
-    C->p = (int64_t *) Cp ;
+    C->p = (int64_t *) Cp ; C->p_size = Cp_size ;
+    (*Cp_handle) = NULL ;
 
     // add Ch as the hypersparse list for C, from GB_subref_phase0
     if (C_is_hyper)
     { 
         // transplant Ch into C
-        C->h = (int64_t *) Ch ;
+        C->h = (int64_t *) Ch ; C->h_size = Ch_size ;
+        (*Ch_handle) = NULL ;
         C->nvec = Cnvec ;
     }
 
     // now Cp and Ch have been transplanted into C, so they must not be freed.
-
+    ASSERT ((*Cp_handle) == NULL) ;
+    ASSERT ((*Ch_handle) == NULL) ;
     C->nvec_nonempty = Cnvec_nonempty ;
     C->magic = GB_MAGIC ;
 
@@ -99,13 +109,13 @@ GrB_Info GB_subref_phase2   // C=A(I,J)
 
     #define GB_PHASE_2_OF_2
     if (symbolic)
-    {
+    { 
         #define GB_SYMBOLIC
         #include "GB_subref_template.c"
         #undef  GB_SYMBOLIC
     }
     else
-    {
+    { 
         #define GB_NUMERIC
         #include "GB_subref_template.c"
         #undef  GB_NUMERIC
@@ -119,7 +129,7 @@ GrB_Info GB_subref_phase2   // C=A(I,J)
     if (info != GrB_SUCCESS)
     { 
         // out of memory
-        GB_MATRIX_FREE (&C) ;
+        GB_phbix_free (C) ;
         return (info) ;
     }
 
@@ -127,11 +137,8 @@ GrB_Info GB_subref_phase2   // C=A(I,J)
     // return result
     //--------------------------------------------------------------------------
 
-    // caller must not free Cp or Ch.   The matrix may have jumbled indices.
-    // If it will be transposed in GB_accum_mask, but needs sorting, then the
-    // sort is skipped since the transpose will handle the sort.
-    ASSERT_MATRIX_OK_OR_JUMBLED (C, "C output for subref phase2", GB0) ;
-    (*Chandle) = C ;
+    // caller must not free Cp or Ch
+    ASSERT_MATRIX_OK (C, "C output for subref phase2", GB0) ;
     return (GrB_SUCCESS) ;
 }
 
