@@ -2,8 +2,8 @@
 // GB_unused.h: pragmas to disable compiler warnings
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,7 +15,7 @@
 
 #if ( _MSC_VER && !__INTEL_COMPILER )
 // disable MS Visual Studio warnings
-GB_PRAGMA (warning (disable : 4101 ))
+#pragma warning (disable: 4101 )
 #elif defined ( __INTEL_COMPILER )
 // disable icc -w3 warnings
 #pragma warning (disable: 177 593)
