@@ -2,14 +2,14 @@
 // GrB_BinaryOp_wait: wait for a user-defined GrB_BinaryOp to complete
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
 // In SuiteSparse:GraphBLAS, a user-defined GrB_BinaryOp has no pending
 // operations to wait for.  All this method does is verify that the op is
-// properly initialized.
+// properly initialized, and then it does an OpenMP flush.
 
 #include "GB.h"
 
@@ -23,7 +23,8 @@ GrB_Info GrB_BinaryOp_wait   // no work, just check if the GrB_BinaryOp is valid
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GrB_BinaryOp_wait (&op)") ;
+    #pragma omp flush
+    GB_WHERE1 ("GrB_BinaryOp_wait (&op)") ;
     GB_RETURN_IF_NULL (op) ;
     GB_RETURN_IF_NULL_OR_FAULTY (*op) ;
 
@@ -31,6 +32,7 @@ GrB_Info GrB_BinaryOp_wait   // no work, just check if the GrB_BinaryOp is valid
     // return result
     //--------------------------------------------------------------------------
 
+    #pragma omp flush
     return (GrB_SUCCESS) ;
 }
 
