@@ -2,15 +2,15 @@
 // GrB_Vector_build: build a sparse GraphBLAS vector
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
 #include "GB_build.h"
 
 #define GB_VECTOR_BUILD(prefix,type,T)                                        \
-GrB_Info prefix ## Vector_build_ ## T   /* build a vector from (I,X) tuples*/ \
+GrB_Info GB_EVAL3 (prefix, _Vector_build_, T) /* build a vector from tuples*/ \
 (                                                                             \
     GrB_Vector w,                   /* vector to build                    */  \
     const GrB_Index *I,             /* array of row indices of tuples     */  \
@@ -19,7 +19,7 @@ GrB_Info prefix ## Vector_build_ ## T   /* build a vector from (I,X) tuples*/ \
     const GrB_BinaryOp dup          /* binary op to assemble duplicates   */  \
 )                                                                             \
 {                                                                             \
-    GB_WHERE (GB_STR(prefix) "Vector_build_" GB_STR(T)                        \
+    GB_WHERE (w, "GrB_Vector_build_" GB_STR(T)                                \
         " (w, I, X, nvals, dup)") ;                                           \
     GB_BURBLE_START ("GrB_Vector_build") ;                                    \
     GB_RETURN_IF_NULL_OR_FAULTY (w) ;                                         \
@@ -31,18 +31,18 @@ GrB_Info prefix ## Vector_build_ ## T   /* build a vector from (I,X) tuples*/ \
     return (info) ;                                                           \
 }
 
-GB_VECTOR_BUILD (GrB_, bool      , BOOL   )
-GB_VECTOR_BUILD (GrB_, int8_t    , INT8   )
-GB_VECTOR_BUILD (GrB_, uint8_t   , UINT8  )
-GB_VECTOR_BUILD (GrB_, int16_t   , INT16  )
-GB_VECTOR_BUILD (GrB_, uint16_t  , UINT16 )
-GB_VECTOR_BUILD (GrB_, int32_t   , INT32  )
-GB_VECTOR_BUILD (GrB_, uint32_t  , UINT32 )
-GB_VECTOR_BUILD (GrB_, int64_t   , INT64  )
-GB_VECTOR_BUILD (GrB_, uint64_t  , UINT64 )
-GB_VECTOR_BUILD (GrB_, float     , FP32   )
-GB_VECTOR_BUILD (GrB_, double    , FP64   )
-GB_VECTOR_BUILD (GxB_, GxB_FC32_t, FC32   )
-GB_VECTOR_BUILD (GxB_, GxB_FC64_t, FC64   )
-GB_VECTOR_BUILD (GrB_, void      , UDT    )
+GB_VECTOR_BUILD (GrB, bool      , BOOL   )
+GB_VECTOR_BUILD (GrB, int8_t    , INT8   )
+GB_VECTOR_BUILD (GrB, uint8_t   , UINT8  )
+GB_VECTOR_BUILD (GrB, int16_t   , INT16  )
+GB_VECTOR_BUILD (GrB, uint16_t  , UINT16 )
+GB_VECTOR_BUILD (GrB, int32_t   , INT32  )
+GB_VECTOR_BUILD (GrB, uint32_t  , UINT32 )
+GB_VECTOR_BUILD (GrB, int64_t   , INT64  )
+GB_VECTOR_BUILD (GrB, uint64_t  , UINT64 )
+GB_VECTOR_BUILD (GrB, float     , FP32   )
+GB_VECTOR_BUILD (GrB, double    , FP64   )
+GB_VECTOR_BUILD (GxB, GxB_FC32_t, FC32   )
+GB_VECTOR_BUILD (GxB, GxB_FC64_t, FC64   )
+GB_VECTOR_BUILD (GrB, void      , UDT    )
 
