@@ -2,8 +2,8 @@
 // GrB_Vector_dup: make a deep copy of a sparse vector
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2019, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -22,7 +22,8 @@ GrB_Info GrB_Vector_dup     // make an exact copy of a vector
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GrB_Vector_dup (&w, u)") ;
+    GB_WHERE1 ("GrB_Vector_dup (&w, u)") ;
+    GB_BURBLE_START ("GrB_Vector_dup") ;
     GB_RETURN_IF_NULL (w) ;
     GB_RETURN_IF_NULL_OR_FAULTY (u) ;
     ASSERT (GB_VECTOR_OK (u)) ;
@@ -31,6 +32,8 @@ GrB_Info GrB_Vector_dup     // make an exact copy of a vector
     // duplicate the vector
     //--------------------------------------------------------------------------
 
-    return (GB_dup ((GrB_Matrix *) w, (GrB_Matrix) u, true, NULL, Context)) ;
+    GrB_Info info = GB_dup ((GrB_Matrix *) w, (GrB_Matrix) u, Context) ;
+    GB_BURBLE_END ;
+    return (info) ;
 }
 
