@@ -2,8 +2,8 @@
 // GxB_BinaryOp_fprint: print and check a GrB_BinaryOp object
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -22,12 +22,12 @@ GrB_Info GxB_BinaryOp_fprint        // print and check a GrB_BinaryOp
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GxB_BinaryOp_fprint (binaryop, name, pr, f)") ;
+    GB_WHERE1 ("GxB_BinaryOp_fprint (binaryop, name, pr, f)") ;
 
     //--------------------------------------------------------------------------
     // print and check the object
     //--------------------------------------------------------------------------
 
-    return (GB_BinaryOp_check (binaryop, name, pr, f, Context)) ;
+    return (GB_BinaryOp_check (binaryop, name, pr, f)) ;
 }
 
