@@ -2,8 +2,8 @@
 // GxB_Desc_get: get a field in a descriptor
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -21,7 +21,7 @@ GrB_Info GxB_Desc_get           // get a parameter from a descriptor
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GxB_Desc_get (desc, field, &value)") ;
+    GB_WHERE1 ("GxB_Desc_get (desc, field, &value)") ;
     GB_RETURN_IF_FAULTY (desc) ;
 
     //--------------------------------------------------------------------------
@@ -110,15 +110,21 @@ GrB_Info GxB_Desc_get           // get a parameter from a descriptor
             }
             break ;
 
+        case GxB_SORT :
+
+            {
+                va_start (ap, field) ;
+                int *do_sort = va_arg (ap, int *) ;
+                va_end (ap) ;
+                GB_RETURN_IF_NULL (do_sort) ;
+                int s = (desc == NULL) ? GxB_DEFAULT : desc->do_sort ;
+                (*do_sort) = s ;
+            }
+            break ;
+
         default : 
 
-            return (GB_ERROR (GrB_INVALID_VALUE, (GB_LOG,
-                "invalid descriptor field [%d], must be one of:\n"
-                "GrB_OUTP [%d], GrB_MASK [%d], GrB_INP0 [%d], GrB_INP1 [%d],\n"
-                "GxB_NTHREADS [%d], GxB_CHUNK [%d] or GxB_AxB_METHOD [%d]",
-                (int) field, (int) GrB_OUTP, (int) GrB_MASK, (int) GrB_INP0,
-                (int) GrB_INP1, (int) GxB_NTHREADS, (int) GxB_CHUNK, 
-                (int) GxB_AxB_METHOD))) ;
+            return (GrB_INVALID_VALUE) ;
     }
 
     return (GrB_SUCCESS) ;
