@@ -2,8 +2,8 @@
 // GxB_Matrix_fprint: print and check a GrB_Matrix object
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -22,13 +22,13 @@ GrB_Info GxB_Matrix_fprint          // print and check a GrB_Matrix
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GxB_Matrix_fprint (A, name, pr, f)") ;
+    GB_WHERE1 ("GxB_Matrix_fprint (A, name, pr, f)") ;
 
     //--------------------------------------------------------------------------
     // print and check the object
     //--------------------------------------------------------------------------
 
-    GrB_Info info = GB_Matrix_check (A, name, pr, f, Context) ;
+    GrB_Info info = GB_Matrix_check (A, name, pr, f) ;
 
     //--------------------------------------------------------------------------
     // return result
@@ -36,8 +36,8 @@ GrB_Info GxB_Matrix_fprint          // print and check a GrB_Matrix
 
     if (info == GrB_INDEX_OUT_OF_BOUNDS)
     { 
-        return (GB_ERROR (GrB_INVALID_OBJECT, (GB_LOG,
-            "matrix invalid: indices out of order [%s]", GB_NAME))) ;
+        // indices out of order
+        return (GrB_INVALID_OBJECT) ;
     }
     else
     { 
