@@ -2,8 +2,8 @@
 // GxB_UnaryOp_xtype: return the type of x for z=f(x)
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2019, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -20,10 +20,10 @@ GrB_Info GxB_UnaryOp_xtype          // return the type of x
     // check inputs
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("GxB_UnaryOp_xtype (&xtype, unaryop)") ;
+    GB_WHERE1 ("GxB_UnaryOp_xtype (&xtype, unaryop)") ;
     GB_RETURN_IF_NULL (xtype) ;
     GB_RETURN_IF_NULL_OR_FAULTY (unaryop) ;
-    ASSERT_OK (GB_check (unaryop, "unaryop for xtype", GB0)) ;
+    ASSERT_UNARYOP_OK (unaryop, "unaryop for xtype", GB0) ;
 
     //--------------------------------------------------------------------------
     // return the xtype
