@@ -1,9 +1,9 @@
 //------------------------------------------------------------------------------
-// GB_AxB_dot4:  C+=A'*B via dot products, where C is dense
+// GB_AxB_dot4_template:  C+=A'*B via dot products, where C is dense
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -12,133 +12,289 @@
 // accum.  That is, no typecasting can be done with C.
 
 // The PAIR operator as the multiplier provides important special cases.
-// See Template/GB_AxB_dot_cij.c for details.
-
-// cij += A(k,i) * B(k,j)
-#undef  GB_DOT_MERGE
-#define GB_DOT_MERGE                                                \
-{                                                                   \
-    if (!cij_updated)                                               \
-    {                                                               \
-        cij_updated = true ;                                        \
-        GB_GETC (cij, pC) ;                                         \
-    }                                                               \
-    GB_GETA (aki, Ax, pA) ;         /* aki = A(k,i) */              \
-    GB_GETB (bkj, Bx, pB) ;         /* bkj = B(k,j) */              \
-    GB_MULTADD (cij, aki, bkj) ;    /* cij += aki * bkj */          \
-    GB_DOT_TERMINAL (cij) ;         /* break if cij == terminal */  \
-    pA++ ;                                                          \
-    pB++ ;                                                          \
-}
 
 {
 
-    //--------------------------------------------------------------------------
-    // get A, B, and C
-    //--------------------------------------------------------------------------
-
-    GB_CTYPE *GB_RESTRICT Cx = (GB_CTYPE *) C->x ;
-    const int64_t cvlen = C->vlen ;
-
-    const int64_t  *GB_RESTRICT Bp = B->p ;
-    const int64_t  *GB_RESTRICT Bh = B->h ;
-    const int64_t  *GB_RESTRICT Bi = B->i ;
-    const GB_BTYPE *GB_RESTRICT Bx = (GB_BTYPE *) (B_is_pattern ? NULL : B->x) ;
-    const int64_t bvlen = B->vlen ;
-
-    const int64_t  *GB_RESTRICT Ap = A->p ;
-    const int64_t  *GB_RESTRICT Ah = A->h ;
-    const int64_t  *GB_RESTRICT Ai = A->i ;
-    const GB_ATYPE *GB_RESTRICT Ax = (GB_ATYPE *) (A_is_pattern ? NULL : A->x) ;
-    ASSERT (A->vlen == B->vlen) ;
-
-    int ntasks = naslice * nbslice ;
-    bool cij_is_terminal ;
-
     //--------------------------------------------------------------------------
     // C += A'*B
     //--------------------------------------------------------------------------
 
-    int taskid ;
+    int tid ;
     #pragma omp parallel for num_threads(nthreads) schedule(dynamic,1)
-    for (taskid = 0 ; taskid < ntasks ; taskid++)
+    for (tid = 0 ; tid < ntasks ; tid++)
     {
 
         //----------------------------------------------------------------------
-        // get the entries in A and B to compute
+        // get the task descriptor
         //----------------------------------------------------------------------
 
-        int a_taskid = taskid / nbslice ;
-        int b_taskid = taskid % nbslice ;
-
-        int64_t akfirst = A_slice [a_taskid] ;
-        int64_t aklast  = A_slice [a_taskid+1] ;
-        if (akfirst >= aklast) continue ;
-
-        int64_t bkfirst = B_slice [b_taskid] ;
-        int64_t bklast  = B_slice [b_taskid+1] ;
-        if (bkfirst >= bklast) continue ;
+        const int a_tid = tid / nbslice ;
+        const int b_tid = tid % nbslice ;
+        const int64_t kA_start = A_slice [a_tid] ;
+        const int64_t kA_end   = A_slice [a_tid+1] ;
+        const int64_t kB_start = B_slice [b_tid] ;
+        const int64_t kB_end   = B_slice [b_tid+1] ;
 
         //----------------------------------------------------------------------
         // C+=A'*B via dot products
         //----------------------------------------------------------------------
 
-        for (int64_t bk = bkfirst ; bk < bklast ; bk++)
+        for (int64_t kB = kB_start ; kB < kB_end ; kB++)
         {
 
             //------------------------------------------------------------------
-            // get B(:,j)
+            // get B(:,j) and C(:,j)
             //------------------------------------------------------------------
 
-            int64_t j = (Bh == NULL) ? bk : Bh [bk] ;
-            int64_t pB_start = Bp [bk] ;
-            int64_t pB_end   = Bp [bk+1] ;
-            int64_t pC_start = j * cvlen ;
-            int64_t bjnz = pB_end - pB_start ;
-            if (bjnz == 0) continue ;
+            #if GB_B_IS_HYPER
+            const int64_t j = Bh [kB] ;
+            #else
+            const int64_t j = kB ;
+            #endif
 
-            if (bjnz == bvlen)
+            const int64_t pC_start = j * cvlen ;
+
+            #if ( GB_B_IS_HYPER || GB_B_IS_SPARSE )
+                // B is sparse or hyper
+                const int64_t pB_start = Bp [kB] ;
+                const int64_t pB_end = Bp [kB+1] ;
+                const int64_t bjnz = pB_end - pB_start ;
+                if (bjnz == 0) continue ;
+                #if ( GB_A_IS_HYPER || GB_A_IS_SPARSE )
+                    // Both A and B are sparse/hyper; get first & last in B(:,j)
+                    const int64_t ib_first = Bi [pB_start] ;
+                    const int64_t ib_last  = Bi [pB_end-1] ;
+                #endif
+            #else
+                // B is bitmap or full
+                const int64_t pB_start = j * vlen ;
+            #endif
+
+            //------------------------------------------------------------------
+            // C(:,j) += A'*B(:,j) where C is full
+            //------------------------------------------------------------------
+
+            for (int64_t kA = kA_start ; kA < kA_end ; kA++)
             {
 
                 //--------------------------------------------------------------
-                // B(:,j) is dense
+                // get A(:,i)
                 //--------------------------------------------------------------
 
-                for (int64_t ak = akfirst ; ak < aklast ; ak++)
+                #if GB_A_IS_HYPER
+                const int64_t i = Ah [kA] ;
+                #else
+                const int64_t i = kA ;
+                #endif
+
+                #if ( GB_A_IS_HYPER || GB_A_IS_SPARSE )
+                // A is sparse or hyper
+                int64_t pA = Ap [kA] ;
+                const int64_t pA_end = Ap [kA+1] ;
+                const int64_t ainz = pA_end - pA ;
+                if (ainz == 0) continue ;
+                #else
+                // A is bitmap or full
+                const int64_t pA = kA * vlen ;
+                #endif
+
+                //--------------------------------------------------------------
+                // get C(i,j)
+                //--------------------------------------------------------------
+
+                GB_CIJ_DECLARE (cij) ;          // declare the cij scalar
+                int64_t pC = i + pC_start ;     // C(i,j) is at Cx [pC]
+                bool cij_updated = false ;
+
+                //--------------------------------------------------------------
+                // C(i,j) += A (:,i)*B(:,j): a single dot product
+                //--------------------------------------------------------------
+
+                int64_t pB = pB_start ;
+
+                #if ( GB_A_IS_FULL && GB_B_IS_FULL )
                 {
 
                     //----------------------------------------------------------
-                    // get A(:,i)
+                    // both A and B are full
                     //----------------------------------------------------------
 
-                    int64_t i = (Ah == NULL) ? ak : Ah [ak] ;
-                    int64_t pA     = Ap [ak] ;
-                    int64_t pA_end = Ap [ak+1] ;
-                    int64_t ainz = pA_end - pA ;
-                    if (ainz == 0) continue ;
-
-                    GB_CIJ_DECLARE (cij) ;          // declare the cij scalar
-                    int64_t pC = i + pC_start ;     // C(i,j) is at Cx [pC]
-                    int64_t pB = pB_start ;
                     GB_GETC (cij, pC) ;             // cij = Cx [pC]
-
-                    //----------------------------------------------------------
-                    // special cases for the PAIR multiplier
-                    //----------------------------------------------------------
-
-                    // Since B(:,j) is dense, C(i,j) += A(:,i)'*B(:,j) is
-                    // trivial to compute with the PAIR multiplier.
-
                     #if GB_IS_PAIR_MULTIPLIER
+                    { 
+                        #if GB_IS_ANY_MONOID
+                        // ANY monoid: take the first entry found
+                        GB_MULT (cij, ignore, ignore, 0, 0, 0) ;
+                        #elif GB_IS_EQ_MONOID
+                        // EQ_PAIR semiring
+                        cij = (cij == 1) ;
+                        #elif (GB_CTYPE_BITS > 0)
+                        // PLUS, XOR monoids: A(:,i)'*B(:,j) is nnz(A(:,i)),
+                        // for bool, 8-bit, 16-bit, or 32-bit integer
+                        uint64_t t = ((uint64_t) cij) + vlen ;
+                        cij = (GB_CTYPE) (t & GB_CTYPE_BITS) ;
+                        #elif GB_IS_PLUS_FC32_MONOID
+                        // PLUS monoid for float complex
+                        cij = GxB_CMPLXF (crealf (cij) + (float) vlen, 0) ;
+                        #elif GB_IS_PLUS_FC64_MONOID
+                        // PLUS monoid for double complex
+                        cij = GxB_CMPLX (creal (cij) + (double) vlen, 0) ;
+                        #else
+                        // PLUS monoid for float, double, or 64-bit integers 
+                        cij += (GB_CTYPE) vlen ;
+                        #endif
+                    }
+                    #else
+                    {
+                        GB_PRAGMA_SIMD_DOT (cij)
+                        for (int64_t k = 0 ; k < vlen ; k++)
+                        { 
+                            GB_DOT_TERMINAL (cij) ;         // break if terminal
+                            // cij += A(k,i) * B(k,j)
+                            GB_GETA (aki, Ax, pA+k) ;       // aki = A(k,i)
+                            GB_GETB (bkj, Bx, pB+k) ;       // bkj = B(k,j)
+                            // cij += aki * bkj
+                            GB_MULTADD (cij, aki, bkj, i, k, j) ;
+                        }
+                    }
+                    #endif
+                    GB_DOT_ALWAYS_SAVE_CIJ ;
 
+                }
+                #elif ( GB_A_IS_FULL && GB_B_IS_BITMAP )
+                {
+
+                    //----------------------------------------------------------
+                    // A is full and B is bitmap
+                    //----------------------------------------------------------
+
+                    for (int64_t k = 0 ; k < vlen ; k++)
+                    {
+                        if (Bb [pB+k])
+                        { 
+                            GB_DOT (k, pA+k, pB+k) ;
+                        }
+                    }
+                    GB_DOT_SAVE_CIJ ;
+
+                }
+                #elif ( GB_A_IS_FULL && ( GB_B_IS_SPARSE || GB_B_IS_HYPER ) )
+                {
+
+                    //----------------------------------------------------------
+                    // A is full and B is sparse/hyper
+                    //----------------------------------------------------------
+
+                    GB_GETC (cij, pC) ;                 // cij = Cx [pC]
+                    #if GB_IS_PAIR_MULTIPLIER
+                    { 
                         #if GB_IS_ANY_MONOID
                         // ANY monoid: take the first entry found
                         // cij = 1, or CMPLX(1,0) for complex ANY
-                        GB_MULT (cij, ignore, ignore) ;
+                        GB_MULT (cij, ignore, ignore, 0, 0, 0) ;
                         #elif GB_IS_EQ_MONOID
-                        // A(:,i)'*B(:j) is one, so this result must be
-                        // accumulated into cij, as cij += 1, where the
-                        // accumulator is the EQ operator.
+                        // EQ_PAIR semiring
+                        cij = (cij == 1) ;
+                        #elif (GB_CTYPE_BITS > 0)
+                        // PLUS, XOR monoids: A(:,i)'*B(:,j) is nnz(A(:,i)),
+                        // for bool, 8-bit, 16-bit, or 32-bit integer
+                        uint64_t t = ((uint64_t) cij) + bjnz ;
+                        cij = (GB_CTYPE) (t & GB_CTYPE_BITS) ;
+                        #elif GB_IS_PLUS_FC32_MONOID
+                        // PLUS monoid for float complex
+                        cij = GxB_CMPLXF (crealf (cij) + (float) bjnz, 0) ;
+                        #elif GB_IS_PLUS_FC64_MONOID
+                        // PLUS monoid for double complex
+                        cij = GxB_CMPLX (creal (cij) + (double) bjnz, 0) ;
+                        #else
+                        // PLUS monoid for float, double, or 64-bit integers
+                        cij += (GB_CTYPE) bjnz ;
+                        #endif
+                    }
+                    #else
+                    {
+                        GB_PRAGMA_SIMD_DOT (cij)
+                        for (int64_t p = pB ; p < pB_end ; p++)
+                        { 
+                            GB_DOT_TERMINAL (cij) ;   // break if terminal
+                            int64_t k = Bi [p] ;
+                            // cij += A(k,i) * B(k,j)
+                            GB_GETA (aki, Ax, pA+k) ;     // aki = A(k,i)
+                            GB_GETB (bkj, Bx, p   ) ;     // bkj = B(k,j)
+                            GB_MULTADD (cij, aki, bkj, i, k, j) ;
+                        }
+                    }
+                    #endif
+                    GB_DOT_ALWAYS_SAVE_CIJ ;
+
+                }
+                #elif ( GB_A_IS_BITMAP && GB_B_IS_FULL )
+                {
+
+                    //----------------------------------------------------------
+                    // A is bitmap and B is full
+                    //----------------------------------------------------------
+
+                    for (int64_t k = 0 ; k < vlen ; k++)
+                    {
+                        if (Ab [pA+k])
+                        { 
+                            GB_DOT (k, pA+k, pB+k) ;
+                        }
+                    }
+                    GB_DOT_SAVE_CIJ ;
+
+                }
+                #elif ( GB_A_IS_BITMAP && GB_B_IS_BITMAP )
+                {
+
+                    //----------------------------------------------------------
+                    // both A and B are bitmap
+                    //----------------------------------------------------------
+
+                    for (int64_t k = 0 ; k < vlen ; k++)
+                    {
+                        if (Ab [pA+k] && Bb [pB+k])
+                        { 
+                            GB_DOT (k, pA+k, pB+k) ;
+                        }
+                    }
+                    GB_DOT_SAVE_CIJ ;
+
+                }
+                #elif ( GB_A_IS_BITMAP && ( GB_B_IS_SPARSE || GB_B_IS_HYPER ) )
+                {
+
+                    //----------------------------------------------------------
+                    // A is bitmap and B is sparse/hyper
+                    //----------------------------------------------------------
+
+                    for (int64_t p = pB ; p < pB_end ; p++)
+                    {
+                        int64_t k = Bi [p] ;
+                        if (Ab [pA+k])
+                        { 
+                            GB_DOT (k, pA+k, p) ;
+                        }
+                    }
+                    GB_DOT_SAVE_CIJ ;
+
+                }
+                #elif ( (GB_A_IS_SPARSE || GB_A_IS_HYPER) && GB_B_IS_FULL )
+                {
+
+                    //----------------------------------------------------------
+                    // A is sparse/hyper and B is full
+                    //----------------------------------------------------------
+
+                    GB_GETC (cij, pC) ;             // cij = Cx [pC]
+                    #if GB_IS_PAIR_MULTIPLIER
+                    { 
+                        #if GB_IS_ANY_MONOID
+                        // ANY monoid: take the first entry found
+                        GB_MULT (cij, ignore, ignore, 0, 0, 0) ;
+                        #elif GB_IS_EQ_MONOID
+                        // EQ_PAIR semiring
                         cij = (cij == 1) ;
                         #elif (GB_CTYPE_BITS > 0)
                         // PLUS, XOR monoids: A(:,i)'*B(:,j) is nnz(A(:,i)),
@@ -155,38 +311,9 @@
                         // PLUS monoid for float, double, or 64-bit integers 
                         cij += (GB_CTYPE) ainz ;
                         #endif
-
-                    #else
-
-                    //----------------------------------------------------------
-                    // general case
-                    //----------------------------------------------------------
-
-                    if (ainz == bvlen)
-                    {
-
-                        //------------------------------------------------------
-                        // both A(:,i) and B(:,j) are dense
-                        //------------------------------------------------------
-
-                        GB_PRAGMA_SIMD_DOT (cij)
-                        for (int64_t k = 0 ; k < bvlen ; k++)
-                        { 
-                            GB_DOT_TERMINAL (cij) ;         // break if terminal
-                            // cij += A(k,i) * B(k,j)
-                            GB_GETA (aki, Ax, pA+k) ;       // aki = A(k,i)
-                            GB_GETB (bkj, Bx, pB+k) ;       // bkj = B(k,j)
-                            GB_MULTADD (cij, aki, bkj) ;    // cij += aki * bkj
-                        }
-
                     }
-                    else
+                    #else
                     {
-
-                        //------------------------------------------------------
-                        // A(:,i) is sparse and B(:,j) is dense
-                        //------------------------------------------------------
-
                         GB_PRAGMA_SIMD_DOT (cij)
                         for (int64_t p = pA ; p < pA_end ; p++)
                         { 
@@ -195,102 +322,45 @@
                             // cij += A(k,i) * B(k,j)
                             GB_GETA (aki, Ax, p   ) ;       // aki = A(k,i)
                             GB_GETB (bkj, Bx, pB+k) ;       // bkj = B(k,j)
-                            GB_MULTADD (cij, aki, bkj) ;    // cij += aki * bkj
+                            GB_MULTADD (cij, aki, bkj, i, k, j) ;
                         }
                     }
-
                     #endif
-                    GB_PUTC (cij, pC) ;                 // Cx [pC] = cij
+                    GB_DOT_ALWAYS_SAVE_CIJ ;
+
                 }
-
-            }
-            else
-            {
-
-                //--------------------------------------------------------------
-                // B(:,j) is sparse
-                //--------------------------------------------------------------
-
-                // get the first and last index in B(:,j)
-                int64_t ib_first = Bi [pB_start] ;
-                int64_t ib_last  = Bi [pB_end-1] ;
-
-                for (int64_t ak = akfirst ; ak < aklast ; ak++)
+                #elif ( (GB_A_IS_SPARSE || GB_A_IS_HYPER) && GB_B_IS_BITMAP )
                 {
 
                     //----------------------------------------------------------
-                    // get A(:,i)
+                    // A is sparse/hyper and B is bitmap
                     //----------------------------------------------------------
 
-                    int64_t i = (Ah == NULL) ? ak : Ah [ak] ;
-                    int64_t pA     = Ap [ak] ;
-                    int64_t pA_end = Ap [ak+1] ;
-                    int64_t ainz = pA_end - pA ;
-                    if (ainz == 0) continue ;
-                    // get the first and last index in A(:,i)
-                    if (Ai [pA_end-1] < ib_first || ib_last < Ai [pA]) continue;
-
-                    //----------------------------------------------------------
-                    // C(i,j) += A(:,i)'*B(:,j)
-                    //----------------------------------------------------------
-
-                    GB_CIJ_DECLARE (cij) ;          // declare the cij scalar
-                    int64_t pC = i + pC_start ;     // C(i,j) is at Cx [pC]
-                    int64_t pB = pB_start ;
-
-                    if (ainz == bvlen)
+                    for (int64_t p = pA ; p < pA_end ; p++)
                     {
+                        int64_t k = Ai [p] ;
+                        if (Bb [pB+k])
+                        { 
+                            GB_DOT (k, p, pB+k) ;
+                        }
+                    }
+                    GB_DOT_SAVE_CIJ ;
+
+                }
+                #else
+                {
+
+                    //----------------------------------------------------------
+                    // both A and B are sparse/hyper
+                    //----------------------------------------------------------
+
+                    if (Ai [pA_end-1] < ib_first || ib_last < Ai [pA])
+                    { 
 
                         //------------------------------------------------------
-                        // A(:,i) is dense and B(:,j) is sparse
+                        // pattern of A(:,i) and B(:,j) don't overlap
                         //------------------------------------------------------
 
-                        GB_GETC (cij, pC) ;                 // cij = Cx [pC]
-
-                        #if GB_IS_PAIR_MULTIPLIER
-
-                            #if GB_IS_ANY_MONOID
-                            // ANY monoid: take the first entry found
-                            // cij = 1, or CMPLX(1,0) for complex ANY
-                            GB_MULT (cij, ignore, ignore) ;
-                            #elif GB_IS_EQ_MONOID
-                            // A(:,i)'*B(:j) is one, so this result must be
-                            // accumulated into cij, as cij += 1, where the
-                            // accumulator is the EQ operator.
-                            cij = (cij == 1) ;
-                            #elif (GB_CTYPE_BITS > 0)
-                            // PLUS, XOR monoids: A(:,i)'*B(:,j) is nnz(A(:,i)),
-                            // for bool, 8-bit, 16-bit, or 32-bit integer
-                            uint64_t t = ((uint64_t) cij) + bjnz ;
-                            cij = (GB_CTYPE) (t & GB_CTYPE_BITS) ;
-                            #elif GB_IS_PLUS_FC32_MONOID
-                            // PLUS monoid for float complex
-                            cij = GxB_CMPLXF (crealf (cij) + (float) bjnz, 0) ;
-                            #elif GB_IS_PLUS_FC64_MONOID
-                            // PLUS monoid for double complex
-                            cij = GxB_CMPLX (creal (cij) + (double) bjnz, 0) ;
-                            #else
-                            // PLUS monoid for float, double, or 64-bit integers
-                            cij += (GB_CTYPE) bjnz ;
-                            #endif
-
-                        #else
-
-                            GB_PRAGMA_SIMD_DOT (cij)
-                            for (int64_t p = pB ; p < pB_end ; p++)
-                            { 
-                                GB_DOT_TERMINAL (cij) ;   // break if terminal
-                                int64_t k = Bi [p] ;
-                                // cij += A(k,i) * B(k,j)
-                                GB_GETA (aki, Ax, pA+k) ;     // aki = A(k,i)
-                                GB_GETB (bkj, Bx, p   ) ;     // bkj = B(k,j)
-                                GB_MULTADD (cij, aki, bkj) ;  // cij += aki*bkj
-                            }
-
-                        #endif
-
-                        GB_PUTC (cij, pC) ;                 // Cx [pC] = cij
-
                     }
                     else if (ainz > 8 * bjnz)
                     {
@@ -299,7 +369,6 @@
                         // B(:,j) is very sparse compared to A(:,i)
                         //------------------------------------------------------
 
-                        bool cij_updated = false ;
                         while (pA < pA_end && pB < pB_end)
                         {
                             int64_t ia = Ai [pA] ;
@@ -322,10 +391,12 @@
                             else // ia == ib == k
                             { 
                                 // A(k,i) and B(k,j) are next entries to merge
-                                GB_DOT_MERGE ;
+                                GB_DOT (ia, pA, pB) ;
+                                pA++ ;
+                                pB++ ;
                             }
                         }
-                        if (cij_updated) GB_PUTC (cij, pC) ;
+                        GB_DOT_SAVE_CIJ ;
 
                     }
                     else if (bjnz > 8 * ainz)
@@ -335,7 +406,6 @@
                         // A(:,i) is very sparse compared to B(:,j)
                         //------------------------------------------------------
 
-                        bool cij_updated = false ;
                         while (pA < pA_end && pB < pB_end)
                         {
                             int64_t ia = Ai [pA] ;
@@ -358,10 +428,12 @@
                             else // ia == ib == k
                             { 
                                 // A(k,i) and B(k,j) are next entries to merge
-                                GB_DOT_MERGE ;
+                                GB_DOT (ia, pA, pB) ;
+                                pA++ ;
+                                pB++ ;
                             }
                         }
-                        if (cij_updated) GB_PUTC (cij, pC) ;
+                        GB_DOT_SAVE_CIJ ;
 
                     }
                     else
@@ -371,7 +443,6 @@
                         // A(:,i) and B(:,j) have about the same sparsity
                         //------------------------------------------------------
 
-                        bool cij_updated = false ;
                         while (pA < pA_end && pB < pB_end)
                         {
                             int64_t ia = Ai [pA] ;
@@ -389,12 +460,15 @@
                             else // ia == ib == k
                             { 
                                 // A(k,i) and B(k,j) are the entries to merge
-                                GB_DOT_MERGE ;
+                                GB_DOT (ia, pA, pB) ;
+                                pA++ ;
+                                pB++ ;
                             }
                         }
-                        if (cij_updated) GB_PUTC (cij, pC) ;
+                        GB_DOT_SAVE_CIJ ;
                     }
                 }
+                #endif
             }
         }
     }
