@@ -2,24 +2,31 @@
 // GB_AxB_factory: switch factory for C=A*B
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
-// This is used by GB_AxB_saxpy3.c and GB_AxB_dot[234].c to create 1438
+// This is used by GB_AxB_saxpy3.c and GB_AxB_dot[234].c to create the
 // built-in versions of sparse matrix-matrix multiplication.  The #include'ing
 // file defines the GB_AxB_WORKER macro, and mult_opcode, add_opcode, xcode,
 // ycode, and zcode.
 
-// Two 2nd level switch factories are used:
+// Three 2nd level switch factories are used:
 
 //      GB_AxB_type_factory: handles all semirings where the multiply operator
 //          is TxT->T (as is the monoid).
 
 //      GB_AxB_compare_factory: handles all semirings where the multiply
-//          operator is TxT->bool (for the comparison operators, LT, GT, etc),
-//          and where the monoid is bool x bool -> bool.
+//          operator is TxT -> bool (for the comparison operators, LT, GT,
+//          etc), and where the monoid is bool x bool -> bool.
+
+//      GB_AxB_bitwise_factory: handles all semirings for bitwise operators.
+
+//      GxB_AxB_positional_factory: handles all semirings for positional
+//          multiply operators.  Those operators are of the for XxX -> int64,
+//          where "X" denotes any type.  No typecasting is needed from the
+//          types of A and B.
 
 // If the multiplicative operator is ANY, then it has already been renamed to
 // SECOND, prior to using this factory, since that is faster for the
@@ -44,7 +51,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50: (min,max,plus,times,any) for 10 non-boolean real
             // 5: (or,and,xor,eq,any) for boolean
             // 6: (plus,times,any) for 2 complex
-            #define GB_MULT_NAME _first
+            #define GB_MNAME _first
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -57,7 +64,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50: (min,max,plus,times,any) for 10 real non-boolean
             // 5: (or,and,xor,eq,any) for boolean
             // 6: (plus,times,any) for 2 complex
-            #define GB_MULT_NAME _second
+            #define GB_MNAME _second
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -73,7 +80,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // land_pair, lor_pair, max_pair, min_pair, times_pair, eq_pair
             // all become any_pair.
             #define GB_MULT_IS_PAIR_OPERATOR
-            #define GB_MULT_NAME _pair
+            #define GB_MNAME _pair
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             #undef  GB_MULT_IS_PAIR_OPERATOR
@@ -86,7 +93,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // MIN == TIMES == AND for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _min
+            #define GB_MNAME _min
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -97,7 +104,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // MAX == PLUS == OR for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _max
+            #define GB_MNAME _max
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -110,7 +117,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 6: (plus,times,any) for 2 complex types
             // MAX == PLUS == OR for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _plus
+            #define GB_MNAME _plus
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -124,7 +131,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 6: (plus,times,any) for 2 complex types
             // MINUS == RMINUS == NE == ISNE == XOR for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _minus
+            #define GB_MNAME _minus
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -137,7 +144,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // 6: (plus,times,any) for 2 complex types
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _rminus
+            #define GB_MNAME _rminus
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -150,7 +157,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // 6: (plus,times,any) for 2 complex types
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _times
+            #define GB_MNAME _times
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -164,7 +171,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 6: (plus,times,any) for 2 complex types
             // FIRST == DIV for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _div
+            #define GB_MNAME _div
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
@@ -178,11 +185,14 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 6: (plus,times,any) for 2 complex types
             // SECOND == RDIV for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _rdiv
+            #define GB_MNAME _rdiv
             #define GB_COMPLEX
             #include "GB_AxB_type_factory.c"
             break ;
 
+#if 0
+// in v5.0.1, the *_IS* semirings are completely disabled.
+
         //----------------------------------------------------------------------
         case GB_ISEQ_opcode    :    // z = (x == y)
         //----------------------------------------------------------------------
@@ -190,7 +200,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // ISEQ == EQ for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _iseq
+            #define GB_MNAME _iseq
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -201,7 +211,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // MINUS == RMINUS == NE == ISNE == XOR for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _isne
+            #define GB_MNAME _isne
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -212,7 +222,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // ISGT == GT for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _isgt
+            #define GB_MNAME _isgt
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -223,7 +233,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // ISLT == LT for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _islt
+            #define GB_MNAME _islt
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -234,7 +244,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // ISGE == GE for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _isge
+            #define GB_MNAME _isge
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -245,16 +255,17 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (min,max,plus,times,any) for 10 real non-boolean
             // ISLE == LE for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _isle
+            #define GB_MNAME _isle
             #include "GB_AxB_type_factory.c"
             break ;
+#endif
 
         //----------------------------------------------------------------------
         case GB_EQ_opcode      :    // z = (x == y)
         //----------------------------------------------------------------------
 
             // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _eq
+            #define GB_MNAME _eq
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -265,7 +276,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
             // 50 semirings: (and,or,xor,eq,any) * (10 real non-boolean types)
             // MINUS == RMINUS == NE == ISNE == XOR for boolean
             #define GB_NO_BOOLEAN
-            #define GB_MULT_NAME _ne
+            #define GB_MNAME _ne
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -274,7 +285,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _gt
+            #define GB_MNAME _gt
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -283,7 +294,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _lt
+            #define GB_MNAME _lt
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -292,7 +303,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _ge
+            #define GB_MNAME _ge
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -301,7 +312,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _le
+            #define GB_MNAME _le
             #include "GB_AxB_compare_factory.c"
             break ;
 
@@ -309,8 +320,11 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         case GB_LOR_opcode     :    // z = x || y
         //----------------------------------------------------------------------
 
-            // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _lor
+            // 14 semirings:
+            // 10 semirings: plus_lor for 10 real non-boolean types
+            // 4 semirings: (lor,land,eq,lxor) for boolean
+            #define GB_NO_MIN_MAX_ANY_TIMES_MONOIDS
+            #define GB_MNAME _lor
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -318,8 +332,8 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         case GB_LAND_opcode    :    // z = x && y
         //----------------------------------------------------------------------
 
-            // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _land
+            // 14 semirings: same as LOR
+            #define GB_MNAME _land
             #include "GB_AxB_type_factory.c"
             break ;
 
@@ -327,9 +341,10 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         case GB_LXOR_opcode    :    // z = x != y
         //----------------------------------------------------------------------
 
-            // 55 semirings: (and,or,xor,eq,any) * 11 types (all but complex)
-            #define GB_MULT_NAME _lxor
+            // 14 semirings: same as LOR
+            #define GB_MNAME _lxor
             #include "GB_AxB_type_factory.c"
+            #define GB_NO_MIN_MAX_TIMES_MONOIDS
             break ;
 
         //----------------------------------------------------------------------
@@ -337,7 +352,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 16 semirings: (bor,band,bxor,bxnor) * (uint8,16,32,64)
-            #define GB_MULT_NAME _bor
+            #define GB_MNAME _bor
             #include "GB_AxB_bitwise_factory.c"
             break ;
 
@@ -346,7 +361,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 16 semirings: (bor,band,bxor,bxnor) * (uint8,16,32,64)
-            #define GB_MULT_NAME _band
+            #define GB_MNAME _band
             #include "GB_AxB_bitwise_factory.c"
             break ;
 
@@ -355,7 +370,7 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 16 semirings: (bor,band,bxor,bxnor) * (uint8,16,32,64)
-            #define GB_MULT_NAME _bxor
+            #define GB_MNAME _bxor
             #include "GB_AxB_bitwise_factory.c"
             break ;
 
@@ -364,10 +379,68 @@ ASSERT (mult_opcode != GB_ANY_opcode) ;
         //----------------------------------------------------------------------
 
             // 16 semirings: (bor,band,bxor,bxnor) * (uint8,16,32,64)
-            #define GB_MULT_NAME _bxnor
+            #define GB_MNAME _bxnor
             #include "GB_AxB_bitwise_factory.c"
             break ;
 
+        //----------------------------------------------------------------------
+        case GB_FIRSTI_opcode   :   // z = first_i(A(i,k),y) == i
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            #define GB_MNAME _firsti
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
+        //----------------------------------------------------------------------
+        case GB_FIRSTI1_opcode  :   // z = first_i1(A(i,k),y) == i+1
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            #define GB_MNAME _firsti1
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
+        //----------------------------------------------------------------------
+        case GB_FIRSTJ_opcode   :   // z = first_j(A(i,k),y) == k
+        case GB_SECONDI_opcode  :   // z = second_i(x,B(k,j)) == k
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            // FIRSTJ and SECONDI are identical when used in a semiring
+            #define GB_MNAME _firstj
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
+        //----------------------------------------------------------------------
+        case GB_FIRSTJ1_opcode  :   // z = first_j1(A(i,k),y) == k+1
+        case GB_SECONDI1_opcode :   // z = second_i1(x,B(k,j)) == k+1
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            // FIRSTJ1 and SECONDI1 are identical when used in a semiring
+            #define GB_MNAME _firstj1
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
+        //----------------------------------------------------------------------
+        case GB_SECONDJ_opcode  :   // z = second_j(x,B(i,j)) == j
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            #define GB_MNAME _secondj
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
+        //----------------------------------------------------------------------
+        case GB_SECONDJ1_opcode :   // z = second_j1(x,B(i,j)) == j+1
+        //----------------------------------------------------------------------
+
+            // 10 semirings: (min,max,times,plus,any) * (int32,int64)
+            #define GB_MNAME _secondj1
+            #include "GB_AxB_positional_factory.c"
+            break ;
+
         default: ;
     }
 }
