@@ -2,12 +2,13 @@
 // GB_AxB_saxpy3_template.h: C=A*B, C<M>=A*B, or C<!M>=A*B via saxpy3 method
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
-// Definitions for GB_AxB_saxpy3_template.c
+// Definitions for GB_AxB_saxpy3_template.c.  These do not depend on the
+// sparsity of A and B.
 
 #ifndef GB_AXB_SAXPY3_TEMPLATE_H
 #define GB_AXB_SAXPY3_TEMPLATE_H
@@ -23,45 +24,70 @@
     int64_t mpleft = 0 ;                                        \
     int64_t mpright = mnvec-1 ;                                 \
     int64_t pM_start, pM_end ;                                  \
-    GB_lookup (M_is_hyper, Mh, Mp, &mpleft, mpright,            \
-        ((Bh == NULL) ? kk : Bh [kk]), &pM_start, &pM_end) ;    \
-    int64_t mjnz = pM_end - pM_start ;    /* nnz (M (:,j)) */
+    GB_lookup (M_is_hyper, Mh, Mp, mvlen, &mpleft, mpright,     \
+        GBH (Bh, kk), &pM_start, &pM_end) ;                     \
+    int64_t mjnz = pM_end - pM_start ;
 
 //------------------------------------------------------------------------------
-// GB_GET_M_j_RANGE: get the first and last indices in M(:,j)
+// GB_GET_M_j_RANGE
 //------------------------------------------------------------------------------
 
 #define GB_GET_M_j_RANGE(gamma)                                 \
-    int64_t im_first = -1, im_last = -1 ;                       \
-    if (mjnz > 0)                                               \
-    {                                                           \
-        im_first = Mi [pM_start] ;  /* get first M(:,j) */      \
-        im_last  = Mi [pM_end-1] ;  /* get last M(:,j) */       \
-    }                                                           \
     int64_t mjnz_much = mjnz * gamma
 
 //------------------------------------------------------------------------------
-// GB_SCATTER_M_j: scatter M(:,j) for a fine or coarse Gustavson task
+// GB_SCATTER_M_j_TYPE: scatter M(:,j) of the given type into Gus. workspace
 //------------------------------------------------------------------------------
 
 #define GB_SCATTER_M_j_TYPE(mask_t,pMstart,pMend,mark)                  \
 {                                                                       \
-    const mask_t *GB_RESTRICT Mxx = (mask_t *) Mx ;                     \
-    for (int64_t pM = pMstart ; pM < pMend ; pM++) /* scan M(:,j) */    \
+    const mask_t *restrict Mxx = (mask_t *) Mx ;                     \
+    if (M_is_bitmap)                                                    \
     {                                                                   \
-        if (Mxx [pM]) Hf [Mi [pM]] = mark ;   /* Hf [i] = M(i,j) */     \
+        /* M is bitmap */                                               \
+        for (int64_t pM = pMstart ; pM < pMend ; pM++)                  \
+        {                                                               \
+            /* if (M (i,j) == 1) mark Hf [i] */                         \
+            if (Mb [pM] && Mxx [pM]) Hf [GBI (Mi, pM, mvlen)] = mark ;  \
+        }                                                               \
+    }                                                                   \
+    else                                                                \
+    {                                                                   \
+        /* M is hyper, sparse, or full */                               \
+        for (int64_t pM = pMstart ; pM < pMend ; pM++)                  \
+        {                                                               \
+            /* if (M (i,j) == 1) mark Hf [i] */                         \
+            if (Mxx [pM]) Hf [GBI (Mi, pM, mvlen)] = mark ;             \
+        }                                                               \
     }                                                                   \
 }                                                                       \
 break ;
 
-// scatter M(:,j) for a coarse Gustavson task, C<M>=A*B or C<!M>=A*B
+//------------------------------------------------------------------------------
+// GB_SCATTER_M_j:  scatter M(:,j) into the Gustavson workpace
+//------------------------------------------------------------------------------
+
 #define GB_SCATTER_M_j(pMstart,pMend,mark)                                  \
     if (Mx == NULL)                                                         \
     {                                                                       \
-        /* mask is structural, not valued */                                \
-        for (int64_t pM = pMstart ; pM < pMend ; pM++)                      \
+        /* M is structural, not valued */                                   \
+        if (M_is_bitmap)                                                    \
         {                                                                   \
-            Hf [Mi [pM]] = mark ;   /* Hf [i] = M(i,j) */                   \
+            /* M is bitmap */                                               \
+            for (int64_t pM = pMstart ; pM < pMend ; pM++)                  \
+            {                                                               \
+                /* if (M (i,j) is present) mark Hf [i] */                   \
+                if (Mb [pM]) Hf [GBI (Mi, pM, mvlen)] = mark ;              \
+            }                                                               \
+        }                                                                   \
+        else                                                                \
+        {                                                                   \
+            /* M is hyper, sparse, or full */                               \
+            for (int64_t pM = pMstart ; pM < pMend ; pM++)                  \
+            {                                                               \
+                /* mark Hf [i] */                                           \
+                Hf [GBI (Mi, pM, mvlen)] = mark ;                           \
+            }                                                               \
         }                                                                   \
     }                                                                       \
     else                                                                    \
@@ -74,6 +100,20 @@ break ;
             case 2: GB_SCATTER_M_j_TYPE (uint16_t, pMstart, pMend, mark) ;  \
             case 4: GB_SCATTER_M_j_TYPE (uint32_t, pMstart, pMend, mark) ;  \
             case 8: GB_SCATTER_M_j_TYPE (uint64_t, pMstart, pMend, mark) ;  \
+            case 16:                                                        \
+            {                                                               \
+                const uint64_t *restrict Mxx = (uint64_t *) Mx ;         \
+                for (int64_t pM = pMstart ; pM < pMend ; pM++)              \
+                {                                                           \
+                    /* if (M (i,j) == 1) mark Hf [i] */                     \
+                    if (!GBB (Mb, pM)) continue ;                           \
+                    if (Mxx [2*pM] || Mxx [2*pM+1])                         \
+                    {                                                       \
+                        /* Hf [i] = M(i,j) */                               \
+                        Hf [GBI (Mi, pM, mvlen)] = mark ;                   \
+                    }                                                       \
+                }                                                           \
+            }                                                               \
         }                                                                   \
     }
 
@@ -83,11 +123,11 @@ break ;
 
 // hash M(:,j) into Hf and Hi for coarse hash task, C<M>=A*B or C<!M>=A*B
 #define GB_HASH_M_j                                                     \
-    for (int64_t pM = pM_start ; pM < pM_end ; pM++) /* scan M(:,j) */  \
+    for (int64_t pM = pM_start ; pM < pM_end ; pM++)                    \
     {                                                                   \
-        GB_GET_M_ij ;           /* get M(i,j) */                        \
+        GB_GET_M_ij (pM) ;      /* get M(i,j) */                        \
         if (!mij) continue ;    /* skip if M(i,j)=0 */                  \
-        int64_t i = Mi [pM] ;                                           \
+        int64_t i = GBI (Mi, pM, mvlen) ;                               \
         for (GB_HASH (i))       /* find i in hash */                    \
         {                                                               \
             if (Hf [hash] < mark)                                       \
@@ -100,60 +140,77 @@ break ;
     }
 
 //------------------------------------------------------------------------------
-// GB_GET_B_j: prepare to iterate over B(:,j)
+// GB_GET_T_FOR_SECONDJ: define t for SECONDJ and SECONDJ1 semirings
 //------------------------------------------------------------------------------
 
-// prepare to iterate over the vector B(:,j), the (kk)th vector in B,
-// where j == ((Bh == NULL) ? kk : Bh [kk]).  Note that j itself is never
-// needed; just kk.
-#define GB_GET_B_j                                                          \
+#if GB_IS_SECONDJ_MULTIPLIER
+    #define GB_GET_T_FOR_SECONDJ                            \
+        GB_CIJ_DECLARE (t) ;                                \
+        GB_MULT (t, ignore, ignore, ignore, ignore, j) ;
+#else
+    #define GB_GET_T_FOR_SECONDJ
+#endif
+
+//------------------------------------------------------------------------------
+// GB_GET_B_j_FOR_ALL_FORMATS: prepare to iterate over B(:,j)
+//------------------------------------------------------------------------------
+
+// prepare to iterate over the vector B(:,j), the (kk)th vector in B, where 
+// j == GBH (Bh, kk).  This macro works regardless of the sparsity of A and B.
+#define GB_GET_B_j_FOR_ALL_FORMATS(A_is_hyper,B_is_sparse,B_is_hyper)       \
     int64_t pleft = 0 ;                                                     \
     int64_t pright = anvec-1 ;                                              \
-    int64_t pB = Bp [kk] ;                                                  \
-    int64_t pB_end = Bp [kk+1] ;                                            \
+    int64_t j = (B_is_hyper) ? Bh [kk] : kk ;                               \
+    GB_GET_T_FOR_SECONDJ ;  /* t = j for SECONDJ, or j+1 for SECONDJ1 */    \
+    int64_t pB = (B_is_sparse || B_is_hyper) ? Bp [kk] : (kk * bvlen) ;     \
+    int64_t pB_end = (B_is_sparse || B_is_hyper) ? Bp [kk+1] : (pB+bvlen) ; \
     int64_t bjnz = pB_end - pB ;  /* nnz (B (:,j) */                        \
     /* FUTURE::: can skip if mjnz == 0 for C<M>=A*B tasks */                \
-    if (A_is_hyper && bjnz > 2)                                             \
+    if (A_is_hyper && (B_is_sparse || B_is_hyper) && bjnz > 2 && !B_jumbled)\
     {                                                                       \
         /* trim Ah [0..pright] to remove any entries past last B(:,j), */   \
-        /* to speed up GB_lookup in GB_GET_A_k. */                          \
-        GB_bracket_right (Bi [pB_end-1], Ah, 0, &pright) ;                  \
+        /* to speed up GB_lookup in GB_GET_A_k_FOR_ALL_FORMATS. */          \
+        /* This requires that B is not jumbled */                           \
+        GB_bracket_right (GBI (Bi, pB_end-1, bvlen), Ah, 0, &pright) ;      \
     }
 
 //------------------------------------------------------------------------------
 // GB_GET_B_kj: get the numeric value of B(k,j)
 //------------------------------------------------------------------------------
 
-#define GB_GET_B_kj \
-    GB_GETB (bkj, Bx, pB)       /* bkj = Bx [pB] */
+#if GB_IS_FIRSTJ_MULTIPLIER
+
+    // FIRSTJ or FIRSTJ1 multiplier
+    // t = aik * bkj = k or k+1
+    #define GB_GET_B_kj                                     \
+        GB_CIJ_DECLARE (t) ;                                \
+        GB_MULT (t, ignore, ignore, ignore, k, ignore)
+
+#else
+
+    #define GB_GET_B_kj \
+        GB_GETB (bkj, Bx, pB)       /* bkj = Bx [pB] */
+
+#endif
 
 //------------------------------------------------------------------------------
-// GB_GET_A_k: prepare to iterate over the vector A(:,k)
+// GB_GET_A_k_FOR_ALL_FORMATS: prepare to iterate over the vector A(:,k)
 //------------------------------------------------------------------------------
 
-#define GB_GET_A_k                                                          \
+#define GB_GET_A_k_FOR_ALL_FORMATS(A_is_hyper)                              \
+    if (B_jumbled) pleft = 0 ;  /* reuse pleft if B is not jumbled */       \
     int64_t pA_start, pA_end ;                                              \
-    GB_lookup (A_is_hyper, Ah, Ap, &pleft, pright, k, &pA_start, &pA_end) ; \
-    int64_t aknz = pA_end - pA_start ;    /* nnz (A (:,k)) */
-
-//------------------------------------------------------------------------------
-// GB_SKIP_IF_A_k_DISJOINT_WITH_M_j:  skip if A(:,k) and M(:,j) are disjoint
-//------------------------------------------------------------------------------
-
-// skip C(:,j)<M> += A(:,k)*B(k,j) if A(:,k) and M(:,j), for C<M>=A*B methods
-#define GB_SKIP_IF_A_k_DISJOINT_WITH_M_j                    \
-    if (aknz == 0) continue ;                               \
-    int64_t alo = Ai [pA_start] ;   /* get first A(:,k) */  \
-    int64_t ahi = Ai [pA_end-1] ;   /* get last A(:,k) */   \
-    if (ahi < im_first || alo > im_last) continue
+    GB_lookup (A_is_hyper, Ah, Ap, avlen, &pleft, pright, k,                \
+        &pA_start, &pA_end) ;                                               \
+    int64_t aknz = pA_end - pA_start
 
 //------------------------------------------------------------------------------
 // GB_GET_M_ij: get the numeric value of M(i,j)
 //------------------------------------------------------------------------------
 
-#define GB_GET_M_ij                                 \
+#define GB_GET_M_ij(pM)                             \
     /* get M(i,j), at Mi [pM] and Mx [pM] */        \
-    bool mij = GB_mcast (Mx, pM, msize)
+    bool mij = GBB (Mb, pM) && GB_mcast (Mx, pM, msize)
 
 //------------------------------------------------------------------------------
 // GB_MULT_A_ik_B_kj: declare t and compute t = A(i,k) * B(k,j)
@@ -167,84 +224,18 @@ break ;
     #define t (GB_CTYPE_CAST (1, 0))
     #define GB_MULT_A_ik_B_kj
 
-#else
+#elif ( GB_IS_FIRSTJ_MULTIPLIER || GB_IS_SECONDJ_MULTIPLIER )
 
-    // typical semiring
-    #define GB_MULT_A_ik_B_kj                                   \
-        GB_GETA (aik, Ax, pA) ;     /* aik = Ax [pA] ;  */      \
-        GB_CIJ_DECLARE (t) ;        /* ctype t ;        */      \
-        GB_MULT (t, aik, bkj)       /* t = aik * bkj ;  */
-
-#endif
-
-//------------------------------------------------------------------------------
-// GB_COMPUTE_DENSE_C_j: compute C(:,j)=A*B(:,j) when C(:,j) is completely dense
-//------------------------------------------------------------------------------
-
-#if GB_IS_ANY_PAIR_SEMIRING
-
-    // ANY_PAIR: result is purely symbolic; no numeric work to do
-    #define GB_COMPUTE_DENSE_C_j                                \
-        for (int64_t i = 0 ; i < cvlen ; i++)                   \
-        {                                                       \
-            Ci [pC + i] = i ;                                   \
-        }
+    // nothing to do; t = aik*bkj already defined in an outer loop
+    #define GB_MULT_A_ik_B_kj
 
 #else
 
     // typical semiring
-    #define GB_COMPUTE_DENSE_C_j                                    \
-        for (int64_t i = 0 ; i < cvlen ; i++)                       \
-        {                                                           \
-            Ci [pC + i] = i ;                                       \
-            GB_CIJ_WRITE (pC + i, GB_IDENTITY) ; /* C(i,j)=0 */     \
-        }                                                           \
-        for ( ; pB < pB_end ; pB++)     /* scan B(:,j) */           \
-        {                                                           \
-            int64_t k = Bi [pB] ;       /* get B(k,j) */            \
-            GB_GET_A_k ;                /* get A(:,k) */            \
-            if (aknz == 0) continue ;                               \
-            GB_GET_B_kj ;               /* bkj = B(k,j) */          \
-            /* FUTURE::: handle the case when A(:,k) is dense */    \
-            /* scan A(:,k) */                                       \
-            for (int64_t pA = pA_start ; pA < pA_end ; pA++)        \
-            {                                                       \
-                int64_t i = Ai [pA] ;    /* get A(i,k) */           \
-                GB_MULT_A_ik_B_kj ;      /* t = A(i,k)*B(k,j) */    \
-                GB_CIJ_UPDATE (pC + i, t) ; /* Cx [pC+i]+=t */      \
-            }                                                       \
-        }
-
-#endif
-
-//------------------------------------------------------------------------------
-// GB_COMPUTE_C_j_WHEN_NNZ_B_j_IS_ONE: compute C(:,j) when nnz(B(:,j)) == 1
-//------------------------------------------------------------------------------
-
-// C(:,j) = A(:,k)*B(k,j) when there is a single entry in B(:,j)
-#if GB_IS_ANY_PAIR_SEMIRING
-
-    // ANY_PAIR: result is purely symbolic; no numeric work to do
-    #define GB_COMPUTE_C_j_WHEN_NNZ_B_j_IS_ONE                      \
-        int64_t k = Bi [pB] ;       /* get B(k,j) */                \
-        GB_GET_A_k ;                /* get A(:,k) */                \
-        memcpy (Ci + pC, Ai + pA_start, aknz * sizeof (int64_t)) ;
-
-#else
-
-    // typical semiring
-    #define GB_COMPUTE_C_j_WHEN_NNZ_B_j_IS_ONE                      \
-        int64_t k = Bi [pB] ;       /* get B(k,j) */                \
-        GB_GET_A_k ;                /* get A(:,k) */                \
-        GB_GET_B_kj ;               /* bkj = B(k,j) */              \
-        /* scan A(:,k) */                                           \
-        for (int64_t pA = pA_start ; pA < pA_end ; pA++)            \
-        {                                                           \
-            int64_t i = Ai [pA] ;       /* get A(i,k) */            \
-            GB_MULT_A_ik_B_kj ;         /* t = A(i,k)*B(k,j) */     \
-            GB_CIJ_WRITE (pC, t) ;      /* Cx [pC] = t */           \
-            Ci [pC++] = i ;                                         \
-        }
+    #define GB_MULT_A_ik_B_kj                                       \
+        GB_GETA (aik, Ax, pA) ;         /* aik = Ax [pA] ;  */      \
+        GB_CIJ_DECLARE (t) ;            /* ctype t ;        */      \
+        GB_MULT (t, aik, bkj, i, k, j)  /* t = aik * bkj ;  */
 
 #endif
 
@@ -252,7 +243,9 @@ break ;
 // GB_GATHER_ALL_C_j: gather the values and pattern of C(:,j)
 //------------------------------------------------------------------------------
 
-// gather the pattern and values of C(:,j) for a coarse Gustavson task (no sort)
+// gather the pattern and values of C(:,j) for a coarse Gustavson task;
+// the pattern is not flagged as jumbled.
+
 #if GB_IS_ANY_PAIR_SEMIRING
 
     // ANY_PAIR: result is purely symbolic; no numeric work to do
@@ -281,23 +274,40 @@ break ;
 #endif
 
 //------------------------------------------------------------------------------
-// GB_SORT_AND_GATHER_C_j: sort the pattern of C(:,j) and gather values
+// GB_SORT_C_j_PATTERN: sort C(:,j) for a coarse task, or flag as jumbled
 //------------------------------------------------------------------------------
 
-// sort the pattern of C(:,j) then gather the values for a coarse Gustavson task
+// Only coarse tasks do the optional sort.  Fine hash tasks always leave C
+// jumbled.
+
+#define GB_SORT_C_j_PATTERN                                     \
+    if (do_sort)                                                \
+    {                                                           \
+        /* sort the pattern of C(:,j) (non-default) */          \
+        GB_qsort_1a (Ci + Cp [kk], cjnz) ;                      \
+    }                                                           \
+    else                                                        \
+    {                                                           \
+        /* lazy sort: C(:,j) is now jumbled (default) */        \
+        task_C_jumbled = true ;                                 \
+    }
+
+//------------------------------------------------------------------------------
+// GB_SORT_AND_GATHER_C_j: sort and gather C(:,j) for a coarse Gustavson task
+//------------------------------------------------------------------------------
+
+// gather the values of C(:,j) for a coarse Gustavson task
 #if GB_IS_ANY_PAIR_SEMIRING
 
-    // ANY_PAIR: result is purely symbolic; just sort the pattern
+    // ANY_PAIR: result is purely symbolic
     #define GB_SORT_AND_GATHER_C_j                              \
-        /* sort the pattern of C(:,j) */                        \
-        GB_qsort_1a (Ci + Cp [kk], cjnz) ;
+        GB_SORT_C_j_PATTERN ;
 
 #else
 
     // typical semiring
     #define GB_SORT_AND_GATHER_C_j                              \
-        /* sort the pattern of C(:,j) */                        \
-        GB_qsort_1a (Ci + Cp [kk], cjnz) ;                      \
+        GB_SORT_C_j_PATTERN ;                                   \
         /* gather the values into C(:,j) */                     \
         for (int64_t pC = Cp [kk] ; pC < Cp [kk+1] ; pC++)      \
         {                                                       \
@@ -308,29 +318,26 @@ break ;
 #endif
 
 //------------------------------------------------------------------------------
-// GB_SORT_AND_GATHER_HASHED_C_j: sort pattern, gather values, for coarse hash 
+// GB_SORT_AND_GATHER_HASHED_C_j: sort and gather C(:,j) for a coarse hash task
 //------------------------------------------------------------------------------
 
 #if GB_IS_ANY_PAIR_SEMIRING
 
-    // ANY_PAIR: result is purely symbolic; just sort the pattern
-    #define GB_SORT_AND_GATHER_HASHED_C_j(hash_mark,Hi_hash_equals_i)       \
-        /* sort the pattern of C(:,j) */                                    \
-        GB_qsort_1a (Ci + Cp [kk], cjnz) ;
+    // ANY_PAIR: result is purely symbolic
+    #define GB_SORT_AND_GATHER_HASHED_C_j(hash_mark)            \
+        GB_SORT_C_j_PATTERN ;
 
 #else
 
-    // sort the pattern of C(:,j) then gather the values for a coarse hash task
-    #define GB_SORT_AND_GATHER_HASHED_C_j(hash_mark,Hi_hash_equals_i)       \
-        /* sort the pattern of C(:,j) */                                    \
-        GB_qsort_1a (Ci + Cp [kk], cjnz) ;                                  \
+    // gather the values of C(:,j) for a coarse hash task
+    #define GB_SORT_AND_GATHER_HASHED_C_j(hash_mark)                        \
+        GB_SORT_C_j_PATTERN ;                                               \
         for (int64_t pC = Cp [kk] ; pC < Cp [kk+1] ; pC++)                  \
         {                                                                   \
             int64_t i = Ci [pC] ;                                           \
-            int64_t marked = (hash_mark) ;                                  \
             for (GB_HASH (i))           /* find i in hash table */          \
             {                                                               \
-                if (Hf [hash] == marked && (Hi_hash_equals_i))              \
+                if (Hf [hash] == (hash_mark) && (Hi [hash] == i))           \
                 {                                                           \
                     /* i found in the hash table */                         \
                     /* Cx [pC] = Hx [hash] ; */                             \
@@ -342,53 +349,6 @@ break ;
 
 #endif
 
-//------------------------------------------------------------------------------
-// GB_SCAN_M_j_OR_A_k: compute C(:,j) using linear scan or binary search
-//------------------------------------------------------------------------------
-
-// C(:,j)<M(:,j)>=A(:,k)*B(k,j) using one of two methods
-#define GB_SCAN_M_j_OR_A_k                                              \
-{                                                                       \
-    if (aknz > 256 && mjnz_much < aknz)                                 \
-    /* nnz(M(:,j)) much less than nnz(A(:,k)) */                        \
-    {                                                                   \
-        /* scan M(:,j), and do binary search for A(i,k) */              \
-        int64_t pA = pA_start ;                                         \
-        for (int64_t pM = pM_start ; pM < pM_end ; pM++)                \
-        {                                                               \
-            GB_GET_M_ij ;           /* get M(i,j) */                    \
-            if (!mij) continue ;    /* skip if M(i,j)=0 */              \
-            int64_t i = Mi [pM] ;                                       \
-            bool found ;            /* search for A(i,k) */             \
-            int64_t apright = pA_end - 1 ;                              \
-            GB_BINARY_SEARCH (i, Ai, pA, apright, found) ;              \
-            if (found)                                                  \
-            {                                                           \
-                /* C(i,j)<M(i,j)> += A(i,k) * B(k,j) for this method. */\
-                /* M(i,j) is now known to be equal to 1, so there are */\
-                /* cases in the GB_IKJ operation that can never */      \
-                /* occur.  This could be pruned from the GB_IKJ */      \
-                /* operation, but then this operation would differ */   \
-                /* from the GB_IKJ operation in the linear-time scan */ \
-                /* of A(:,j), below.  It's unlikely that pruning this */\
-                /* case would lead to much performance improvement. */  \
-                GB_IKJ ;                                                \
-            }                                                           \
-        }                                                               \
-    }                                                                   \
-    else                                                                \
-    {                                                                   \
-        /* scan A(:,k), and lookup M(i,j) */                            \
-        for (int64_t pA = pA_start ; pA < pA_end ; pA++)                \
-        {                                                               \
-            int64_t i = Ai [pA] ;    /* get A(i,k) */                   \
-            /* do C(i,j)<M(i,j)> += A(i,k) * B(k,j) for this method */  \
-            /* M(i,j) may be 0 or 1, as given in the hash table */      \
-            GB_IKJ ;                                                    \
-        }                                                               \
-    }                                                                   \
-}
-
 //------------------------------------------------------------------------------
 // GB_ATOMIC_UPDATE_HX:  Hx [i] += t
 //------------------------------------------------------------------------------
@@ -407,9 +367,67 @@ break ;
     // Hx [i] += t via atomic update
     //--------------------------------------------------------------------------
 
-    #if GB_IS_PLUS_FC32_MONOID
+    // for built-in MIN/MAX monoids only, on built-in types
+    #define GB_MINMAX(i,t,done)                                     \
+    {                                                               \
+        GB_CTYPE xold, xnew, *px = Hx + (i) ;                       \
+        do                                                          \
+        {                                                           \
+            /* xold = Hx [i] via atomic read */                     \
+            GB_ATOMIC_READ                                          \
+            xold = (*px) ;                                          \
+            /* done if xold <= t for MIN, or xold >= t for MAX, */  \
+            /* but not done if xold is NaN */                       \
+            if (done) break ;                                       \
+            xnew = t ;  /* t should be assigned; it is not NaN */   \
+        }                                                           \
+        while (!GB_ATOMIC_COMPARE_EXCHANGE (px, xold, xnew)) ;      \
+    }
 
-        // built-in PLUS_FC32 monoid
+    #if GB_IS_IMIN_MONOID
+
+        // built-in MIN monoids for signed and unsigned integers
+        #define GB_ATOMIC_UPDATE_HX(i,t)                            \
+            GB_MINMAX (i, t, xold <= t)
+
+    #elif GB_IS_IMAX_MONOID
+
+        // built-in MAX monoids for signed and unsigned integers
+        #define GB_ATOMIC_UPDATE_HX(i,t)                            \
+            GB_MINMAX (i, t, xold >= t)
+
+    #elif GB_IS_FMIN_MONOID
+
+        // built-in MIN monoids for float and double, with omitnan behavior.
+        // The update is skipped entirely if t is NaN.  Otherwise, if t is not
+        // NaN, xold is checked.  If xold is NaN, islessequal (xold, t) is
+        // always false, so the non-NaN t must be always be assigned to Hx [i].
+        // If both terms are not NaN, then islessequal (xold,t) is just the
+        // comparison xold <= t.  If that is true, there is no work to do and
+        // the loop breaks.  Otherwise, t is smaller than xold and so it must
+        // be assigned to Hx [i].
+        #define GB_ATOMIC_UPDATE_HX(i,t)                            \
+        {                                                           \
+            if (!isnan (t))                                         \
+            {                                                       \
+                GB_MINMAX (i, t, islessequal (xold, t)) ;           \
+            }                                                       \
+        }
+
+    #elif GB_IS_FMAX_MONOID
+
+        // built-in MAX monoids for float and double, with omitnan behavior.
+        #define GB_ATOMIC_UPDATE_HX(i,t)                            \
+        {                                                           \
+            if (!isnan (t))                                         \
+            {                                                       \
+                GB_MINMAX (i, t, isgreaterequal (xold, t)) ;        \
+            }                                                       \
+        }
+
+    #elif GB_IS_PLUS_FC32_MONOID
+
+        // built-in PLUS_FC32 monoid can be done as two independent atomics
         #define GB_ATOMIC_UPDATE_HX(i,t)                            \
             GB_ATOMIC_UPDATE                                        \
             Hx_real [2*(i)] += crealf (t) ;                         \
@@ -418,7 +436,7 @@ break ;
 
     #elif GB_IS_PLUS_FC64_MONOID
 
-        // built-in PLUS_FC64 monoid
+        // built-in PLUS_FC64 monoid can be done as two independent atomics
         #define GB_ATOMIC_UPDATE_HX(i,t)                            \
             GB_ATOMIC_UPDATE                                        \
             Hx_real [2*(i)] += creal (t) ;                          \
@@ -427,17 +445,19 @@ break ;
 
     #elif GB_HAS_OMP_ATOMIC
 
-        // built-in PLUS, TIMES, LOR, LAND, LXOR monoids can be
-        // implemented with an OpenMP pragma
+        // built-in PLUS and TIMES for integers and real, and boolean LOR,
+        // LAND, LXOR monoids can be implemented with an OpenMP pragma.
         #define GB_ATOMIC_UPDATE_HX(i,t)                            \
             GB_ATOMIC_UPDATE                                        \
             GB_HX_UPDATE (i, t)
 
     #else
 
-        // built-in MIN, MAX, and EQ monoids only, which cannot
-        // be implemented with an OpenMP pragma
+        // all other atomic monoids (EQ, XNOR) on boolean, signed and unsigned
+        // integers, float, and double (not used for single and double
+        // complex).
         #define GB_ATOMIC_UPDATE_HX(i,t)                            \
+        {                                                           \
             GB_CTYPE xold, xnew, *px = Hx + (i) ;                   \
             do                                                      \
             {                                                       \
@@ -447,7 +467,8 @@ break ;
                 /* xnew = xold + t */                               \
                 xnew = GB_ADD_FUNCTION (xold, t) ;                  \
             }                                                       \
-            while (!GB_ATOMIC_COMPARE_EXCHANGE (px, xold, xnew))
+            while (!GB_ATOMIC_COMPARE_EXCHANGE (px, xold, xnew)) ;  \
+        }
 
     #endif
 
@@ -457,13 +478,18 @@ break ;
     // Hx [i] += t can only be done inside the critical section
     //--------------------------------------------------------------------------
 
+    // all user-defined monoids go here, and all complex monoids (except PLUS)
     #define GB_ATOMIC_UPDATE_HX(i,t)    \
-        GB_PRAGMA (omp flush)           \
+        GB_OMP_FLUSH                    \
         GB_HX_UPDATE (i, t) ;           \
-        GB_PRAGMA (omp flush)
+        GB_OMP_FLUSH
 
 #endif
 
+#define GB_IS_MINMAX_MONOID \
+    (GB_IS_IMIN_MONOID || GB_IS_IMAX_MONOID ||  \
+     GB_IS_FMIN_MONOID || GB_IS_FMAX_MONOID)
+
 //------------------------------------------------------------------------------
 // GB_ATOMIC_WRITE_HX:  Hx [i] = t
 //------------------------------------------------------------------------------
@@ -476,6 +502,22 @@ break ;
 
     #define GB_ATOMIC_WRITE_HX(i,t)
 
+    //--------------------------------------------------------------------------
+    // ANY_PAIR: for the bitmap case only: Hx [i] = 1
+    //--------------------------------------------------------------------------
+
+    #if GB_IS_ANY_FC32_MONOID || GB_IS_ANY_FC64_MONOID
+        #define GB_ATOMIC_SET_HX_ONE(i)             \
+            GB_ATOMIC_WRITE                         \
+            Hx_real [2*(i)] = 1 ;                   \
+            GB_ATOMIC_WRITE                         \
+            Hx_imag [2*(i)] = 0 ;
+    #else
+        #define GB_ATOMIC_SET_HX_ONE(i)             \
+            GB_ATOMIC_WRITE                         \
+            Hx [i] = 1 ;
+    #endif
+
 #elif GB_HAS_ATOMIC
 
     //--------------------------------------------------------------------------
@@ -485,26 +527,26 @@ break ;
     #if GB_IS_PLUS_FC32_MONOID
 
         // built-in PLUS_FC32 monoid
-        #define GB_ATOMIC_WRITE_HX(i,t)                             \
-            GB_ATOMIC_WRITE                                         \
-            Hx_real [2*(i)] = crealf (t) ;                          \
-            GB_ATOMIC_WRITE                                         \
+        #define GB_ATOMIC_WRITE_HX(i,t)         \
+            GB_ATOMIC_WRITE                     \
+            Hx_real [2*(i)] = crealf (t) ;      \
+            GB_ATOMIC_WRITE                     \
             Hx_imag [2*(i)] = cimagf (t) ;
 
     #elif GB_IS_PLUS_FC64_MONOID
 
         // built-in PLUS_FC64 monoid
-        #define GB_ATOMIC_WRITE_HX(i,t)                             \
-            GB_ATOMIC_WRITE                                         \
-            Hx_real [2*(i)] = creal (t) ;                           \
-            GB_ATOMIC_WRITE                                         \
+        #define GB_ATOMIC_WRITE_HX(i,t)         \
+            GB_ATOMIC_WRITE                     \
+            Hx_real [2*(i)] = creal (t) ;       \
+            GB_ATOMIC_WRITE                     \
             Hx_imag [2*(i)] = cimag (t) ;
 
     #else
 
         // all other atomic monoids
-        #define GB_ATOMIC_WRITE_HX(i,t)                             \
-            GB_ATOMIC_WRITE                                         \
+        #define GB_ATOMIC_WRITE_HX(i,t)         \
+            GB_ATOMIC_WRITE                     \
             GB_HX_WRITE (i, t)
 
     #endif
@@ -515,50 +557,41 @@ break ;
     // Hx [i] = t via critical section
     //--------------------------------------------------------------------------
 
-    #define GB_ATOMIC_WRITE_HX(i,t)    \
-        GB_PRAGMA (omp flush)          \
-        GB_HX_WRITE (i, t) ;           \
-        GB_PRAGMA (omp flush)
+    #define GB_ATOMIC_WRITE_HX(i,t)             \
+        GB_OMP_FLUSH                            \
+        GB_HX_WRITE (i, t) ;                    \
+        GB_OMP_FLUSH
 
 #endif
 
 //------------------------------------------------------------------------------
-// hash
+// hash iteration
 //------------------------------------------------------------------------------
 
 // to iterate over the hash table, looking for index i:
-// for (GB_HASH (i)) { ... }
-#define GB_HASH(i) int64_t hash = GB_HASH_FUNCTION (i) ; ; GB_REHASH (hash,i)
+// 
+//      for (GB_HASH (i))
+//      {
+//          ...
+//      }
+//
+// which expands into the following, where f(i) is the GB_HASHF(i) hash
+// function:
+//
+//      for (int64_t hash = f(i) ; ; hash = (hash+1)&(hash_size-1))
+//      {
+//          ...
+//      }
+
+#define GB_HASH(i) \
+    int64_t hash = GB_HASHF (i) ; ; GB_REHASH (hash,i)
+
+//------------------------------------------------------------------------------
+// define macros for any sparsity of A and B
+//------------------------------------------------------------------------------
+
+#undef GB_META16
+#include "GB_meta16_definitions.h"
 
 #endif
 
-//------------------------------------------------------------------------------
-// free workspace
-//------------------------------------------------------------------------------
-
-#undef  GB_FREE_INITIAL_WORK
-#define GB_FREE_INITIAL_WORK ;
-
-#undef  GB_FREE_TASKLIST_AND_HASH_TABLES
-#define GB_FREE_TASKLIST_AND_HASH_TABLES                                    \
-{                                                                           \
-    GB_FREE (*(TaskList_handle)) ;                                          \
-    GB_FREE (Hi_all) ;                                                      \
-    GB_FREE (Hf_all) ;                                                      \
-    GB_FREE (Hx_all) ;                                                      \
-}
-
-#undef  GB_FREE_WORK
-#define GB_FREE_WORK                                                        \
-{                                                                           \
-    GB_FREE_INITIAL_WORK ;                                                  \
-    GB_FREE_TASKLIST_AND_HASH_TABLES ;                                      \
-}
-
-#undef  GB_FREE_ALL
-#define GB_FREE_ALL                                                         \
-{                                                                           \
-    GB_FREE_WORK ;                                                          \
-    GB_MATRIX_FREE (Chandle) ;                                              \
-}
-
