@@ -2,8 +2,8 @@
 // GB_dense_subassign_05d_template: C<M> = x where C is dense
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -13,27 +13,35 @@
     // get C and M
     //--------------------------------------------------------------------------
 
-    const int64_t *GB_RESTRICT Mp = M->p ;
-    const int64_t *GB_RESTRICT Mh = M->h ;
-    const int64_t *GB_RESTRICT Mi = M->i ;
-    const GB_void *GB_RESTRICT Mx = (GB_void *) (Mask_struct ? NULL : (M->x)) ;
-    const size_t msize = M->type->size ;
+    ASSERT (GB_JUMBLED_OK (M)) ;
 
-    GB_CTYPE *GB_RESTRICT Cx = (GB_CTYPE *) C->x ;
+    const int64_t *restrict Mp = M->p ;
+    const int8_t  *restrict Mb = M->b ;
+    const int64_t *restrict Mh = M->h ;
+    const int64_t *restrict Mi = M->i ;
+    const GB_void *restrict Mx = (GB_void *) (Mask_struct ? NULL : (M->x)) ;
+    const size_t msize = M->type->size ;
+    const size_t mvlen = M->vlen ;
+
+    GB_CTYPE *restrict Cx = (GB_CTYPE *) C->x ;
     const int64_t cvlen = C->vlen ;
 
+    const int64_t *restrict kfirst_Mslice = M_ek_slicing ;
+    const int64_t *restrict klast_Mslice  = M_ek_slicing + M_ntasks ;
+    const int64_t *restrict pstart_Mslice = M_ek_slicing + M_ntasks * 2 ;
+
     //--------------------------------------------------------------------------
     // C<M> = x
     //--------------------------------------------------------------------------
 
     int taskid ;
-    #pragma omp parallel for num_threads(nthreads) schedule(dynamic,1)
-    for (taskid = 0 ; taskid < ntasks ; taskid++)
+    #pragma omp parallel for num_threads(M_nthreads) schedule(dynamic,1)
+    for (taskid = 0 ; taskid < M_ntasks ; taskid++)
     {
 
         // if kfirst > klast then taskid does no work at all
-        int64_t kfirst = kfirst_slice [taskid] ;
-        int64_t klast  = klast_slice  [taskid] ;
+        int64_t kfirst = kfirst_Mslice [taskid] ;
+        int64_t klast  = klast_Mslice  [taskid] ;
 
         //----------------------------------------------------------------------
         // C<M(:,kfirst:klast)> = x
@@ -46,10 +54,10 @@
             // find the part of M(:,k) to be operated on by this task
             //------------------------------------------------------------------
 
-            int64_t j = (Mh == NULL) ? k : Mh [k] ;
+            int64_t j = GBH (Mh, k) ;
             int64_t pM_start, pM_end ;
-            GB_get_pA_and_pC (&pM_start, &pM_end, NULL,
-                taskid, k, kfirst, klast, pstart_slice, NULL, NULL, Mp) ;
+            GB_get_pA (&pM_start, &pM_end, taskid, k,
+                kfirst, klast, pstart_Mslice, Mp, mvlen) ;
 
             // pC points to the start of C(:,j) if C is dense
             int64_t pC = j * cvlen ;
@@ -58,12 +66,12 @@
             // C<M(:,j)> = x
             //------------------------------------------------------------------
 
-            if (Mx == NULL)
+            if (Mx == NULL && Mb == NULL)
             {
                 GB_PRAGMA_SIMD_VECTORIZE
                 for (int64_t pM = pM_start ; pM < pM_end ; pM++)
                 { 
-                    int64_t p = pC + Mi [pM] ;
+                    int64_t p = pC + GBI (Mi, pM, mvlen) ;
                     GB_COPY_SCALAR_TO_C (p, cwork) ;        // Cx [p] = scalar
                 }
             }
@@ -72,9 +80,9 @@
                 GB_PRAGMA_SIMD_VECTORIZE
                 for (int64_t pM = pM_start ; pM < pM_end ; pM++)
                 {
-                    if (GB_mcast (Mx, pM, msize))
+                    if (GBB (Mb, pM) && GB_mcast (Mx, pM, msize))
                     { 
-                        int64_t p = pC + Mi [pM] ;
+                        int64_t p = pC + GBI (Mi, pM, mvlen) ;
                         GB_COPY_SCALAR_TO_C (p, cwork) ;    // Cx [p] = scalar
                     }
                 }
