@@ -2,8 +2,8 @@
 // GB_dense_subassign_22_template: C += b where C is dense and b is a scalar
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -13,7 +13,7 @@
     // get C
     //--------------------------------------------------------------------------
 
-    GB_CTYPE *GB_RESTRICT Cx = (GB_CTYPE *) C->x ;
+    GB_CTYPE *restrict Cx = (GB_CTYPE *) C->x ;
     const int64_t cnz = GB_NNZ (C) ;
 
     //--------------------------------------------------------------------------
@@ -24,7 +24,7 @@
     #pragma omp parallel for num_threads(nthreads) schedule(static)
     for (pC = 0 ; pC < cnz ; pC++)
     { 
-        GB_BINOP (GB_CX (pC), GB_CX (pC), bwork) ;
+        GB_BINOP (GB_CX (pC), GB_CX (pC), bwork, 0, 0) ;
     }
 }
 
