@@ -2,8 +2,8 @@
 // GB_mex_assign_alias_mask: C<A> = A
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -13,10 +13,10 @@
 
 #define FREE_ALL                            \
 {                                           \
-    GB_MATRIX_FREE (&C) ;                   \
-    GB_MATRIX_FREE (&A) ;                   \
+    GrB_Matrix_free_(&C) ;                  \
+    GrB_Matrix_free_(&A) ;                  \
     GrB_Descriptor_free_(&desc) ;           \
-    GB_mx_put_global (true, 0) ;            \
+    GB_mx_put_global (true) ;               \
 }
 
 void mexFunction
@@ -33,29 +33,25 @@ void mexFunction
     GrB_Descriptor desc = NULL ;
 
     // check inputs
-    GB_WHERE (USAGE) ;
     if (nargout > 1 || nargin < 2 || nargin > 3)
     {
         mexErrMsgTxt ("Usage: " USAGE) ;
     }
 
-    // get C (make a deep copy)
-    #define GET_DEEP_COPY \
-        C = GB_mx_mxArray_to_Matrix (pargin [0], "C input", true, true) ;
-    #define FREE_DEEP_COPY GB_MATRIX_FREE (&C) ;
+    // get A C (make a deep copy)
+    #define GET_DEEP_COPY       \
+        C = GB_mx_mxArray_to_Matrix (pargin [0], "C input", true, true) ;   \
+        GxB_Matrix_Option_set (C, GxB_SPARSITY_CONTROL, C->sparsity) ;      \
+        A = GB_mx_mxArray_to_Matrix (pargin [1], "A input", true, true) ;   \
+        GxB_Matrix_Option_set (A, GxB_SPARSITY_CONTROL, A->sparsity) ;
+    #define FREE_DEEP_COPY      \
+        GrB_Matrix_free_(&C) ;  \
+        GrB_Matrix_free_(&A) ;
     GET_DEEP_COPY ;
-    if (C == NULL)
+    if (C == NULL || A == NULL)
     {
         FREE_ALL ;
-        mexErrMsgTxt ("C failed") ;
-    }
-
-    // get A (shallow copy)
-    A = GB_mx_mxArray_to_Matrix (pargin [1], "A input", false, true) ;
-    if (A == NULL)
-    {
-        FREE_ALL ;
-        mexErrMsgTxt ("A failed") ;
+        mexErrMsgTxt ("C or A failed") ;
     }
 
     // get desc
@@ -66,11 +62,11 @@ void mexFunction
     }
 
     GrB_Index nrows, ncols ;
-    GrB_Matrix_nvals (&nrows, C) ;
-    GrB_Matrix_nvals (&ncols, C) ;
+    GrB_Matrix_nrows (&nrows, C) ;
+    GrB_Matrix_ncols (&ncols, C) ;
 
     // C<A> = A
-    METHOD (GrB_Matrix_assign_(C, A, NULL, A,
+    METHOD (GxB_Matrix_subassign_(C, A, NULL, A,
         GrB_ALL, nrows, GrB_ALL, ncols, desc)) ;
 
     // return C to MATLAB as a struct and free the GraphBLAS C
