@@ -2,8 +2,8 @@
 // GB_mex_band: C = tril (triu (A,lo), hi), or with A'
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,13 +15,13 @@
 
 #define FREE_ALL                        \
 {                                       \
-    GB_SCALAR_FREE (&Thunk) ;           \
-    GB_MATRIX_FREE (&C) ;               \
-    GB_MATRIX_FREE (&A) ;               \
+    GxB_Scalar_free_(&Thunk) ;          \
+    GrB_Matrix_free_(&C) ;              \
+    GrB_Matrix_free_(&A) ;              \
     GxB_Scalar_free_(&Thunk_type) ;     \
     GxB_SelectOp_free_(&op) ;           \
     GrB_Descriptor_free_(&desc) ;       \
-    GB_mx_put_global (true, 0) ;        \
+    GB_mx_put_global (true) ;           \
 }
 
 #define OK(method)                                      \
@@ -30,7 +30,6 @@
     if (info != GrB_SUCCESS)                            \
     {                                                   \
         FREE_ALL ;                                      \
-        printf ("%s\n", GrB_error ()) ;                 \
         mexErrMsgTxt ("GraphBLAS failed") ;             \
     }                                                   \
 }
@@ -41,11 +40,11 @@ typedef struct
     int64_t hi ;
 } LoHi_type ; 
 
-bool LoHi_band (GrB_Index i, GrB_Index j, GrB_Index nrows,
-    GrB_Index ncols, /* x is unused: */ const void *x, const LoHi_type *thunk) ;
+bool LoHi_band (GrB_Index i, GrB_Index j,
+    /* x is unused: */ const void *x, const LoHi_type *thunk) ;
 
-bool LoHi_band (GrB_Index i, GrB_Index j, GrB_Index nrows,
-    GrB_Index ncols, /* x is unused: */ const void *x, const LoHi_type *thunk)
+bool LoHi_band (GrB_Index i, GrB_Index j,
+    /* x is unused: */ const void *x, const LoHi_type *thunk)
 {
     int64_t i2 = (int64_t) i ;
     int64_t j2 = (int64_t) j ;
@@ -76,7 +75,6 @@ void mexFunction
     #define FREE_DEEP_COPY ;
 
     // check inputs
-    GB_WHERE (USAGE) ;
     if (nargout > 1 || nargin < 3 || nargin > 4)
     {
         mexErrMsgTxt ("Usage: " USAGE) ;
@@ -100,8 +98,7 @@ void mexFunction
 
     OK (GxB_Scalar_new (&Thunk, Thunk_type)) ;
     OK (GxB_Scalar_setElement_UDT (Thunk, (void *) &bandwidth)) ;
-    GrB_Index ignore ;
-    OK (GxB_Scalar_nvals (&ignore, Thunk)) ;
+    OK (GxB_Scalar_wait_(&Thunk)) ;
 
     // get atranspose
     bool atranspose = false ;
