@@ -2,8 +2,8 @@
 // GB_mex_ewise_alias4: C<M> = M+M
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -13,10 +13,10 @@
 
 #define FREE_ALL                            \
 {                                           \
-    GB_MATRIX_FREE (&M) ;                   \
-    GB_MATRIX_FREE (&C) ;                   \
+    GrB_Matrix_free_(&M) ;                  \
+    GrB_Matrix_free_(&C) ;                  \
     GrB_Descriptor_free_(&desc) ;           \
-    GB_mx_put_global (true, 0) ;            \
+    GB_mx_put_global (true) ;               \
 }
 
 void mexFunction
@@ -33,7 +33,6 @@ void mexFunction
     GrB_Descriptor desc = NULL ;
 
     // check inputs
-    GB_WHERE (USAGE) ;
     if (nargout > 1 || nargin < 3 || nargin > 4)
     {
         mexErrMsgTxt ("Usage: " USAGE) ;
@@ -42,7 +41,7 @@ void mexFunction
     // get C (make a deep copy)
     #define GET_DEEP_COPY \
     C = GB_mx_mxArray_to_Matrix (pargin [0], "C input", true, true) ;
-    #define FREE_DEEP_COPY GB_MATRIX_FREE (&C) ;
+    #define FREE_DEEP_COPY GrB_Matrix_free_(&C) ;
     GET_DEEP_COPY ;
     if (C == NULL)
     {
