@@ -2,8 +2,8 @@
 // GB_mex_ipagerank: compute pagerank with an integer semiring
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -17,8 +17,8 @@
 #define FREE_ALL                        \
 {                                       \
     if (P != NULL) mxFree (P) ;         \
-    GB_MATRIX_FREE (&A) ;               \
-    GB_mx_put_global (true, 0) ;        \
+    GrB_Matrix_free_(&A) ;              \
+    GB_mx_put_global (true) ;           \
 }
 
 void mexFunction
@@ -36,8 +36,6 @@ void mexFunction
     GrB_Index n = 0 ;
     bool malloc_debug = GB_mx_get_global (true) ;
 
-    GB_WHERE (USAGE) ;
-
     // check inputs
     if (nargout > 2 || nargin != 1)
     {
