@@ -2,8 +2,8 @@
 // GB_mex_rdiv2: compute C=A*B with the rdiv2 operator
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -18,32 +18,33 @@
 
 #define FREE_ALL                            \
 {                                           \
-    GB_MATRIX_FREE (&A) ;                   \
-    GB_MATRIX_FREE (&B) ;                   \
-    GB_MATRIX_FREE (&B64) ;                 \
-    GB_MATRIX_FREE (&C) ;                   \
-    GB_MATRIX_FREE (&T) ;                   \
+    GrB_Matrix_free_(&A) ;                  \
+    GrB_Matrix_free_(&B) ;                  \
+    GrB_Matrix_free_(&B64) ;                \
+    GrB_Matrix_free_(&C) ;                  \
+    GrB_Matrix_free_(&T) ;                  \
     GrB_BinaryOp_free_(&My_rdiv2) ;         \
     GrB_Semiring_free_(&My_plus_rdiv2) ;    \
-    GB_mx_put_global (true, 0) ;            \
+    GB_mx_put_global (true) ;               \
 }
 
 //------------------------------------------------------------------------------
 
 GrB_Info info ;
 bool malloc_debug = false ;
-bool ignore = false, ignore2 = false ;
+bool ignore = false, ignore1 = false, ignore2 = false ;
 bool atranspose = false ;
 bool btranspose = false ;
-GrB_Matrix A = NULL, B = NULL, B64 = NULL, C = NULL, T = NULL ;
+GrB_Matrix A = NULL, B = NULL, B64 = NULL, C = NULL, T = NULL, MT = NULL ;
 int64_t anrows = 0 ;
 int64_t ancols = 0 ;
 int64_t bnrows = 0 ;
 int64_t bncols = 0 ;
-GrB_Desc_Value AxB_method = GxB_DEFAULT, AxB_method_used ;
+GrB_Desc_Value AxB_method = GxB_DEFAULT ;
 bool flipxy = false ;
 bool done_in_place = false ;
 double C_scalar = 0 ;
+struct GB_Matrix_opaque MT_header, T_header ;
 
 GrB_Info axb (GB_Context Context) ;
 
@@ -98,13 +99,15 @@ GrB_Info axb (GB_Context Context)
         }
     }
 
+    MT = GB_clear_static_header (&MT_header) ;
+    T  = GB_clear_static_header (&T_header) ;
+
     // C = A*B or C += A*B
-    info = GB_AxB_meta (
-        &T,
-        C,
+    info = GB_AxB_meta (T, C,  // can be done in place if C != NULL
         false,      // C_replace
         true,       // CSC
-        NULL,       // no MT returned
+        MT,         // no MT returned
+        &ignore1,   // M_transposed will be false
         NULL,       // no Mask
         false,      // mask not complemented
         false,      // mask not structural
@@ -116,28 +119,30 @@ GrB_Info axb (GB_Context Context)
         flipxy,
         &ignore,    // mask_applied
         &done_in_place,
-        AxB_method, &AxB_method_used, Context) ;
+        AxB_method,
+        true,       // do the sort
+        Context) ;
 
     if (info == GrB_SUCCESS)
     {
         if (done_in_place != do_in_place)
         {
-            printf ("done in place: %d %d\n", do_in_place, done_in_place) ;
             mexErrMsgTxt ("failure: not in place as expected\n") ;
         }
         if (!done_in_place)
         {
             GrB_Matrix_free_(&C) ;
-            C = T ;
-            T = NULL ;
+            info = GrB_Matrix_dup (&C, T) ;
         }
     }
-    else
+
+    if (info != GrB_SUCCESS)
     {
         GrB_Matrix_free_(&C) ;
-        GrB_Matrix_free_(&T) ;
     }
 
+    GrB_Matrix_free_(&T) ;
+
     GrB_BinaryOp_free_(&My_rdiv2) ;
     GrB_Semiring_free_(&My_plus_rdiv2) ;
 
@@ -158,6 +163,8 @@ void mexFunction
     info = GrB_SUCCESS ;
     malloc_debug = GB_mx_get_global (true) ;
     ignore = false ;
+    ignore1 = false ;
+    ignore2 = false ;
     A = NULL ;
     B = NULL ;
     B64 = NULL ;
@@ -166,7 +173,7 @@ void mexFunction
     My_rdiv2 = NULL ;
     My_plus_rdiv2 = NULL ;
 
-    GB_WHERE (USAGE) ;
+    GB_CONTEXT (USAGE) ;
 
     // check inputs
     if (nargout > 1 || nargin < 2 || nargin > 7)
@@ -200,7 +207,6 @@ void mexFunction
     // get the axb_method
     // 0 or not present: default
     // 1001: Gustavson
-    // 1002: heap
     // 1003: dot
     // 1004: hash
     // 1005: saxpy
@@ -208,7 +214,6 @@ void mexFunction
 
     if (! ((AxB_method == GxB_DEFAULT) ||
         (AxB_method == GxB_AxB_GUSTAVSON) ||
-        (AxB_method == GxB_AxB_HEAP) ||
         (AxB_method == GxB_AxB_HASH) ||
         (AxB_method == GxB_AxB_SAXPY) ||
         (AxB_method == GxB_AxB_DOT)))
@@ -221,7 +226,6 @@ void mexFunction
 
     // get the C_scalar
     GET_SCALAR (6, double, C_scalar, 0) ;
-    // printf ("C scalar: %g\n", C_scalar) ;
 
     // determine the dimensions
     anrows = (atranspose) ? GB_NCOLS (A) : GB_NROWS (A) ;
@@ -236,7 +240,6 @@ void mexFunction
 
     if (atranspose && btranspose && C_scalar != 0)
     {
-        printf ("C=A'*B'; ignoring C_scalar!\n") ;
         C_scalar = 0 ;
     }
 
@@ -245,8 +248,7 @@ void mexFunction
     GrB_Matrix_assign_(B, NULL, NULL, B64, GrB_ALL, 0, GrB_ALL, 0, NULL) ;
 
     // B must be completed
-    GrB_Index nvals ;
-    GrB_Matrix_nvals (&nvals, B) ;
+    GrB_Matrix_wait (&B) ;
 
     METHOD (axb (Context)) ;
 
