@@ -2,8 +2,8 @@
 // GB_mex_subref_symbolic: S=A(I,J)
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -13,9 +13,9 @@
 
 #define FREE_ALL                        \
 {                                       \
-    GB_MATRIX_FREE (&A) ;               \
-    GB_MATRIX_FREE (&C) ;               \
-    GB_mx_put_global (true, 0) ;        \
+    GrB_Matrix_free_(&A) ;              \
+    GrB_Matrix_free_(&C) ;              \
+    GB_mx_put_global (true) ;           \
 }
 
 void mexFunction
@@ -26,16 +26,17 @@ void mexFunction
     const mxArray *pargin [ ]
 )
 {
+    struct GB_Matrix_opaque C_header ;
+    GrB_Matrix C = GB_clear_static_header (&C_header) ;
 
     bool malloc_debug = GB_mx_get_global (true) ;
     GrB_Matrix A = NULL ;
-    GrB_Matrix C = NULL ;
     GrB_Index *I = NULL, ni = 0, I_range [3] ;
     GrB_Index *J = NULL, nj = 0, J_range [3] ;
     bool ignore ;
 
     // check inputs
-    GB_WHERE (USAGE) ;
+    GB_CONTEXT (USAGE) ;
     if (nargout > 1 || nargin != 3)
     {
         mexErrMsgTxt ("Usage: " USAGE) ;
@@ -66,8 +67,16 @@ void mexFunction
         mexErrMsgTxt ("J failed") ;
     }
 
-    // C = A(I,J) or A(J,I)', no need to check dimensions of C
-    METHOD (GB_subref (&C, true , A, I, ni, J, nj, true, true, Context)) ;
+    // symbolic subref is not needed when A is bitmap.
+    int sparsity = 0 ;
+    GxB_Matrix_Option_get_(A, GxB_SPARSITY_STATUS, &sparsity) ;
+    if (sparsity == GxB_BITMAP)
+    {
+        mexErrMsgTxt ("A failed: cannot be bitmap") ;
+    }
+
+    // C = A(I,J) or A(J,I)', no need to check dimensions of C; symbolic
+    METHOD (GB_subref (C, true , A, I, ni, J, nj, true, Context)) ;
 
     // return C to MATLAB as a struct
     pargout [0] = GB_mx_Matrix_to_mxArray (&C, "C subref symbolic", true) ;
