@@ -2,8 +2,8 @@
 // GB_mex_vxm: w'<mask> = accum(w',u'A)
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -15,7 +15,7 @@
 {                                                   \
     GrB_Vector_free_(&w) ;                          \
     GrB_Vector_free_(&u) ;                          \
-    GB_MATRIX_FREE (&A) ;                           \
+    GrB_Matrix_free_(&A) ;                          \
     GrB_Vector_free_(&mask) ;                       \
     if (semiring != Complex_plus_times)             \
     {                                               \
@@ -26,7 +26,7 @@
         GrB_Semiring_free_(&semiring) ;             \
     }                                               \
     GrB_Descriptor_free_(&desc) ;                   \
-    GB_mx_put_global (true, AxB_method_used) ;      \
+    GB_mx_put_global (true) ;                       \
 }
 
 void mexFunction
@@ -45,10 +45,8 @@ void mexFunction
     GrB_Vector mask = NULL ;
     GrB_Semiring semiring = NULL ;
     GrB_Descriptor desc = NULL ;
-    GrB_Desc_Value AxB_method_used = GxB_DEFAULT ;
 
     // check inputs
-    GB_WHERE (USAGE) ;
     if (nargout > 1 || nargin < 6 || nargin > 7)
     {
         mexErrMsgTxt ("Usage: " USAGE) ;
@@ -118,8 +116,6 @@ void mexFunction
     // w'<mask> = accum(w',u'*A)
     METHOD (GrB_vxm (w, mask, accum, semiring, u, A, desc)) ;
 
-    if (w != NULL) AxB_method_used = w->AxB_method_used ;
-
     // return w to MATLAB as a struct and free the GraphBLAS w
     pargout [0] = GB_mx_Vector_to_mxArray (&w, "w output", true) ;
 
