@@ -2,23 +2,23 @@
 // GB_mx_mxArray_to_Matrix
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
-// Convert a MATLAB sparse matrix or struct to a GraphBLAS sparse matrix.  The
-// mxArray is either a struct containing two terms: a sparse matrix or vector,
-// and type (a string, "logical", "double", etc), or it is just a plain sparse
-// matrix.  If A.class is present, it is used to typecast the MATLAB matrix
-// into the corresponding type in GraphBLAS.
+// Convert a MATLAB sparse or full matrix, or a struct to a GraphBLAS sparse
+// matrix.  The mxArray is either a struct containing two terms: a sparse or
+// full matrix or vector, and type (a string, "logical", "double", etc), or it
+// is just a plain sparse or full matrix.  If A.class is present, it is used to
+// typecast the MATLAB matrix into the corresponding type in GraphBLAS.
 
 // That is:
 // A = sparse (...) ;   % a sparse double or logical GraphBLAS matrix
 
-// A.matrix = A ; A.class = 'int8' ; Represents  A MATLAB sparse matrix that
-// represents a GraphBLAS int8 matrix.  On input, the MATLAB sparse matrix is
-// typecasted.
+// A.matrix = A ; A.class = 'int8' ; Represents a MATLAB sparse or full matrix
+// that represents a GraphBLAS int8 matrix.  On input, the MATLAB sparse or
+// full matrix is typecasted.
 
 // The MATLAB matrix or struct is not modified.  If deep_copy is true, the
 // GraphBLAS matrix is always a deep copy and can be modified by GraphBLAS.
@@ -34,11 +34,20 @@
 // Like GB_mx_Matrix_to_mxArray, this could be done using only user-callable
 // GraphBLAS functions, but the method used here is faster.
 
+// A.sparsity sets the GxB_SPARSITY_CONTROL option: 0 to 15 (see GB_conform.c),
+// which is any sum of these 4 flags:
+//
+//    // GxB_SPARSITY_CONTROL can be any sum or bitwise OR of these 4 values:
+//    #define GxB_HYPERSPARSE 1   // hypersparse form
+//    #define GxB_SPARSE      2   // sparse form
+//    #define GxB_BITMAP      4   // a bitmap
+//    #define GxB_FULL        8   // full (all entries must be present)
+
 #include "GB_mex.h"
 
 #define FREE_ALL            \
 {                           \
-    GB_MATRIX_FREE (&A) ;   \
+    GrB_Matrix_free_(&A) ;  \
 }
 
 GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
@@ -55,7 +64,7 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     // check for empty matrix
     //--------------------------------------------------------------------------
 
-    GB_WHERE ("mxArray_to_Matrix") ;
+    GB_CONTEXT ("mxArray_to_Matrix") ;
 
     GrB_Matrix A = NULL ;
 
@@ -87,7 +96,6 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     // get the matrix
     //--------------------------------------------------------------------------
 
-
     const mxArray *Amatrix = NULL ;
     GrB_Type atype_in, atype_out ;
     GB_Type_code atype_in_code, atype_out_code ;
@@ -139,12 +147,7 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         atype_out = atype_in ;
     }
 
-    if (!mxIsSparse (Amatrix))
-    {
-        FREE_ALL ;
-        mexWarnMsgIdAndTxt ("GB:warn", "input matrix must be sparse") ;
-        return (NULL) ;
-    }
+    bool A_is_sparse = mxIsSparse (Amatrix) ;
 
     //--------------------------------------------------------------------------
     // get the matrix type
@@ -159,11 +162,24 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
 
     int64_t nrows = mxGetM (Amatrix) ;
     int64_t ncols = mxGetN (Amatrix) ;
-    int64_t *Mp = (int64_t *) mxGetJc (Amatrix) ;
-    int64_t *Mi = (int64_t *) mxGetIr (Amatrix) ;
-    int64_t anz = Mp [ncols] ;
-    GB_void *Mx = mxGetData (Amatrix) ;     // OK:any type
-    int64_t anzmax = mxGetNzmax (Amatrix) ;
+    int64_t *Mp, *Mi, anz, anzmax ;
+
+    if (A_is_sparse)
+    {
+        Mp = (int64_t *) mxGetJc (Amatrix) ;
+        Mi = (int64_t *) mxGetIr (Amatrix) ;
+        anz = Mp [ncols] ;
+        anzmax = mxGetNzmax (Amatrix) ;
+    }
+    else
+    {
+        Mp = NULL ;
+        Mi = NULL ;
+        anz = nrows * ncols ;
+        anzmax = anz ;
+    }
+
+    GB_void *Mx = mxGetData (Amatrix) ;
 
     //--------------------------------------------------------------------------
     // look for A.values
@@ -182,7 +198,7 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
             }
             if (mxGetNumberOfElements (values) >= anz)
             {
-                Mx = mxGetData (values) ;       // OK:any type
+                Mx = mxGetData (values) ;
                 atype_in = GB_mx_Type (values) ;
                 atype_in_code = atype_in->code ;
                 anzmax = mxGetNumberOfElements (values) ;
@@ -202,9 +218,9 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
 
     GrB_Info info ;
 
-    // MATLAB matrices are non-hypersparse CSC
+    // MATLAB matrices are sparse or full CSC, not hypersparse or bitmap
     bool is_csc = true ;
-    bool is_hyper = false ;
+    int sparsity = (A_is_sparse) ? GxB_SPARSE : GxB_FULL ;
 
     //--------------------------------------------------------------------------
     // get the pattern of A
@@ -214,8 +230,9 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     {
 
         // create the GraphBLAS matrix
-        info = GB_new (&A, atype_out, (GrB_Index) nrows, (GrB_Index) ncols,
-            GB_Ap_calloc, is_csc, is_hyper, GB_HYPER_DEFAULT, 0, Context) ;
+        info = GB_new (&A, false, // sparse or full, new mx header
+            atype_out, (GrB_Index) nrows, (GrB_Index) ncols,
+            GB_Ap_calloc, is_csc, sparsity, GxB_HYPER_DEFAULT, 0, Context) ;
         if (info != GrB_SUCCESS)
         {
             FREE_ALL ;
@@ -224,7 +241,8 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         }
 
         // A is a deep copy and can be modified by GraphBLAS
-        info = GB_ix_alloc (A, anz, true, Context) ;
+        info = GB_bix_alloc (A, anz, false, false, sparsity != GxB_FULL, true,
+            Context) ;
         if (info != GrB_SUCCESS)
         {
             FREE_ALL ;
@@ -232,8 +250,11 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
             return (NULL) ;
         }
 
-        memcpy (A->p, Mp, (ncols+1) * sizeof (int64_t)) ;
-        memcpy (A->i, Mi, anz * sizeof (int64_t)) ;
+        if (sparsity != GxB_FULL)
+        {
+            memcpy (A->p, Mp, (ncols+1) * sizeof (int64_t)) ;
+            memcpy (A->i, Mi, anz * sizeof (int64_t)) ;
+        }
         A->magic = GB_MAGIC ;
 
     }
@@ -244,8 +265,9 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         // MATLAB matrix and must not be modified.
 
         // [ create the GraphBLAS matrix, do not allocate A->p
-        info = GB_new (&A, atype_out, (GrB_Index) nrows, (GrB_Index) ncols,
-            GB_Ap_null, is_csc, is_hyper, GB_HYPER_DEFAULT, 0, Context) ;
+        info = GB_new (&A, false, // sparse or full, new mx header
+            atype_out, (GrB_Index) nrows, (GrB_Index) ncols,
+            GB_Ap_null, is_csc, sparsity, GxB_HYPER_DEFAULT, 0, Context) ;
         if (info != GrB_SUCCESS)
         {
             FREE_ALL ;
@@ -253,23 +275,44 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
             return (NULL) ;
         }
 
-        A->p = Mp ;
-        A->magic = GB_MAGIC ;       // A->p now initialized ]
-        A->i = Mi ;
-        A->p_shallow = true ;
+        A->p_size = 0 ;
+        A->i_size = 0 ;
+
+        if (sparsity != GxB_FULL)
+        {
+            A->p = Mp ;
+            A->i = Mi ;
+            A->p_shallow = true ;
+            A->i_shallow = true ;
+        }
+        else
+        {
+            A->p = NULL ;
+            A->i = NULL ;
+            A->p_shallow = false ;
+            A->i_shallow = false ;
+        }
+
         A->h_shallow = false ;      // A->h is NULL
-        A->i_shallow = true ;
+        A->magic = GB_MAGIC ;       // A->p now initialized ]
     }
 
     //--------------------------------------------------------------------------
     // copy the numerical values from MATLAB to the GraphBLAS matrix
     //--------------------------------------------------------------------------
 
-    A->x_shallow = (!deep_copy &&
-           ((atype_out_code == GB_BOOL_code ||
-             atype_out_code == GB_FP64_code ||
-             atype_out_code == GB_FC64_code)
-         && (atype_out_code == atype_in_code))) ;
+    if (sparsity == GxB_FULL)
+    {
+        A->x_shallow = (!deep_copy && (atype_out_code == atype_in_code)) ;
+    }
+    else
+    {
+        A->x_shallow = (!deep_copy &&
+               ((atype_out_code == GB_BOOL_code ||
+                 atype_out_code == GB_FP64_code ||
+                 atype_out_code == GB_FC64_code)
+             && (atype_out_code == atype_in_code))) ;
+    }
 
     if (A->x_shallow)
     {
@@ -278,6 +321,7 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         // make a shallow copy.
         A->nzmax = anzmax ;
         A->x = Mx ;
+        A->x_size = 0 ;     // A->x is shallow
     }
     else
     {
@@ -285,7 +329,8 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         {
             // allocate new space for the GraphBLAS values
             A->nzmax = GB_IMAX (anz, 1) ;
-            A->x = GB_MALLOC (A->nzmax * atype_out->size, GB_void) ;
+            A->x = (GB_void *) GB_malloc_memory (A->nzmax * atype_out->size,
+                sizeof (GB_void), &(A->x_size)) ;
             if (A->x == NULL)
             {
                 FREE_ALL ;
@@ -299,7 +344,8 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
             (atype_out_code == GB_UDT_code) ? GB_FC64_code : atype_out_code,
             Mx,
             (atype_in_code == GB_UDT_code) ? GB_FC64_code : atype_in_code,
-            (atype_in_code == GB_UDT_code) ? sizeof(GxB_FC64_t) : atype_in->size,
+            NULL,
+            (atype_in_code == GB_UDT_code) ? sizeof(GxB_FC64_t) :atype_in->size,
             anz, 1) ;
     }
 
@@ -307,8 +353,11 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     // look for CSR/CSC and hyper/non-hyper format
     //--------------------------------------------------------------------------
 
-    bool has_hyper_ratio = false ;
-    double hyper_ratio = GB_HYPER_DEFAULT ;
+    bool A_is_hyper = false ;
+    bool has_hyper_switch = false ;
+    bool has_sparsity_control = false ;
+    int sparsity_control = GxB_AUTO_SPARSITY ;
+    double hyper_switch = GxB_HYPER_DEFAULT ;
 
     if (mxIsStruct (A_matlab))
     {
@@ -320,20 +369,30 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
                 0, fieldnumber)) ;
         }
 
-        // look for A.is_hyper (ignored if hyper_ratio present)
+        // look for A.is_hyper (ignored if hyper_switch present
+        // or if A is full)
         fieldnumber = mxGetFieldNumber (A_matlab, "is_hyper") ;
         if (fieldnumber >= 0)
         {
-            is_hyper = mxGetScalar (mxGetFieldByNumber (A_matlab,
+            A_is_hyper = mxGetScalar (mxGetFieldByNumber (A_matlab,
                 0, fieldnumber)) ;
         }
 
-        // look for A.hyper_ratio
-        fieldnumber = mxGetFieldNumber (A_matlab, "hyper_ratio") ;
+        // look for A.hyper_switch (ignored if A is full)
+        fieldnumber = mxGetFieldNumber (A_matlab, "hyper_switch") ;
         if (fieldnumber >= 0)
         {
-            has_hyper_ratio = true ;
-            hyper_ratio = mxGetScalar (mxGetFieldByNumber (A_matlab,
+            has_hyper_switch = true ;
+            hyper_switch = mxGetScalar (mxGetFieldByNumber (A_matlab,
+                0, fieldnumber)) ;
+        }
+
+        // look for A.sparsity
+        fieldnumber = mxGetFieldNumber (A_matlab, "sparsity") ;
+        if (fieldnumber >= 0)
+        {
+            has_sparsity_control = true ;
+            sparsity_control = mxGetScalar (mxGetFieldByNumber (A_matlab,
                 0, fieldnumber)) ;
         }
     }
@@ -342,10 +401,13 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     // compute the # of non-empty vectors in A only when needed
     //--------------------------------------------------------------------------
 
-    A->nvec_nonempty = -1 ; // compute when needed; see also GxB_Matrix_import
+    if (sparsity != GxB_FULL)
+    {
+        A->nvec_nonempty = -1 ;
+    }
 
     ASSERT_MATRIX_OK (A, "got natural A from MATLAB", GB0) ;
-    ASSERT (!A->is_hyper) ;
+    ASSERT (A->h == NULL) ;
 
     //--------------------------------------------------------------------------
     // convert to CSR if requested
@@ -359,40 +421,51 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
         // this might convert A to hypersparse
         GxB_Matrix_Option_set_(A, GxB_FORMAT, GxB_BY_ROW) ;
         // so convert it back; hypersparsity is defined below
-        GB_to_nonhyper (A, Context) ;
+        if (sparsity != GxB_FULL)
+        {
+            GB_convert_hyper_to_sparse (A, Context) ;
+        }
         ASSERT (!A->is_csc) ;
     }
 
     ASSERT_MATRIX_OK (A, "conformed from MATLAB", GB0) ;
-    ASSERT (!A->is_hyper) ;
+    ASSERT (A->h == NULL) ;
     ASSERT (A->is_csc == is_csc) ;
 
     //--------------------------------------------------------------------------
     // convert to hypersparse or set hypersparse ratio, if requested
     //--------------------------------------------------------------------------
 
-    if (has_hyper_ratio)
+    if (sparsity == GxB_FULL)
     {
-        // this sets the hyper_ratio and then conforms the matrix to its
-        // desired hypersparsity.  It may stay non-hypersparse.
-        GxB_Matrix_Option_set_(A, GxB_HYPER, hyper_ratio) ;
+        // leave as-is
+        ;
     }
-    else if (is_hyper)
+    else if (has_hyper_switch)
+    {
+        // this sets the hyper_switch and then conforms the matrix to its
+        // desired hypersparsity.  It may stay non-hypersparse.
+        GxB_Matrix_Option_set_(A, GxB_HYPER_SWITCH, hyper_switch) ;
+    }
+    else if (A_is_hyper)
     {
         // this forces the matrix to be always hypersparse
-        GxB_Matrix_Option_set_(A, GxB_HYPER, GxB_ALWAYS_HYPER) ;
-        if (A->vdim > 1)
-        {
-            ASSERT (A->is_hyper == is_hyper) ;
-        }
-        else
-        {
-            // column vectors are never hypersparse
-            ASSERT (!A->is_hyper) ;
-        }
+        ASSERT_MATRIX_OK (A, "to always hyper", GB0) ;
+        GxB_Matrix_Option_set_(A, GxB_SPARSITY_CONTROL, GxB_HYPERSPARSE) ;
+        ASSERT_MATRIX_OK (A, "always hyper", GB0) ;
+    }
+
+    //--------------------------------------------------------------------------
+    // set the sparsity control and conform the matrix
+    //--------------------------------------------------------------------------
+
+    if (has_sparsity_control)
+    {
+        ASSERT_MATRIX_OK (A, "setting sparsity", GB0) ;
+        GxB_Matrix_Option_set_(A, GxB_SPARSITY_CONTROL, sparsity_control) ;
+        ASSERT_MATRIX_OK (A, "set sparsity", GB0) ;
     }
 
-    ASSERT_MATRIX_OK (A, "final hyper/nonhyper", GB0) ;
     ASSERT (A->is_csc == is_csc) ;
     ASSERT (nrows_old == GB_NROWS (A)) ;
     ASSERT (ncols_old == GB_NCOLS (A)) ;
@@ -401,6 +474,14 @@ GrB_Matrix GB_mx_mxArray_to_Matrix     // returns GraphBLAS version of A
     // return the GraphBLAS matrix
     //--------------------------------------------------------------------------
 
+    info = GrB_Matrix_wait (&A) ;
+    if (info != GrB_SUCCESS)
+    {
+        FREE_ALL ;
+        mexWarnMsgIdAndTxt ("GB:warn", "matrix wait failed") ;
+        return (NULL) ;
+    }
+
     ASSERT_MATRIX_OK (A, "got A from MATLAB", GB0) ;
     return (A) ;
 }
