@@ -2,8 +2,8 @@
 // GB_mx_mxArray_to_SelectOp
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -18,8 +18,6 @@ bool GB_mx_mxArray_to_SelectOp          // true if successful
     const char *name                    // name of the argument
 )
 {
-    GB_WHERE ("GB_mx_mxArray_to_SelectOp") ;
-
     (*handle) = NULL ;
     const mxArray *opname_mx = NULL ;
 
@@ -70,6 +68,8 @@ bool GB_mx_mxArray_to_SelectOp          // true if successful
     else if (MATCH (opname, "lt_thunk" )) { op = GxB_LT_THUNK ; }
     else if (MATCH (opname, "le_thunk" )) { op = GxB_LE_THUNK ; }
 
+    else if (MATCH (opname, "isnan"    )) { op = NULL ; }
+
     else
     {
         mexWarnMsgIdAndTxt ("GB:warn", "unknown select op") ;
@@ -77,7 +77,10 @@ bool GB_mx_mxArray_to_SelectOp          // true if successful
     }
 
     // return the op
-    ASSERT_SELECTOP_OK (op, name, GB0) ;
+    if (op != NULL)
+    {
+        ASSERT_SELECTOP_OK (op, name, GB0) ;
+    }
     (*handle) = op ;
     return (true) ;
 }
