@@ -2,8 +2,8 @@
 // GB_mx_put_global: put the GraphBLAS status in MATLAB workspace
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-// http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+// SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
 
@@ -11,8 +11,7 @@
 
 void GB_mx_put_global
 (
-    bool cover,
-    GrB_Desc_Value AxB_method_used
+    bool cover
 )
 {
 
@@ -26,7 +25,7 @@ void GB_mx_put_global
     // return the time to MATLAB, if it was computed
     //--------------------------------------------------------------------------
 
-    GB_mx_put_time (AxB_method_used) ;
+    GB_mx_put_time ( ) ;
 
     //--------------------------------------------------------------------------
     // log the statement coverage
@@ -42,15 +41,48 @@ void GB_mx_put_global
 
     GrB_finalize ( ) ;
 
+    // disable the memory pool, in case a @GrB method is called next
+    int64_t free_pool_limit [64] =
+        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } ;
+    GxB_Global_Option_set (GxB_MEMORY_POOL, free_pool_limit) ;
+
     //--------------------------------------------------------------------------
-    // check nmalloc
+    // check nmemtable and nmalloc
     //--------------------------------------------------------------------------
 
+    int nmemtable = GB_Global_memtable_n ( ) ;
+    if (nmemtable != 0)
+    {
+        printf ("in GB_mx_put_global: GraphBLAS nmemtable %d!\n", nmemtable) ;
+        GB_Global_free_pool_dump (2) ;
+        GB_Global_memtable_dump ( ) ;
+        mexErrMsgTxt ("memory leak in test!") ;
+    }
+
+    int64_t nblocks = GB_Global_free_pool_nblocks_total ( ) ;
+    if (nblocks != 0)
+    {
+        printf ("in GB_mx_put_global: GraphBLAS nblocks "GBd" in free_pool!\n",
+            nblocks) ;
+        GB_Global_free_pool_dump (2) ;
+        GB_Global_memtable_dump ( ) ;
+        mexErrMsgTxt ("memory leak in test!") ;
+    }
+
     int64_t nmalloc = GB_Global_nmalloc_get ( ) ;
     if (nmalloc != 0)
     {
-        printf ("GraphBLAS nmalloc "GBd"!\n", nmalloc) ;
-        mexErrMsgTxt ("memory leak!") ;
+        printf ("in GB_mx_put_global: GraphBLAS nmalloc "GBd"!\n", nmalloc) ;
+        GB_Global_free_pool_dump (2) ;
+        GB_Global_memtable_dump ( ) ;
+        mexErrMsgTxt ("memory leak in test!") ;
     }
+
+    //--------------------------------------------------------------------------
+    // allow GrB_init to be called again
+    //--------------------------------------------------------------------------
+
+    GB_Global_GrB_init_called_set (false) ;
 }
 
