@@ -4,8 +4,8 @@ function C = GB_spec_apply (C, Mask, accum, op, A, descriptor)
 % Usage:
 % C = GB_spec_apply (C, Mask, accum, op, A, descriptor)
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 %-------------------------------------------------------------------------------
 % get inputs
@@ -36,13 +36,22 @@ end
 T.matrix = GB_spec_zeros (size (A.matrix), ztype) ;
 T.pattern = A.pattern ;
 T.class = ztype ;
-
 p = T.pattern ;
-x = A.matrix (p) ;
 
-z = GB_spec_op (op, x) ;
-
-T.matrix (p) = z ;
+if (GB_spec_is_positional (opname))
+    [m, n] = size (A.matrix) ;
+    for i = 1:m
+        for j = 1:n
+            if (p (i,j))
+                T.matrix (i,j) = GB_spec_unop_positional (opname, i, j) ;
+            end
+        end
+    end
+else
+    x = A.matrix (p) ;
+    z = GB_spec_op (op, x) ;
+    T.matrix (p) = z ;
+end
 
 % C<Mask> = accum (C,T): apply the accum, then Mask, and return the result
 C = GB_spec_accum_mask (C, Mask, accum, T, C_replace, Mask_comp, 0) ;
