@@ -1,7 +1,7 @@
-function A = GB_spec_random (m, n, d, scale, type, is_csc,is_hyper,hyper_ratio)
+function A = GB_spec_random (m, n, d, scale, type, is_csc,is_hyper,hyper_switch)
 %GB_SPEC_RANDOM generate random matrix
 %
-% A = GB_spec_random (m, n, d, scale, type, is_csc, is_hyper, hyper_ratio)
+% A = GB_spec_random (m, n, d, scale, type, is_csc, is_hyper, hyper_switch)
 %
 % m,n,d: parameters to sprandn (m,n,d)
 % m,n: defaults to 4
@@ -11,8 +11,8 @@ function A = GB_spec_random (m, n, d, scale, type, is_csc,is_hyper,hyper_ratio)
 % is_csc: true for CSC, false for CSR; defaults to true
 % is_hyper: false for non-hypersparse, true for hypersparse, default false
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 if (nargin < 1)
     m = 4 ;
@@ -43,7 +43,7 @@ if (nargin >= 7 && ~isempty (is_hyper))
 end
 
 if (nargin >= 8)
-    A.hyper_ratio = hyper_ratio ;
+    A.hyper_switch = hyper_switch ;
 end
 
 if (isinf (d))
