@@ -3,8 +3,8 @@ function [nthreads chunk] = nthreads_get
 %
 % [nthreads chunk] = nthreads_get
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 global GraphBLAS_nthreads
 if (isempty (GraphBLAS_nthreads))
@@ -15,7 +15,7 @@ nthreads = GraphBLAS_nthreads ;
 if (nargout > 1)
     global GraphBLAS_chunk
     if (isempty (GraphBLAS_chunk))
-        [nthreads chunk] = nthreads_set (nthreads, 4096) ;
+        [nthreads chunk] = nthreads_set (nthreads, 64*1024) ;
     end
     chunk = GraphBLAS_chunk ;
 end
