@@ -1,11 +1,8 @@
 function test128
 %TEST128 test eWiseMult and eWiseAdd, special cases
 
-% C = GB_mex_eWiseMult_Matrix (C, Mask, accum, mult, A, B, desc)
-% C = GB_mex_eWiseAdd_Matrix  (C, Mask, accum, add,  A, B, desc, test)
-
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 fprintf ('\ntest128: test eWiseMult and eWiseAdd, special cases\n') ;
 rng ('default') ;
@@ -55,53 +52,53 @@ for B_hyper = 0:1
             M.is_hyper = M_hyper ;
 
             C0 = Amat .* Bmat .* Mmat ;
-            C1 = GB_spec_eWiseMult_Matrix (S, M, [ ], 'times', A, B, [ ]) ;
-            C2 = GB_mex_eWiseMult_Matrix  (S, M, [ ], 'times', A, B, [ ]) ;
-            C3 = GB_mex_eWiseMult_Matrix  (S, M, [ ], 'times', B, A, [ ]) ;
+            C1 = GB_spec_Matrix_eWiseMult (S, M, [ ], 'times', A, B, [ ]) ;
+            C2 = GB_mex_Matrix_eWiseMult  (S, M, [ ], 'times', A, B, [ ]) ;
+            C3 = GB_mex_Matrix_eWiseMult  (S, M, [ ], 'times', B, A, [ ]) ;
             GB_spec_compare (C1, C2) ;
             GB_spec_compare (C1, C3) ;
             assert (isequal (C0, C2.matrix)) ;
 
             C0 = (Amat + Bmat) .* Mmat ;
-            C1 = GB_spec_eWiseAdd_Matrix (S, M, [ ], 'plus', A, B, [ ]) ;
-            C2 = GB_mex_eWiseAdd_Matrix  (S, M, [ ], 'plus', A, B, [ ]) ;
-            C3 = GB_mex_eWiseAdd_Matrix  (S, M, [ ], 'plus', B, A, [ ]) ;
+            C1 = GB_spec_Matrix_eWiseAdd (S, M, [ ], 'plus', A, B, [ ]) ;
+            C2 = GB_mex_Matrix_eWiseAdd  (S, M, [ ], 'plus', A, B, [ ]) ;
+            C3 = GB_mex_Matrix_eWiseAdd  (S, M, [ ], 'plus', B, A, [ ]) ;
             GB_spec_compare (C1, C2) ;
             GB_spec_compare (C1, C3) ;
             assert (isequal (C0, C2.matrix)) ;
 
-            C1 = GB_spec_eWiseMult_Matrix (X, M, [ ], 'times', A, B, [ ]) ;
-            C2 = GB_mex_eWiseMult_Matrix  (X, M, [ ], 'times', A, B, [ ]) ;
+            C1 = GB_spec_Matrix_eWiseMult (X, M, [ ], 'times', A, B, [ ]) ;
+            C2 = GB_mex_Matrix_eWiseMult  (X, M, [ ], 'times', A, B, [ ]) ;
             GB_spec_compare (C1, C2) ;
 
-            C1 = GB_spec_eWiseAdd_Matrix (X, M, [ ], 'plus', A, B, [ ]) ;
-            C2 = GB_mex_eWiseAdd_Matrix  (X, M, [ ], 'plus', A, B, [ ]) ;
+            C1 = GB_spec_Matrix_eWiseAdd (X, M, [ ], 'plus', A, B, [ ]) ;
+            C2 = GB_mex_Matrix_eWiseAdd  (X, M, [ ], 'plus', A, B, [ ]) ;
             GB_spec_compare (C1, C2) ;
 
         end
 
         C0 = Amat .* Bmat ;
-        C1 = GB_spec_eWiseMult_Matrix (S, [ ], [ ], 'times', A, B, [ ]) ;
-        C2 = GB_mex_eWiseMult_Matrix  (S, [ ], [ ], 'times', A, B, [ ]) ;
-        C3 = GB_mex_eWiseMult_Matrix  (S, [ ], [ ], 'times', B, A, [ ]) ;
+        C1 = GB_spec_Matrix_eWiseMult (S, [ ], [ ], 'times', A, B, [ ]) ;
+        C2 = GB_mex_Matrix_eWiseMult  (S, [ ], [ ], 'times', A, B, [ ]) ;
+        C3 = GB_mex_Matrix_eWiseMult  (S, [ ], [ ], 'times', B, A, [ ]) ;
         GB_spec_compare (C1, C2) ;
         GB_spec_compare (C1, C3) ;
         assert (isequal (C0, C2.matrix)) ;
 
         C0 = Amat + Bmat ;
-        C1 = GB_spec_eWiseAdd_Matrix (S, [ ], [ ], 'plus', A, B, [ ]) ;
-        C2 = GB_mex_eWiseAdd_Matrix  (S, [ ], [ ], 'plus', A, B, [ ]) ;
-        C3 = GB_mex_eWiseAdd_Matrix  (S, [ ], [ ], 'plus', B, A, [ ]) ;
+        C1 = GB_spec_Matrix_eWiseAdd (S, [ ], [ ], 'plus', A, B, [ ]) ;
+        C2 = GB_mex_Matrix_eWiseAdd  (S, [ ], [ ], 'plus', A, B, [ ]) ;
+        C3 = GB_mex_Matrix_eWiseAdd  (S, [ ], [ ], 'plus', B, A, [ ]) ;
         GB_spec_compare (C1, C2) ;
         GB_spec_compare (C1, C3) ;
         assert (isequal (C0, C2.matrix)) ;
 
-        C1 = GB_spec_eWiseMult_Matrix (X, [ ], [ ], 'times', A, B, [ ]) ;
-        C2 = GB_mex_eWiseMult_Matrix  (X, [ ], [ ], 'times', A, B, [ ]) ;
+        C1 = GB_spec_Matrix_eWiseMult (X, [ ], [ ], 'times', A, B, [ ]) ;
+        C2 = GB_mex_Matrix_eWiseMult  (X, [ ], [ ], 'times', A, B, [ ]) ;
         GB_spec_compare (C1, C2) ;
 
-        C1 = GB_spec_eWiseAdd_Matrix (X, [ ], [ ], 'plus', A, B, [ ]) ;
-        C2 = GB_mex_eWiseAdd_Matrix  (X, [ ], [ ], 'plus', A, B, [ ]) ;
+        C1 = GB_spec_Matrix_eWiseAdd (X, [ ], [ ], 'plus', A, B, [ ]) ;
+        C2 = GB_mex_Matrix_eWiseAdd  (X, [ ], [ ], 'plus', A, B, [ ]) ;
         GB_spec_compare (C1, C2) ;
 
     end
