@@ -1,13 +1,13 @@
 function test155
 %TEST155 test GrB_*_setElement and GrB_*_removeElement
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 rng ('default') ;
-mlist  = [ 1  1  10  20 ] ;
-nlist  = [ 1 10   1  10 ] ;
-nzlist = [ 5 100 100 1000 ] ;
+mlist  = [ 1  1  10  20    5  ] ;
+nlist  = [ 1 10   1  10    5  ]  ;
+nzlist = [ 5 100 100 1000  100] ;
 
 for trial = 1:4
 
@@ -20,6 +20,10 @@ for trial = 1:4
     X = rand (nz, 1) ;
     Action = double (rand (nz, 1) > 0.4) ;
 
+    %---------------------------------------------------------------------------
+    % starting with an empty matrix:
+    %---------------------------------------------------------------------------
+
     % do the work in MATLAB
     C1 = sparse (m, n) ;
     for k = 1:nz
@@ -45,6 +49,30 @@ for trial = 1:4
             assert (isequal (C1, C2)) ;
         end
     end
+
+    %---------------------------------------------------------------------------
+    % starting with a full matrix:
+    %---------------------------------------------------------------------------
+
+    % do the work in MATLAB
+    C1 = rand (m, n) ;
+    C1_start = C1 ;
+    for k = 1:nz
+        if (Action (k) == 0)
+            C1 (I (k), J (k)) = sparse (0) ;
+        else
+            C1 (I (k), J (k)) = sparse (X (k)) ;
+        end
+    end
+
+    % do the work in GraphBLAS, testing all sparsity control options
+    C0.matrix = C1_start ;
+    for sparsity_control = 1:15
+        C0.sparsity = sparsity_control ;
+        C2 = GB_mex_edit (C0, I, J, X, Action) ;
+        assert (isequal (C1, C2)) ;
+    end
+
 end
 
 fprintf ('test155: all tests passed\n') ;
