@@ -1,8 +1,8 @@
 function test50
 %TEST50 test AxB numeric and symbolic
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 fprintf ('\n----------------------------- GB_mex_AxB\n') ;
 
@@ -44,7 +44,7 @@ for trial = 1:2
     tic
     S = GB_mex_AxB (A, B) ;
     toc
-    assert (spok (S*1) == 1) ;
+    assert (GB_spok (S*1) == 1) ;
     err = norm (C-S,1) / cnorm ;
     fprintf ('err %g\n', err) ;
     assert (isequal (C, S)) ;
