@@ -1,13 +1,12 @@
 function test90
 %TEST90 test AxB with user-defined semirings: plus_rdiv and plus_rdiv2
 
-% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2020, All Rights Reserved.
-% http://suitesparse.com   See GraphBLAS/Doc/License.txt for license.
+% SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2021, All Rights Reserved.
+% SPDX-License-Identifier: Apache-2.0
 
 fprintf ('\n -------------- A*B plus_rdiv (user-defined semiring)\n') ;
 
     % 1001: Gustavson
-    % 1002: heap
     % 1003: dot
     % 1004: hash
     % 1005: saxpy
@@ -16,8 +15,6 @@ rng ('default') ;
 
 for N = [10 100] % 1000]
 
-    N
-
     % create the problem
     A = sprand (4*N, 5*N, 0.01) ;
     B = sprand (5*N, 3*N, 0.01) ;
@@ -39,7 +36,7 @@ for N = [10 100] % 1000]
     t = grbresults ; fprintf ('GB time %g\n', t) ;
     assert (norm (C1-C2,1) / norm (C1,1) < 1e-10) ;
 
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method: %d\n', method) ;
         cprint = (N <= 10) ;
         tic
@@ -89,7 +86,7 @@ for N = [10 100] % 1000]
     tic
     C0 = X*B ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -105,7 +102,7 @@ for N = [10 100] % 1000]
     tic
     C0 = X'*B ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -121,7 +118,7 @@ for N = [10 100] % 1000]
     tic
     C0 = X*B' ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -137,7 +134,7 @@ for N = [10 100] % 1000]
     tic
     C0 = X'*B' ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -153,7 +150,7 @@ for N = [10 100] % 1000]
     tic
     C0 = A*Y ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -169,7 +166,7 @@ for N = [10 100] % 1000]
     tic
     C0 = A'*Y ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -185,7 +182,7 @@ for N = [10 100] % 1000]
     tic
     C0 = A*Y' ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
@@ -201,7 +198,7 @@ for N = [10 100] % 1000]
     tic
     C0 = A'*Y' ;
     toc
-    for method = 1001:1005
+    for method = [1001 1003 1004 1005]
         fprintf ('method %d\n', method) ;
         tic
         C5 = GB_mex_rdiv2 (A, B, at, bt, method, flipxy) ;
